# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# INSTITUTO TECNOLOGICO DE COSTA RICA
# Escuela de Ingenieria en Construccion
# Construction Engineering School
# https://www.tec.ac.cr
# Eng. MSc. Maikel Mendez Morales
# Email: maikel.mendez@gmail.com; mamendez@itcr.ac.cr
# https://orcid.org/0000-0003-1919-141X
# https://www.scopus.com/authid/detail.uri?authorId=51665581300
# https://scholar.google.com/citations?user=JnmSVFYAAAAJ&hl=en
# https://www.youtube.com/c/maikelmendez
# https://twitter.com/MaikelMendezM
# https://github.com/maikelonu
# Skype: maikel.mendez
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

#-------------------------------------------------------------------------------------------------------------------
# INFO: This script is intended for the generation of various figures including heatmaps, 
# Taylor diagrams, time-series and Q-Q plots
#-------------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------------
# MANUSCRIPT TITLE:
# "Performance Evaluation of Bias Correction Methods for Monthly Precipitation 
#  Projections over Costa Rica"
#-------------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------------
# INPUT FILES:

# Binary R format: 
# None

# TXT-ASCII format:
# ECDF_compiled_BC_Taylor_version_04.txt: Compiled raw, observed, modelled and bias-corrected
# monthly precipitation per month and Climatic Region
#-------------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------------
# MANUSCRIPT FIGURES:
# Figures_24,5,6,7,8,9 and 10
#-------------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------------
# OUTPUT FILES:
# Figures_2,4,5,6,7,8,9 and 10
#-------------------------------------------------------------------------------------------------------------------

#-------------------------------------------------------------------------------------------------------------------
# Workspace is cleared
#rm(list = ls())

# CRAN libraries are loaded
require(raster)
require(automap)
require(car)
require(chron)
require(compare)
require(DescTools)
require(doParallel)
require(dplyr)
require(forecast)
require(ggmap)
require(ggplot2)
require(ggrepel)
require(gstat)
require(hydroGOF)
require(hyfo)
require(intamap)
require(KScorrect)
require(lattice)
require(lattice)
require(maps)
require(maptools)
require(maptools)
require(MASS)
require(mefa)
require(ModelMetrics)
require(ncdf4)
require(ncdf4.helpers)
require(openair)
require(parallel)
require(pastecs)
require(PCICt)
require(plyr)
require(RColorBrewer)
require(rcompanion)
require(readxl)
require(reshape)
require(reshape2)
require(rgdal)
require(rgeos)
require(scales)
require(snow)
require(sp)
require(spatialEco)
require(tibble)
require(utf8)

# Working directory is defined
setwd("/mnt/BACKUP/R_ITC/XLSX")  # UBUNTU-LINUX
setwd("C:/DATOS/R_ITC/XLSX")     # MS-WINDOWS

# Scientific notation is disableb
options(scipen=999)

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Bias Correction Methods Post-Processing
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

# Input data is loaded and a data.frame is created
# This *.TXT file was compiled using XXXXXXXXXXXXXXXXXXXX.R
df.taylor.01 <- read.table("ECDF_compiled_BC_Taylor_version_CORDEX_font.txt", header = TRUE)
df.taylor.02 <- read.table("ECDF_compiled_BC_Taylor_version_CORDEX_font.txt", header = TRUE)
df.taylor.03 <- read.table("ECDF_compiled_BC_Taylor_version_CORDEX_font.txt", header = TRUE)

# NA values are removed
df.taylor.03$Model_value <- ifelse(is.nan(df.taylor.03$Model_value), df.taylor.02$Model_value, df.taylor.03$Model_value)
df.taylor.03$Model_value <- ifelse(is.na(df.taylor.03$Model_value), df.taylor.02$Model_value, df.taylor.03$Model_value)

# A "levels" monthly vector is created for visualization puposes
v.select.name <- c("JAN", "FEB", "MAR","APR", "MAY", "JUN", "JUL","AUG", "SEP", "OCT", "NOV","DEC")

# A "levels" monthly vector is created for visualization puposes
v.select.name.02 <- c("DJF", "MAM", "JJA","SON")

# A "Regions" monthly vector is created for visualization puposes
v.select.regioncr <- c("North_Pacific", "Central_Pacific", "South_Pacific", "Central_Valley", "North", "Caribbean")

# A "BC" monthly vector is created for visualization puposes
v.select.bcm <- c("GPQM","GQM","EQM","PTR","DT","LS","UC")

# Months are ordered to avoid ggplot plotting conflicts
df.taylor.01$Month.ordered <- ordered(df.taylor.01$Month, v.select.name)

# data.frame variable is transformed to factor class
df.taylor.01$Region <- as.factor(df.taylor.01$Region)

# A dummy character variable is created
df.taylor.01$quarter <- c("test")

# Quarter sorting loop is initiated
for (i in 1 : length(df.taylor.01$Month.ordered)) {
  
  # Precipitation values are selected [mm/month] (according to month of the year and date)
  if (df.taylor.01$Month.ordered[i] == "JAN") { df.taylor.01$quarter[i] <- c("DJF") } else if (df.taylor.01$Month.ordered[i] == "FEB") {
    df.taylor.01$quarter[i] <- c("DJF") } else if (df.taylor.01$Month.ordered[i] == "MAR") {
      df.taylor.01$quarter[i] <- c("MAM") } else if (df.taylor.01$Month.ordered[i] == "APR") {
        df.taylor.01$quarter[i] <- c("MAM") } else if (df.taylor.01$Month.ordered[i] == "MAY") {
          df.taylor.01$quarter[i] <- c("MAM") } else if (df.taylor.01$Month.ordered[i] == "JUN") {
            df.taylor.01$quarter[i] <- c("JJA") } else if (df.taylor.01$Month.ordered[i] == "JUL") {
              df.taylor.01$quarter[i] <- c("JJA") } else if (df.taylor.01$Month.ordered[i] == "AUG") {
                df.taylor.01$quarter[i] <- c("JJA") } else if (df.taylor.01$Month.ordered[i] == "SEP") {
                  df.taylor.01$quarter[i] <- c("SON") } else if (df.taylor.01$Month.ordered[i] == "OCT") {
                    df.taylor.01$quarter[i] <- c("SON") } else if (df.taylor.01$Month.ordered[i] == "NOV") {
                      df.taylor.01$quarter[i] <- c("SON") } else if (df.taylor.01$Month.ordered[i] == "DEC") {
                        df.taylor.01$quarter[i] <- c("DJF") }
  
} # Quarter sorting loop is closed

# data.frame variable is transformed to factor class
df.taylor.01$quarter <- as.factor(df.taylor.01$quarter)

# Months are ordered to avoid ggplot plotting conflicts
df.taylor.01$quarter <- ordered(df.taylor.01$quarter, v.select.name.02)

# A new scale07 color is defined
scale07 <- c("#E41A1C", "#E41A1B", "#E41A1A", "#E41A19", "#E41919", "#E41819", "#E41719", # red 228,26,28, CanESM2
             "#1B9E77", "#1B9E76", "#1C9F77", "#1B9E75", "#1B9D75", "#1B9C75", "#1B9B75", # green pastel 27,158,119, EARTH
             "#66A61E", "#66A61D", "#66A61C", "#66A61B", "#66A51B", "#66A41B", "#66A31B", # olive green 102,166,30, ENSEMBLE
             "#984EA3", "#984EA2", "#984EA1", "#984EA0", "#984DA0", "#984CA0", "#984BA0", # violet 152,78,163, GPCC
             "#A6761D", "#A6761E", "#A6771E", "#A7771E", "#A7771F", "#A7781F", "#A7791F", # brown 166,118,29, HadGEM2
             "#010101", "#010102", "#010103", "#010104", "#010204", "#010304", "#010404", # black 1,1,1, MIROC
             "#FF7F00", "#FF7F01", "#FF7F02", "#FF7F03", "#FF7E03", "#FF7E02", "#FF7D02", # orange 255,127,0, MPI
             "#377EB8", "#377EB7", "#377EB6", "#377EB5", "#377DB5", "#377DB6", "#377CB7") # blue 55,126,184, PRECIS

# A new scale08 color is defined
scale08 <- c("#E41A1C", "#1B9E77", "#66A61E", "#984EA3", "#377EB8", "#010101", "#FF7F00")

# A new scale08 color is defined
scale09 <- c("#E41A1C", "#1B9E77", "#66A61E", "#984EA3", "#377EB8", "#4d4d4d", "#FF7F00")

# A new scale08 color is defined
scale10 <- c("#984EA3","#66A61E","#377EB8","#E41A1C","#1B9E77","#A6761D","#010101","#FF7F00")

# A new scale08 color is defined
scale11 <- c("#984EA3","#66A61E","#377EB8","#E41A1C","#1B9E77","#A6761D","#010101","#FF7F00", "black")

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Taylor-Diagrams generation for each Climatic Region
# Generates: Figure 5. Seasonal Taylor diagrams of precipitation bias correction methods for GCM-RCMs during 
# the validation period 1981-1995 for all climatic region of Costa Rica.
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

#----------------------------------------------------------------------------------------------------
# SUBBLOCK: North_Pacific Climate Region is isolated
#----------------------------------------------------------------------------------------------------

# Climate Region is selected
df.taylor.subset.North_Pacific <- subset(df.taylor.01, Region == "North_Pacific")

# summarise {dplyr} is used to reduce multiple values down to a single value
df.summ.sta.North_Pacific <- df.taylor.subset.North_Pacific %>%
  group_by(Model,BC_method, quarter) %>%
  dplyr::summarise(
    RMSE = ModelMetrics::rmse(Obs_value, Model_value),
    MAE = ModelMetrics::mae(Obs_value, Model_value)^1,
    R = cor(Obs_value, Model_value)^1,
    BIAS = ((sum(Model_value) - sum(Obs_value))/sum(Obs_value))*100,
    VBIAS = hydroGOF::NSE(Model_value, Obs_value, na.rm=TRUE) # Positive values indicate overestimation bias, whereas negative values indicate model underestimation bias
  )

# data.frame values are requested
df.summ.sta.North_Pacific

# Raw observational values are isolated and repeated
test1 <- mefa:::rep.data.frame(df.summ.sta.North_Pacific[c(25,26,27,28), c(4,5,6,7,8)], times = 7)
test2 <- mefa:::rep.data.frame(df.summ.sta.North_Pacific[c(53,54,55,56), c(4,5,6,7,8)], times = 7)
test3 <- mefa:::rep.data.frame(df.summ.sta.North_Pacific[c(81,82,83,84), c(4,5,6,7,8)], times = 7)
test4 <- mefa:::rep.data.frame(df.summ.sta.North_Pacific[c(109,110,111,112), c(4,5,6,7,8)], times = 7)
test5 <- mefa:::rep.data.frame(df.summ.sta.North_Pacific[c(137,138,139,140), c(4,5,6,7,8)], times = 7)
test6 <- mefa:::rep.data.frame(df.summ.sta.North_Pacific[c(165,166,167,168), c(4,5,6,7,8)], times = 7)
test7 <- mefa:::rep.data.frame(df.summ.sta.North_Pacific[c(193,194,195,196), c(4,5,6,7,8)], times = 7)
test8 <- mefa:::rep.data.frame(df.summ.sta.North_Pacific[c(221,222,223,224), c(4,5,6,7,8)], times = 7)

# A data.frame is creted
test8 <- rbind(test1,test2,test3,test4,test5,test6,test7,test8)

# RMSE, MAE and R relative variables are renamed
names(test8) <- c("xRMSE", "xMAE",  "xR", "xBIAS", "VBIAS")

# RMSE, MAE and R relative variables are incorporated in the original data.frame
df.summ.sta.North_Pacific$xRMSE <- test8$xRMSE
df.summ.sta.North_Pacific$xMAE <- test8$xMAE
df.summ.sta.North_Pacific$xR <- test8$xR
df.summ.sta.North_Pacific$xBIAS <- test8$xBIAS

# RMSE, MAE and R relative variables are created
df.summ.sta.North_Pacific$dRMSE <- (1-(df.summ.sta.North_Pacific$RMSE/df.summ.sta.North_Pacific$xRMSE))*100
df.summ.sta.North_Pacific$dMAE <- (1-(df.summ.sta.North_Pacific$MAE/df.summ.sta.North_Pacific$xMAE))*100
df.summ.sta.North_Pacific$dR <- ((df.summ.sta.North_Pacific$R - df.summ.sta.North_Pacific$xR)/df.summ.sta.North_Pacific$R)*100
df.summ.sta.North_Pacific$dBIAS <- abs(df.summ.sta.North_Pacific$BIAS) - abs(df.summ.sta.North_Pacific$xBIAS)

# Raw observational values are deletd from the original data.frame
#df.summ.sta.North_Pacific <- df.summ.sta.North_Pacific[-c(21,22,23,24,53,54,55,56,81,82,83,84,109,110,111,112,137,138,139,140,165,166,167,168,193,194,195,196), ]

# A region variable is added
df.summ.sta.North_Pacific$Region <- c("North_Pacific")

# RMSE, MAE and R relative variables are rounded to one decimal
df.summ.sta.North_Pacific$dRMSE <- round(df.summ.sta.North_Pacific$dRMSE, 1)
df.summ.sta.North_Pacific$dMAE <- round(df.summ.sta.North_Pacific$dMAE, 1)
df.summ.sta.North_Pacific$dR <- round(df.summ.sta.North_Pacific$dR, 1)
df.summ.sta.North_Pacific$dBIAS <- round(df.summ.sta.North_Pacific$dBIAS, 1)

# A delta variable is created between modelled and observed values 
df.taylor.subset.North_Pacific$delta <- df.taylor.subset.North_Pacific$Model_value/df.taylor.subset.North_Pacific$Obs_value

# A threshold is defined for visualization purposes only
df.taylor.subset.North_Pacific$Model_value <- ifelse(df.taylor.subset.North_Pacific$Model_value > 700, 700, df.taylor.subset.North_Pacific$Model_value)

# A Taylor quarter diagram is created
TaylorDiagram(df.taylor.subset.North_Pacific,
              obs = "Obs_value",
              mod = "Model_value",
              group = c("UNION"),
              type = c("quarter","Region"),
              normalise = FALSE,
              annotate = "RMSE",
              cols = scale07,
              fontsize= 18, # 28
              pch = c(15,16,17,10,13,8,11), 
              key = FALSE,
              cex = 0.80)# size 0.80

# A Taylor annual diagram is created
TaylorDiagram(df.taylor.subset.North_Pacific,
              obs = "Obs_value",
              mod = "Model_value",
              group = c("UNION"),
              type = c("Region"),
              normalise = FALSE,
              annotate = "RMSE",
              cols = scale07,
              fontsize= 22, # 28
              pch = c(15,16,17,10,13,8,11), 
              #key = FALSE,
              cex=0.95) # size 0.95

#----------------------------------------------------------------------------------------------------
# SUBBLOCK: Central_Pacific Climate Region is isolated
#----------------------------------------------------------------------------------------------------

# Climate Region is selected
df.taylor.subset.Central_Pacific <- subset(df.taylor.01, Region == "Central_Pacific")

# summarise {dplyr} is used to reduce multiple values down to a single value
df.summ.sta.Central_Pacific <- df.taylor.subset.Central_Pacific %>%
  group_by(Model,BC_method, quarter) %>%
  dplyr::summarise(
    RMSE = ModelMetrics::rmse(Obs_value, Model_value),
    MAE = ModelMetrics::mae(Obs_value, Model_value)^1,
    R = cor(Obs_value, Model_value)^1,
    BIAS = ((sum(Model_value) - sum(Obs_value))/sum(Obs_value))*100,
    VBIAS = hydroGOF::NSE(Model_value, Obs_value, na.rm=TRUE) # Positive values indicate overestimation bias, whereas negative values indicate model underestimation bias
  )

# data.frame values are requested
df.summ.sta.Central_Pacific

# Raw observational values are isolated and repeated
test1 <- mefa:::rep.data.frame(df.summ.sta.Central_Pacific[c(25,26,27,28), c(4,5,6,7,8)], times = 7)
test2 <- mefa:::rep.data.frame(df.summ.sta.Central_Pacific[c(53,54,55,56), c(4,5,6,7,8)], times = 7)
test3 <- mefa:::rep.data.frame(df.summ.sta.Central_Pacific[c(81,82,83,84), c(4,5,6,7,8)], times = 7)
test4 <- mefa:::rep.data.frame(df.summ.sta.Central_Pacific[c(109,110,111,112), c(4,5,6,7,8)], times = 7)
test5 <- mefa:::rep.data.frame(df.summ.sta.Central_Pacific[c(137,138,139,140), c(4,5,6,7,8)], times = 7)
test6 <- mefa:::rep.data.frame(df.summ.sta.Central_Pacific[c(165,166,167,168), c(4,5,6,7,8)], times = 7)
test7 <- mefa:::rep.data.frame(df.summ.sta.Central_Pacific[c(193,194,195,196), c(4,5,6,7,8)], times = 7)
test8 <- mefa:::rep.data.frame(df.summ.sta.Central_Pacific[c(221,222,223,224), c(4,5,6,7,8)], times = 7)

# A data.frame is creted
test8 <- rbind(test1,test2,test3,test4,test5,test6,test7,test8)

# RMSE, MAE and R relative variables are renamed
names(test8) <- c("xRMSE", "xMAE",  "xR", "xBIAS", "VBIAS")

# RMSE, MAE and R relative variables are incorporated in the original data.frame
df.summ.sta.Central_Pacific$xRMSE <- test8$xRMSE
df.summ.sta.Central_Pacific$xMAE <- test8$xMAE
df.summ.sta.Central_Pacific$xR <- test8$xR
df.summ.sta.Central_Pacific$xBIAS <- test8$xBIAS
df.summ.sta.Central_Pacific$VBIAS <- round(test8$VBIAS, 1)

# RMSE, MAE and R relative variables are created
df.summ.sta.Central_Pacific$dRMSE <- (1-(df.summ.sta.Central_Pacific$RMSE/df.summ.sta.Central_Pacific$xRMSE))*100
df.summ.sta.Central_Pacific$dMAE <- (1-(df.summ.sta.Central_Pacific$MAE/df.summ.sta.Central_Pacific$xMAE))*100
df.summ.sta.Central_Pacific$dR <- ((df.summ.sta.Central_Pacific$R - df.summ.sta.Central_Pacific$xR)/df.summ.sta.Central_Pacific$R)*100
df.summ.sta.Central_Pacific$dBIAS <- abs(df.summ.sta.Central_Pacific$BIAS) - abs(df.summ.sta.Central_Pacific$xBIAS)

# Raw observational values are deletd from the original data.frame
#df.summ.sta.Central_Pacific <- df.summ.sta.Central_Pacific[-c(21,22,23,24,53,54,55,56,81,82,83,84,109,110,111,112,137,138,139,140,165,166,167,168,193,194,195,196), ]

# A region variable is added
df.summ.sta.Central_Pacific$Region <- c("Central_Pacific")

# RMSE, MAE and R relative variables are rounded to one decimal
df.summ.sta.Central_Pacific$dRMSE <- round(df.summ.sta.Central_Pacific$dRMSE, 1)
df.summ.sta.Central_Pacific$dMAE <- round(df.summ.sta.Central_Pacific$dMAE, 1)
df.summ.sta.Central_Pacific$dR <- round(df.summ.sta.Central_Pacific$dR, 1)
df.summ.sta.Central_Pacific$dBIAS <- round(df.summ.sta.Central_Pacific$dBIAS, 1)

# A delta variable is created between modelled and observed values 
df.taylor.subset.Central_Pacific$delta <- df.taylor.subset.Central_Pacific$Model_value/df.taylor.subset.Central_Pacific$Obs_value

# A threshold is defined for visualization purposes only
df.taylor.subset.Central_Pacific$Model_value <- ifelse(df.taylor.subset.Central_Pacific$Model_value > 700, 700, df.taylor.subset.Central_Pacific$Model_value)

# A Taylor quarter diagram is created
TaylorDiagram(df.taylor.subset.Central_Pacific,
              obs = "Obs_value",
              mod = "Model_value",
              group = c("UNION"),
              type = c("quarter","Region"),
              normalise = FALSE,
              annotate = "RMSE",
              cols = scale07,
              fontsize= 18, # 28
              pch = c(15,16,17,10,13,8,11), 
              key = FALSE,
              cex = 0.80)# size 0.80

# A Taylor annual diagram is created
TaylorDiagram(df.taylor.subset.Central_Pacific,
              obs = "Obs_value",
              mod = "Model_value",
              group = c("UNION"),
              type = c("Region"),
              normalise = FALSE,
              annotate = "RMSE",
              cols = scale07,
              fontsize= 22, # 28
              pch = c(15,16,17,10,13,8,11), 
              #key = FALSE,
              cex=0.95) # size 0.95

#----------------------------------------------------------------------------------------------------
# SUBBLOCK: South_Pacific Climate Region is isolated
#----------------------------------------------------------------------------------------------------

# Climate Region is selected
df.taylor.subset.South_Pacific <- subset(df.taylor.01, Region == "South_Pacific")

# summarise {dplyr} is used to reduce multiple values down to a single value
df.summ.sta.South_Pacific <- df.taylor.subset.South_Pacific %>%
  group_by(Model,BC_method, quarter) %>%
  dplyr::summarise(
    RMSE = ModelMetrics::rmse(Obs_value, Model_value),
    MAE = ModelMetrics::mae(Obs_value, Model_value)^1,
    R = cor(Obs_value, Model_value)^1,
    BIAS = ((sum(Model_value) - sum(Obs_value))/sum(Obs_value))*100,
    VBIAS = hydroGOF::NSE(Model_value, Obs_value, na.rm=TRUE) # Positive values indicate overestimation bias, whereas negative values indicate model underestimation bias
  )

# data.frame values are requested
df.summ.sta.South_Pacific

# Raw observational values are isolated and repeated
test1 <- mefa:::rep.data.frame(df.summ.sta.South_Pacific[c(25,26,27,28), c(4,5,6,7,8)], times = 7)
test2 <- mefa:::rep.data.frame(df.summ.sta.South_Pacific[c(53,54,55,56), c(4,5,6,7,8)], times = 7)
test3 <- mefa:::rep.data.frame(df.summ.sta.South_Pacific[c(81,82,83,84), c(4,5,6,7,8)], times = 7)
test4 <- mefa:::rep.data.frame(df.summ.sta.South_Pacific[c(109,110,111,112), c(4,5,6,7,8)], times = 7)
test5 <- mefa:::rep.data.frame(df.summ.sta.South_Pacific[c(137,138,139,140), c(4,5,6,7,8)], times = 7)
test6 <- mefa:::rep.data.frame(df.summ.sta.South_Pacific[c(165,166,167,168), c(4,5,6,7,8)], times = 7)
test7 <- mefa:::rep.data.frame(df.summ.sta.South_Pacific[c(193,194,195,196), c(4,5,6,7,8)], times = 7)
test8 <- mefa:::rep.data.frame(df.summ.sta.South_Pacific[c(221,222,223,224), c(4,5,6,7,8)], times = 7)

# A data.frame is creted
test8 <- rbind(test1,test2,test3,test4,test5,test6,test7,test8)

# RMSE, MAE and R relative variables are renamed
names(test8) <- c("xRMSE", "xMAE",  "xR", "xBIAS", "VBIAS")

# RMSE, MAE and R relative variables are incorporated in the original data.frame
df.summ.sta.South_Pacific$xRMSE <- test8$xRMSE
df.summ.sta.South_Pacific$xMAE <- test8$xMAE
df.summ.sta.South_Pacific$xR <- test8$xR
df.summ.sta.South_Pacific$xBIAS <- test8$xBIAS
df.summ.sta.South_Pacific$VBIAS <- round(test8$VBIAS, 1)

# RMSE, MAE and R relative variables are created
df.summ.sta.South_Pacific$dRMSE <- (1-(df.summ.sta.South_Pacific$RMSE/df.summ.sta.South_Pacific$xRMSE))*100
df.summ.sta.South_Pacific$dMAE <- (1-(df.summ.sta.South_Pacific$MAE/df.summ.sta.South_Pacific$xMAE))*100
df.summ.sta.South_Pacific$dR <- ((df.summ.sta.South_Pacific$R - df.summ.sta.South_Pacific$xR)/df.summ.sta.South_Pacific$R)*100
df.summ.sta.South_Pacific$dBIAS <- abs(df.summ.sta.South_Pacific$BIAS) - abs(df.summ.sta.South_Pacific$xBIAS)

# Raw observational values are deletd from the original data.frame
#df.summ.sta.South_Pacific <- df.summ.sta.South_Pacific[-c(21,22,23,24,53,54,55,56,81,82,83,84,109,110,111,112,137,138,139,140,165,166,167,168,193,194,195,196), ]

# A region variable is added
df.summ.sta.South_Pacific$Region <- c("South_Pacific")

# RMSE, MAE and R relative variables are rounded to one decimal
df.summ.sta.South_Pacific$dRMSE <- round(df.summ.sta.South_Pacific$dRMSE, 1)
df.summ.sta.South_Pacific$dMAE <- round(df.summ.sta.South_Pacific$dMAE, 1)
df.summ.sta.South_Pacific$dR <- round(df.summ.sta.South_Pacific$dR, 1)
df.summ.sta.South_Pacific$dBIAS <- round(df.summ.sta.South_Pacific$dBIAS, 1)

# A delta variable is created between modelled and observed values 
df.taylor.subset.South_Pacific$delta <- df.taylor.subset.South_Pacific$Model_value/df.taylor.subset.South_Pacific$Obs_value

# A threshold is defined for visualization purposes only
df.taylor.subset.South_Pacific$Model_value <- ifelse(df.taylor.subset.South_Pacific$Model_value > 700, 700, df.taylor.subset.South_Pacific$Model_value)

# A Taylor quarter diagram is created
TaylorDiagram(df.taylor.subset.South_Pacific,
              obs = "Obs_value",
              mod = "Model_value",
              group = c("UNION"),
              type = c("quarter","Region"),
              normalise = FALSE,
              annotate = "RMSE",
              cols = scale07,
              fontsize= 18, # 28
              pch = c(15,16,17,10,13,8,11), 
              key = FALSE,
              cex = 0.80)# size 0.80

# A Taylor annual diagram is created
TaylorDiagram(df.taylor.subset.South_Pacific,
              obs = "Obs_value",
              mod = "Model_value",
              group = c("UNION"),
              type = c("Region"),
              normalise = FALSE,
              annotate = "RMSE",
              cols = scale07,
              fontsize= 22, # 28
              pch = c(15,16,17,10,13,8,11), 
              #key = FALSE,
              cex=0.95) # size 0.95

#----------------------------------------------------------------------------------------------------
# SUBBLOCK: Central_Valley Climate Region is isolated
#----------------------------------------------------------------------------------------------------

# Climate Region is selected
df.taylor.subset.Central <- subset(df.taylor.01, Region == "Central_Valley")

# summarise {dplyr} is used to reduce multiple values down to a single value
df.summ.sta.Central <- df.taylor.subset.Central %>%
  group_by(Model,BC_method, quarter) %>%
  dplyr::summarise(
    RMSE = ModelMetrics::rmse(Obs_value, Model_value),
    MAE = ModelMetrics::mae(Obs_value, Model_value)^1,
    R = cor(Obs_value, Model_value)^1,
    BIAS = ((sum(Model_value) - sum(Obs_value))/sum(Obs_value))*100,
    VBIAS = hydroGOF::NSE(Model_value, Obs_value, na.rm=TRUE) # Positive values indicate overestimation bias, whereas negative values indicate model underestimation bias
  )

# data.frame values are requested
df.summ.sta.Central

# Raw observational values are isolated and repeated
test1 <- mefa:::rep.data.frame(df.summ.sta.Central[c(25,26,27,28), c(4,5,6,7,8)], times = 7)
test2 <- mefa:::rep.data.frame(df.summ.sta.Central[c(53,54,55,56), c(4,5,6,7,8)], times = 7)
test3 <- mefa:::rep.data.frame(df.summ.sta.Central[c(81,82,83,84), c(4,5,6,7,8)], times = 7)
test4 <- mefa:::rep.data.frame(df.summ.sta.Central[c(109,110,111,112), c(4,5,6,7,8)], times = 7)
test5 <- mefa:::rep.data.frame(df.summ.sta.Central[c(137,138,139,140), c(4,5,6,7,8)], times = 7)
test6 <- mefa:::rep.data.frame(df.summ.sta.Central[c(165,166,167,168), c(4,5,6,7,8)], times = 7)
test7 <- mefa:::rep.data.frame(df.summ.sta.Central[c(193,194,195,196), c(4,5,6,7,8)], times = 7)
test8 <- mefa:::rep.data.frame(df.summ.sta.Central[c(221,222,223,224), c(4,5,6,7,8)], times = 7)

# A data.frame is creted
test8 <- rbind(test1,test2,test3,test4,test5,test6,test7,test8)

# RMSE, MAE and R relative variables are renamed
names(test8) <- c("xRMSE", "xMAE",  "xR", "xBIAS", "VBIAS")

# RMSE, MAE and R relative variables are incorporated in the original data.frame
df.summ.sta.Central$xRMSE <- test8$xRMSE
df.summ.sta.Central$xMAE <- test8$xMAE
df.summ.sta.Central$xR <- test8$xR
df.summ.sta.Central$xBIAS <- test8$xBIAS
df.summ.sta.Central$VBIAS <- round(test8$VBIAS, 1)

# RMSE, MAE and R relative variables are created
df.summ.sta.Central$dRMSE <- (1-(df.summ.sta.Central$RMSE/df.summ.sta.Central$xRMSE))*100
df.summ.sta.Central$dMAE <- (1-(df.summ.sta.Central$MAE/df.summ.sta.Central$xMAE))*100
df.summ.sta.Central$dR <- ((df.summ.sta.Central$R - df.summ.sta.Central$xR)/df.summ.sta.Central$R)*100
df.summ.sta.Central$dBIAS <- abs(df.summ.sta.Central$BIAS) - abs(df.summ.sta.Central$xBIAS)

# Raw observational values are deletd from the original data.frame
#df.summ.sta.Central <- df.summ.sta.Central[-c(21,22,23,24,53,54,55,56,81,82,83,84,109,110,111,112,137,138,139,140,165,166,167,168,193,194,195,196), ]

# A region variable is added
df.summ.sta.Central$Region <- c("Central_Valley")

# RMSE, MAE and R relative variables are rounded to one decimal
df.summ.sta.Central$dRMSE <- round(df.summ.sta.Central$dRMSE, 1)
df.summ.sta.Central$dMAE <- round(df.summ.sta.Central$dMAE, 1)
df.summ.sta.Central$dR <- round(df.summ.sta.Central$dR, 1)
df.summ.sta.Central$dBIAS <- round(df.summ.sta.Central$dBIAS, 1)

# A delta variable is created between modelled and observed values 
df.taylor.subset.Central$delta <- df.taylor.subset.Central$Model_value/df.taylor.subset.Central$Obs_value

# A threshold is defined for visualization purposes only
df.taylor.subset.Central$Model_value <- ifelse(df.taylor.subset.Central$Model_value > 700, 700, df.taylor.subset.Central$Model_value)

# A Taylor quarter diagram is created
TaylorDiagram(df.taylor.subset.Central,
              obs = "Obs_value",
              mod = "Model_value",
              group = c("UNION"),
              type = c("quarter","Region"),
              normalise = FALSE,
              annotate = "RMSE",
              cols = scale07,
              fontsize= 18, # 28
              pch = c(15,16,17,10,13,8,11), 
              key = FALSE,
              cex = 0.80)# size 0.80

# A Taylor annual diagram is created
TaylorDiagram(df.taylor.subset.Central,
              obs = "Obs_value",
              mod = "Model_value",
              group = c("UNION"),
              type = c("Region"),
              normalise = FALSE,
              annotate = "RMSE",
              cols = scale07,
              fontsize= 22, # 28
              pch = c(15,16,17,10,13,8,11), 
              #key = FALSE,
              cex=0.95) # size 0.95

#----------------------------------------------------------------------------------------------------
# SUBBLOCK: North Climate Region is isolated
#----------------------------------------------------------------------------------------------------

# Climate Region is selected
df.taylor.subset.North <- subset(df.taylor.01, Region == "North")

# summarise {dplyr} is used to reduce multiple values down to a single value
df.summ.sta.North <- df.taylor.subset.North %>%
  group_by(Model,BC_method, quarter) %>%
  dplyr::summarise(
    RMSE = ModelMetrics::rmse(Obs_value, Model_value),
    MAE = ModelMetrics::mae(Obs_value, Model_value)^1,
    R = cor(Obs_value, Model_value)^1,
    BIAS = ((sum(Model_value) - sum(Obs_value))/sum(Obs_value))*100,
    VBIAS = hydroGOF::NSE(Model_value, Obs_value, na.rm=TRUE) # Positive values indicate overestimation bias, whereas negative values indicate model underestimation bias
  )

# data.frame values are requested
df.summ.sta.North

# Raw observational values are isolated and repeated
test1 <- mefa:::rep.data.frame(df.summ.sta.North[c(25,26,27,28), c(4,5,6,7,8)], times = 7)
test2 <- mefa:::rep.data.frame(df.summ.sta.North[c(53,54,55,56), c(4,5,6,7,8)], times = 7)
test3 <- mefa:::rep.data.frame(df.summ.sta.North[c(81,82,83,84), c(4,5,6,7,8)], times = 7)
test4 <- mefa:::rep.data.frame(df.summ.sta.North[c(109,110,111,112), c(4,5,6,7,8)], times = 7)
test5 <- mefa:::rep.data.frame(df.summ.sta.North[c(137,138,139,140), c(4,5,6,7,8)], times = 7)
test6 <- mefa:::rep.data.frame(df.summ.sta.North[c(165,166,167,168), c(4,5,6,7,8)], times = 7)
test7 <- mefa:::rep.data.frame(df.summ.sta.North[c(193,194,195,196), c(4,5,6,7,8)], times = 7)
test8 <- mefa:::rep.data.frame(df.summ.sta.North[c(221,222,223,224), c(4,5,6,7,8)], times = 7)

# A data.frame is creted
test8 <- rbind(test1,test2,test3,test4,test5,test6,test7,test8)


# RMSE, MAE and R relative variables are renamed
names(test8) <- c("xRMSE", "xMAE",  "xR", "xBIAS", "VBIAS")

# RMSE, MAE and R relative variables are incorporated in the original data.frame
df.summ.sta.North$xRMSE <- test8$xRMSE
df.summ.sta.North$xMAE <- test8$xMAE
df.summ.sta.North$xR <- test8$xR
df.summ.sta.North$xBIAS <- test8$xBIAS
df.summ.sta.North$VBIAS <- round(test8$VBIAS, 1)

# RMSE, MAE and R relative variables are created
df.summ.sta.North$dRMSE <- (1-(df.summ.sta.North$RMSE/df.summ.sta.North$xRMSE))*100
df.summ.sta.North$dMAE <- (1-(df.summ.sta.North$MAE/df.summ.sta.North$xMAE))*100
df.summ.sta.North$dR <- ((df.summ.sta.North$R - df.summ.sta.North$xR)/df.summ.sta.North$R)*100
df.summ.sta.North$dBIAS <- abs(df.summ.sta.North$BIAS) - abs(df.summ.sta.North$xBIAS)

# Raw observational values are deletd from the original data.frame
#df.summ.sta.North <- df.summ.sta.North[-c(21,22,23,24,53,54,55,56,81,82,83,84,109,110,111,112,137,138,139,140,165,166,167,168,193,194,195,196), ]

# A region variable is added
df.summ.sta.North$Region <- c("North")

# RMSE, MAE and R relative variables are rounded to one decimal
df.summ.sta.North$dRMSE <- round(df.summ.sta.North$dRMSE, 1)
df.summ.sta.North$dMAE <- round(df.summ.sta.North$dMAE, 1)
df.summ.sta.North$dR <- round(df.summ.sta.North$dR, 1)
df.summ.sta.North$dBIAS <- round(df.summ.sta.North$dBIAS, 1)

# A delta variable is created between modelled and observed values 
df.taylor.subset.North$delta <- df.taylor.subset.North$Model_value/df.taylor.subset.North$Obs_value

# A threshold is defined for visualization purposes only
df.taylor.subset.North$Model_value <- ifelse(df.taylor.subset.North$Model_value > 700, 700, df.taylor.subset.North$Model_value)

# A Taylor quarter diagram is created
TaylorDiagram(df.taylor.subset.North,
              obs = "Obs_value",
              mod = "Model_value",
              group = c("UNION"),
              type = c("quarter","Region"),
              normalise = FALSE,
              annotate = "RMSE",
              cols = scale07,
              fontsize= 18, # 28
              pch = c(15,16,17,10,13,8,11), 
              key = FALSE,
              cex = 0.80)# size 0.80

# A Taylor annual diagram is created
TaylorDiagram(df.taylor.subset.North,
              obs = "Obs_value",
              mod = "Model_value",
              group = c("UNION"),
              type = c("Region"),
              normalise = FALSE,
              annotate = "RMSE",
              cols = scale07,
              fontsize= 22, # 28
              pch = c(15,16,17,10,13,8,11), 
              #key = FALSE,
              cex = 0.95) # size 0.95

#----------------------------------------------------------------------------------------------------
# SUBBLOCK: Caribbean Climate Region is isolated
#----------------------------------------------------------------------------------------------------

# Climate Region is selected
df.taylor.subset.Caribbean <- subset(df.taylor.01, Region == "Caribbean")

# summarise {dplyr} is used to reduce multiple values down to a single value
df.summ.sta.Caribbean <- df.taylor.subset.Caribbean %>%
  group_by(Model,BC_method, quarter) %>%
  dplyr::summarise(
    RMSE = ModelMetrics::rmse(Obs_value, Model_value),
    MAE = ModelMetrics::mae(Obs_value, Model_value)^1,
    R = cor(Obs_value, Model_value)^1,
    BIAS = ((sum(Model_value) - sum(Obs_value))/sum(Obs_value))*100,
    VBIAS = hydroGOF::NSE(Model_value, Obs_value, na.rm=TRUE) # Positive values indicate overestimation bias, whereas negative values indicate model underestimation bias
  )

# data.frame values are requested
df.summ.sta.Caribbean

# Raw observational values are isolated and repeated
test1 <- mefa:::rep.data.frame(df.summ.sta.Caribbean[c(25,26,27,28), c(4,5,6,7,8)], times = 7)
test2 <- mefa:::rep.data.frame(df.summ.sta.Caribbean[c(53,54,55,56), c(4,5,6,7,8)], times = 7)
test3 <- mefa:::rep.data.frame(df.summ.sta.Caribbean[c(81,82,83,84), c(4,5,6,7,8)], times = 7)
test4 <- mefa:::rep.data.frame(df.summ.sta.Caribbean[c(109,110,111,112), c(4,5,6,7,8)], times = 7)
test5 <- mefa:::rep.data.frame(df.summ.sta.Caribbean[c(137,138,139,140), c(4,5,6,7,8)], times = 7)
test6 <- mefa:::rep.data.frame(df.summ.sta.Caribbean[c(165,166,167,168), c(4,5,6,7,8)], times = 7)
test7 <- mefa:::rep.data.frame(df.summ.sta.Caribbean[c(193,194,195,196), c(4,5,6,7,8)], times = 7)
test8 <- mefa:::rep.data.frame(df.summ.sta.Caribbean[c(221,222,223,224), c(4,5,6,7,8)], times = 7)

# A data.frame is creted
test8 <- rbind(test1,test2,test3,test4,test5,test6,test7,test8)

# RMSE, MAE and R relative variables are renamed
names(test8) <- c("xRMSE", "xMAE",  "xR", "xBIAS", "VBIAS")

# RMSE, MAE and R relative variables are incorporated in the original data.frame
df.summ.sta.Caribbean$xRMSE <- test8$xRMSE
df.summ.sta.Caribbean$xMAE <- test8$xMAE
df.summ.sta.Caribbean$xR <- test8$xR
df.summ.sta.Caribbean$xBIAS <- test8$xBIAS
df.summ.sta.Caribbean$VBIAS <- round(test8$VBIAS, 1)

# RMSE, MAE and R relative variables are created
df.summ.sta.Caribbean$dRMSE <- (1-(df.summ.sta.Caribbean$RMSE/df.summ.sta.Caribbean$xRMSE))*100
df.summ.sta.Caribbean$dMAE <- (1-(df.summ.sta.Caribbean$MAE/df.summ.sta.Caribbean$xMAE))*100
df.summ.sta.Caribbean$dR <- ((df.summ.sta.Caribbean$R - df.summ.sta.Caribbean$xR)/df.summ.sta.Caribbean$R)*100
df.summ.sta.Caribbean$dBIAS <- abs(df.summ.sta.Caribbean$BIAS) - abs(df.summ.sta.Caribbean$xBIAS)

# Raw observational values are deletd from the original data.frame
#df.summ.sta.Caribbean <- df.summ.sta.Caribbean[-c(21,22,23,24,53,54,55,56,81,82,83,84,109,110,111,112,137,138,139,140,165,166,167,168,193,194,195,196), ]

# A region variable is added
df.summ.sta.Caribbean$Region <- c("Caribbean")

# RMSE, MAE and R relative variables are rounded to one decimal
df.summ.sta.Caribbean$dRMSE <- round(df.summ.sta.Caribbean$dRMSE, 1)
df.summ.sta.Caribbean$dMAE <- round(df.summ.sta.Caribbean$dMAE, 1)
df.summ.sta.Caribbean$dR <- round(df.summ.sta.Caribbean$dR, 1)
df.summ.sta.Caribbean$dBIAS <- round(df.summ.sta.Caribbean$dBIAS, 1)

# A delta variable is created between modelled and observed values 
df.taylor.subset.Caribbean$delta <- df.taylor.subset.Caribbean$Model_value/df.taylor.subset.Caribbean$Obs_value

# A threshold is defined for visualization purposes only
df.taylor.subset.Caribbean$Model_value <- ifelse(df.taylor.subset.Caribbean$Model_value > 700, 700, df.taylor.subset.Caribbean$Model_value)

# A Taylor quarter diagram is created
TaylorDiagram(df.taylor.subset.Caribbean,
              obs = "Obs_value",
              mod = "Model_value",
              group = c("UNION"),
              type = c("quarter","Region"),
              normalise = FALSE,
              annotate = "RMSE",
              cols = scale07,
              fontsize= 18, # 28
              pch = c(15,16,17,10,13,8,11), 
              key = FALSE,
              cex = 0.80)# size 0.80

# A Taylor annual diagram is created
TaylorDiagram(df.taylor.subset.Caribbean,
              obs = "Obs_value",
              mod = "Model_value",
              group = c("UNION"),
              type = c("Region"),
              normalise = FALSE,
              annotate = "RMSE",
              cols = scale07,
              fontsize= 22, # 28
              pch = c(15,16,17,10,13,8,11), 
              #key = FALSE,
              cex=0.95) # size 0.95

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Heatmaps generation for each Climatic Region
# Generates: Figures 6,7,8,9. Seasonal RMSE/MAE/CORR/BIAS performance of precipitation bias correction methods
# for GCM-RCMs during the validation period 1981-1995 for all climatic region of Costa Rica. Bold cell 
# numbers represent absolute RMSE/MAE/CORR/BIAS (mm/month) values before (RAW) and after bias correction. .
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

# A compiled data.frame is created
df.heatmap <- rbind(df.summ.sta.North_Pacific,
                    df.summ.sta.Central_Pacific,
                    df.summ.sta.South_Pacific,
                    df.summ.sta.Central,
                    df.summ.sta.North,
                    df.summ.sta.Caribbean)

# Objective fuction values ared rounded to N decimals
df.heatmap$RMSE <- round(df.heatmap$RMSE, 0)
df.heatmap$MAE <- round(df.heatmap$MAE, 0)
df.heatmap$R <- round(df.heatmap$R, 2)
df.heatmap$BIAS <- round(df.heatmap$BIAS, 0)

# Objective fuction values ared rounded to N decimals
df.heatmap$dRMSE <- round(df.heatmap$dRMSE, 0)
df.heatmap$dMAE <- round(df.heatmap$dMAE, 0)
df.heatmap$dR <- round(df.heatmap$dR, 1)
df.heatmap$dBIAS <- round(df.heatmap$dBIAS, 0)

# Objective fuction values ared rounded to N decimals
df.heatmap$xRMSE <- round(df.heatmap$xRMSE, 0)
df.heatmap$xMAE <- round(df.heatmap$xMAE, 0)
df.heatmap$xR <- round(df.heatmap$xR, 0)
df.heatmap$xBIAS <- round(df.heatmap$xBIAS, 0)

# A "Model" monthly vector is created for visualization puposes
v.select.model <- c("GPCC", "EM", "PRECIS", "CanESM2", "EARTH", "HadGEM2", "MIROC", "MPI")

# Models are ordered to avoid ggplot plotting conflicts
df.heatmap$Model <- ordered(df.heatmap$Model, v.select.model)

# A "BC" monthly vector is created for visualization puposes
v.select.bcm <- c("GPQM","GQM","EQM","PTR","DT","LS","UC")

# BC are ordered to avoid ggplot plotting conflicts
df.heatmap$BC_method <- ordered(df.heatmap$BC_method, v.select.bcm)

# Regin attribute is converted to factor class
df.heatmap$Region <- as.factor(df.heatmap$Region)

# A "Regions" monthly vector is created for visualization puposes
v.select.regioncr <- c("North_Pacific", "Central_Pacific", "South_Pacific", "Central_Valley", "North", "Caribbean")

# Regions are ordered to avoid ggplot plotting conflicts
df.heatmap$Region <- ordered(df.heatmap$Region, v.select.regioncr)

#----------------------------------------------------------------------------------------------------
# SUBBLOCK: ggplot2 heatmaps configuration
#----------------------------------------------------------------------------------------------------

# A RMSE heat ggplot2 object is created
# Color gradient is defined
RMSE.max.color <- 100#max(df.heatmap$dRMSE)
temp.mean.RMSE <- subset(df.heatmap, dRMSE > 0)
RMSE.mean.color <- median(temp.mean.RMSE$dRMSE)

# Figure_06 of the Manuscript. RMSE Heatmap
ggplot() +
  geom_tile(aes(y = BC_method,x = Model,fill = dRMSE),data=df.heatmap,show.legend=TRUE) + # x = Model,y = BC_method
  facet_grid(facets = quarter ~ Region) + # Region ~ quarter
  scale_fill_gradient2(guide = guide_legend(),breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0),
                       low = '#4575b4',mid = '#ffffbf',high = '#d73027',# low = '#c7e9b4',mid = '#41b6c4',high = '#253494', # low = '#b2182b',mid = '#f7f7f7',high = '#2166ac',
                       midpoint = RMSE.max.color/2, limits = c(0,RMSE.max.color)) +
  geom_text(aes(x = Model,y = BC_method,label = RMSE, fontface = 2),data=df.heatmap, # or dRMSE
            colour = '#252525',size = 2.75,hjust = 0.50,vjust = 0.90,parse = FALSE) +
  geom_text(aes(x = Model,y = BC_method,label = dRMSE, fontface = 3),data=df.heatmap, # or dRMSE
            colour = '#252525',size = 2.75,hjust = 0.50,vjust = -0.45,parse = FALSE) +
  theme_bw() +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), text=element_text(size=18,  family="serif"), legend.position="bottom") +
  xlab("GCM-RCM Model") +
  ylab("Bias Correction (BC) Model")

# A MAE heat ggplot2 object is created
# Color gradient is defined
MAE.max.color <- 100#max(df.heatmap$dMAE)
temp.mean.MAE <- subset(df.heatmap, dMAE > 0)
MAE.mean.color <- median(temp.mean.MAE$dMAE)

# Figure_07 of the Manuscript. MAE Heatmap
ggplot() +
  geom_tile(aes(y = BC_method,x = Model,fill = dMAE),data=df.heatmap,show.legend=TRUE) + # x = Model,y = BC_method
  facet_grid(facets = quarter ~ Region) + # Region ~ quarter
  scale_fill_gradient2(guide = guide_legend(),breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0),
                       low = '#4575b4',mid = '#ffffbf',high = '#d73027',# low = '#c7e9b4',mid = '#41b6c4',high = '#253494', # low = '#b2182b',mid = '#f7f7f7',high = '#2166ac',
                       midpoint = MAE.max.color/2,limits = c(0,MAE.max.color)) +
  geom_text(aes(x = Model,y = BC_method,label = MAE, fontface = 2),data=df.heatmap, # or dMAE
            colour = '#252525',size = 2.75,hjust = 0.50,vjust = 0.90,parse = FALSE) +
  geom_text(aes(x = Model,y = BC_method,label = dMAE, fontface = 3),data=df.heatmap, # or dMAE
            colour = '#252525',size = 2.75,hjust = 0.50,vjust = -0.45,parse = FALSE) +
  theme_bw() +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), text=element_text(size=18,  family="serif"), legend.position="bottom") +
  xlab("GCM-RCM Model") +
  ylab("Bias Correction (BC) Model")

# A R heat ggplot2 object is created
# Color gradient is defined
R.max.color <- max(df.heatmap$dR)
R.max.color <- 250
temp.mean.R <- subset(df.heatmap, dR > 0)
R.mean.color <- median(temp.mean.R$dR)

# A dummy CORR data.frame is created
df.heatmap.dummy <- df.heatmap

# data.frame variables are rounded to 1 decimal
df.heatmap.dummy$R <- round(df.heatmap.dummy$R, 1)
df.heatmap.dummy$dR <- round(df.heatmap.dummy$dR, 0)

# dA values over 100 are deleted for visualization purposes
df.heatmap.dummy$dR <- ifelse(between(df.heatmap.dummy$dR,-999,999), df.heatmap.dummy$dR, NA)

# Figure_08 of the Manuscript. CORR Heatmap
ggplot() +
  geom_tile(aes(y = BC_method,x = Model,fill = dR),data=df.heatmap,show.legend=TRUE) + # x = Model,y = BC_method
  facet_grid(facets = quarter ~ Region) + # Region ~ quarter
  scale_fill_gradient2(guide = guide_legend(),breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0),
                       low = '#4575b4',mid = '#ffffbf',high = '#d73027',# low = '#c7e9b4',mid = '#41b6c4',high = '#253494', # low = '#b2182b',mid = '#f7f7f7',high = '#2166ac',
                       midpoint = 130,limits = c(0,R.max.color)) +
  geom_text(aes(x = Model,y = BC_method,label = R, fontface = 2),data=df.heatmap.dummy, # or dR
            colour = '#252525',size = 2.75,hjust = 0.50,vjust = 0.90,parse = FALSE) +
  geom_text(aes(x = Model,y = BC_method,label = dR, fontface = 3),data=df.heatmap.dummy, # or dR
            colour = '#252525',size = 2.75,hjust = 0.50,vjust = -0.45,parse = FALSE) +
  theme_bw() +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), text=element_text(size=18,  family="serif"), legend.position="bottom") +
  xlab("GCM-RCM Model") +
  ylab("Bias Correction (BC) Model")

# A BIAS heat ggplot2 object is created
# Color gradient is defined
BIAS.max.color <- max(df.heatmap$dBIAS)
BIAS.max.color <- 100
temp.mean.BIAS <- subset(df.heatmap, dBIAS > 0)
BIAS.mean.color <- median(temp.mean.BIAS$dBIAS)

# Figure_09 of the Manuscript. BIAS Heatmap
ggplot() +
  geom_tile(aes(y = BC_method,x = Model,fill = BIAS),data=df.heatmap,show.legend=TRUE) + # x = Model,y = BC_method
  facet_grid(facets = quarter ~ Region) + # Region ~ quarter
  scale_fill_gradient2(guide = guide_legend(),breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0),
                       low ='#d73027' ,mid = '#ffffbf',high = '#4575b4',# low = '#c7e9b4',mid = '#41b6c4',high = '#253494', # low = '#b2182b',mid = '#f7f7f7',high = '#2166ac',
                       midpoint = 0,limits = c(-100,100)) +
  geom_text(aes(x = Model,y = BC_method,label = BIAS, fontface = 2),data=df.heatmap, # or dBIAS
            colour = '#252525',size = 2.75,hjust = 0.50,vjust = 0.50,parse = FALSE) +
  #geom_text(aes(x = Model,y = BC_method,label = xBIAS, fontface = 3),data=df.heatmap, # or dBIAS
  #          colour = '#252525',size = 3.0,hjust = 0.50,vjust = -0.45,parse = FALSE) +
  theme_bw() +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), text=element_text(size=18,  family="serif"), legend.position="bottom") +
  xlab("GCM-RCM Model") +
  ylab("Bias Correction (BC) Model")

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: CDF
# Generates: Figure 10 of the Manuscript
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

# Climate Region is selected
df.taylor.cdf <- subset(df.taylor.01, quarter == "DJF")

# Models are ordered to avoid ggplot plotting conflicts
df.taylor.cdf$Model <- ordered(df.taylor.cdf$Model, v.select.model)

# Regions are ordered to avoid ggplot plotting conflicts
df.taylor.cdf$Region <- ordered(df.taylor.cdf$Region, v.select.regioncr)

# BC are ordered to avoid ggplot plotting conflicts
df.taylor.cdf$BC_method <- ordered(df.taylor.cdf$BC_method, rev(v.select.bcm))

# Figure_10 of the Manuscript. CDF (a)
ggplot() +
  geom_step(aes(x = Model_value,colour = Model), #linetype = BC_method
            data=df.taylor.cdf,size = 1.00,stat = 'ecdf')+
  facet_grid(facets = BC_method ~ Region, scales = 'free') +
  theme_bw() +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0),
                     limits = c(0,800)) +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), 
        text=element_text(size=20,  family="serif")) +
  scale_color_manual(values = scale10) +
  geom_step(aes(x = Obs_value),data=df.taylor.cdf,size = 1.25,stat = 'ecdf') +
  xlab("GCM-RCM Model") +
  ylab("Bias Correction (BC) Model")

# Climate Region is selected
df.taylor.cdf <- subset(df.taylor.01, quarter == "MAM")

# Models are ordered to avoid ggplot plotting conflicts
df.taylor.cdf$Model <- ordered(df.taylor.cdf$Model, v.select.model)

# Regions are ordered to avoid ggplot plotting conflicts
df.taylor.cdf$Region <- ordered(df.taylor.cdf$Region, v.select.regioncr)

# BC are ordered to avoid ggplot plotting conflicts
df.taylor.cdf$BC_method <- ordered(df.taylor.cdf$BC_method, rev(v.select.bcm))

# Figure_10 of the Manuscript. CDF (b)
ggplot() +
  geom_step(aes(x = Model_value,colour = Model), #linetype = BC_method
            data=df.taylor.cdf,size = 1.00,stat = 'ecdf')+
  facet_grid(facets = BC_method ~ Region, scales = 'free') +
  theme_bw() +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0),
                     limits = c(0,800)) +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), 
        text=element_text(size=20,  family="serif")) +
  scale_color_manual(values = scale10) +
  geom_step(aes(x = Obs_value),data=df.taylor.cdf,size = 1.25,stat = 'ecdf') +
  xlab("GCM-RCM Model") +
  ylab("Bias Correction (BC) Model")

# Climate Region is selected
df.taylor.cdf <- subset(df.taylor.01, quarter == "JJA")

# Models are ordered to avoid ggplot plotting conflicts
df.taylor.cdf$Model <- ordered(df.taylor.cdf$Model, v.select.model)

# Regions are ordered to avoid ggplot plotting conflicts
df.taylor.cdf$Region <- ordered(df.taylor.cdf$Region, v.select.regioncr)

# BC are ordered to avoid ggplot plotting conflicts
df.taylor.cdf$BC_method <- ordered(df.taylor.cdf$BC_method, rev(v.select.bcm))

# Figure_10 of the Manuscript. CDF (c)
ggplot() +
  geom_step(aes(x = Model_value,colour = Model), #linetype = BC_method
            data=df.taylor.cdf,size = 1.00,stat = 'ecdf')+
  facet_grid(facets = BC_method ~ Region, scales = 'free') +
  theme_bw() +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0),
                     limits = c(0,800)) +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), 
        text=element_text(size=20,  family="serif")) +
  scale_color_manual(values = scale10) +
  geom_step(aes(x = Obs_value),data=df.taylor.cdf,size = 1.25,stat = 'ecdf') +
  xlab("GCM-RCM Model") +
  ylab("Bias Correction (BC) Model")

# Climate Region is selected
df.taylor.cdf <- subset(df.taylor.01, quarter == "SON")

# Models are ordered to avoid ggplot plotting conflicts
df.taylor.cdf$Model <- ordered(df.taylor.cdf$Model, v.select.model)

# Regions are ordered to avoid ggplot plotting conflicts
df.taylor.cdf$Region <- ordered(df.taylor.cdf$Region, v.select.regioncr)

# BC are ordered to avoid ggplot plotting conflicts
df.taylor.cdf$BC_method <- ordered(df.taylor.cdf$BC_method, rev(v.select.bcm))

# Figure_10 of the Manuscript. CDF (d)
ggplot() +
  geom_step(aes(x = Model_value,colour = Model), #linetype = BC_method
            data=df.taylor.cdf,size = 1.00,stat = 'ecdf')+
  facet_grid(facets = BC_method ~ Region, scales = 'free') +
  theme_bw() +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0),
                     limits = c(0,800)) +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), 
        text=element_text(size=20,  family="serif")) +
  scale_color_manual(values = scale10) +
  geom_step(aes(x = Obs_value),data=df.taylor.cdf,size = 1.25,stat = 'ecdf') +
  xlab("GCM-RCM Model") +
  ylab("Bias Correction (BC) Model")

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Time-series plots generation
# Generates: 
# Figure 2. Spatially-averaged mean monthly precipitation for each climatic region of Costa Rica 
# based on 1961-1990 grid climatology. Error bars represent the standard deviation.
# Figure 4. Observed and RAW mean monthly precipitation for GCM-RCMs during the period 1951-1995 
# for all climatic regions of Costa Rica.
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

# A monthly precipitation subset is created
clim.subset.01 <- subset(df.taylor.01, BC_method == "UC")

# A monthly precipitation subset is created
clim.subset.02 <- clim.subset.01

# A observed ONLY data.frame is created
clim.subset.03 <- subset(df.taylor.01, Model == "GPCC" & BC_method == "UC")

# An observed label is created
clim.subset.03$Model <- c("OBS")

# Modelled and observed values are re-arranged
clim.subset.03$Model_value <- clim.subset.03$Obs_value

# A compiled data.frame is created
clim.subset.04 <- rbind(clim.subset.03,clim.subset.02)

# A simple BIAS variable is created
clim.subset.04$BIAS <- clim.subset.04$Model_value - clim.subset.04$Obs_value

# Observed variable is removed
clim.subset.05 <- subset(clim.subset.04, Model != "OBS")

# Quarter sorting loop is initiated
for (i in 1 : length(clim.subset.04$Month.ordered)) {
  
  # Precipitation values are selected [mm/month] (according to month of the year and date)
  if (clim.subset.04$Month.ordered[i] == "JAN") { clim.subset.04$quarter[i] <- c("DJF") } else if (clim.subset.04$Month.ordered[i] == "FEB") {
    clim.subset.04$quarter[i] <- c("DJF") } else if (clim.subset.04$Month.ordered[i] == "MAR") {
      clim.subset.04$quarter[i] <- c("MAM") } else if (clim.subset.04$Month.ordered[i] == "APR") {
        clim.subset.04$quarter[i] <- c("MAM") } else if (clim.subset.04$Month.ordered[i] == "MAY") {
          clim.subset.04$quarter[i] <- c("MAM") } else if (clim.subset.04$Month.ordered[i] == "JUN") {
            clim.subset.04$quarter[i] <- c("JJA") } else if (clim.subset.04$Month.ordered[i] == "JUL") {
              clim.subset.04$quarter[i] <- c("JJA") } else if (clim.subset.04$Month.ordered[i] == "AUG") {
                clim.subset.04$quarter[i] <- c("JJA") } else if (clim.subset.04$Month.ordered[i] == "SEP") {
                  clim.subset.04$quarter[i] <- c("SON") } else if (clim.subset.04$Month.ordered[i] == "OCT") {
                    clim.subset.04$quarter[i] <- c("SON") } else if (clim.subset.04$Month.ordered[i] == "NOV") {
                      clim.subset.04$quarter[i] <- c("SON") } else if (clim.subset.04$Month.ordered[i] == "DEC") {
                        clim.subset.04$quarter[i] <- c("DJF") }
  
} # Quarter sorting loop is closed

# A new scale08 color is defined
scaleT1 <- c("#E41A1C","#1B9E77","#66A61E","#984EA3","#A6761D","#010101","#FF7F00", "gray50","#377EB8")

# A RAW monthly precipitation stat_summary plot is created
# Observed and RAW mean monthly precipitation for 
# GCM-RCMs during the period 1951-1995 for all climatic regions of Costa Rica.
ggplot() +
  geom_boxplot(aes(y = Model_value,x = quarter,colour = Model),data=clim.subset.04,size = 1.25,na.rm = TRUE) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0),limits = c(0,1000)) +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), text=element_text(size=18,  family="serif"),
        legend.position="bottom") +
  scale_color_manual(values = scaleT1) +
  theme_bw()+
  theme(axis.text.x = element_text(vjust = 0.5,angle = 0.0), text=element_text(size=18,  family="serif"),
        legend.position="bottom") +
  xlab("Month of the year") +
  ylab("Mean precipitation [mm/month]")

# A new scale08 color is defined
scaleT2 <- c("#E41A1C","#1B9E77","#66A61E","#984EA3","#A6761D","#010101","#FF7F00", "#377EB8")

# A RAW monthly precipitation stat_summary plot is created
# Observed and RAW mean monthly precipitation for 
# GCM-RCMs during the period 1951-1995 for all climatic regions of Costa Rica.
ggplot() +
  geom_hline(yintercept = 0.0,colour = 'black',size = 0.95,linetype = 2) +
  geom_boxplot(aes(y = BIAS,x = Model,colour = Model),data=clim.subset.05,size = 0.85,na.rm = TRUE, alpha = 0.85) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0)) +  #,limits = c(-600,600)) +
  #facet_wrap(facets = ~ Month.ordered, ncol = 3, scales = 'free_y') +
  facet_grid(facets = quarter ~ Region, scales = 'free_y') +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), text=element_text(size=18,  family="serif"),
        legend.position="bottom") +
  scale_color_manual(values = scaleT2) +
  theme_bw()+
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), text=element_text(size=12,  family="serif"),
        legend.position="bottom") +
  xlab("Month of the year") +
  ylab("BIAS [mm/month]")

# Months are ordered to avoid ggplot plotting conflicts
clim.subset.05$Region <- ordered(clim.subset.05$Region, v.select.regioncr)

# A RAW monthly precipitation stat_summary plot is created
# Figure_04. Observed and RAW mean monthly precipitation for 
# GCM-RCMs during the period 1951-1995 for all climatic regions of Costa Rica.
ggplot() +
  facet_wrap(facets = ~Region) +
  stat_summary(aes(x = Month.ordered,y = Model_value,colour = Model,group = Model), data=clim.subset.05, linetype = 1, #linetype = 2,
               size = 1.100,alpha = 0.75, fun.data = mean_sdl,fun.args = list(mult = 1),geom = 'line') +
  stat_summary(aes(x = Month.ordered,y = Obs_value,group = Region),data=clim.subset.01, linetype = 1, colour = "black",
               size = 2.300,fun.data = mean_sdl,fun.args = list(mult = 1),geom = 'line') +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0),limits = c(0,400)) +
  theme_bw() +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), text=element_text(size=20,  family="serif"),
        legend.position="bottom") +
  scale_color_manual(values = scaleT2) +
  geom_vline(xintercept = c(5,10), linetype = 2, colour = "gray55", size = 1.5) +
  #ggtitle("SON") +
  xlab("Month of the year") +
  ylab("Mean precipitation [mm/month]")

# data.frame is isolated, melted and aggregated with respecto to the mean
df.RAIN.aggr <- aggregate( Model_value ~ Month.ordered + Region , clim.subset.03, mean)

# data.frame is isolated, melted and aggregated with respecto to the sd
df.RAIN.quant <- aggregate( Model_value ~ Month.ordered + Region , clim.subset.03, sd)

# A simple ratio is calculated
df.RAIN.aggr$SD <- df.RAIN.quant$Model_value

# SD error bar data are defined
df.RAIN.aggr$Ymin <- df.RAIN.aggr$Model_value - df.RAIN.aggr$SD
df.RAIN.aggr$Ymax <- df.RAIN.aggr$Model_value + df.RAIN.aggr$SD

# Months are ordered to avoid ggplot plotting conflicts
df.RAIN.aggr$Region <- ordered(df.RAIN.aggr$Region, v.select.regioncr)

# Figure_02 of the Manuscript. Mean monthly precipitation
ggplot() +
  geom_bar(aes(x = Month.ordered,y = Model_value),data=df.RAIN.aggr,fill = '#cccccc',stat = 'identity') +
  stat_summary(aes(x = Month.ordered,y = Model_value),data=df.RAIN.aggr,size = 2.5,fun.data = mean_cl_normal,fun.args = list(conf.int = 0.95),geom = 'point') +
  facet_wrap(facets = ~Region) +
  geom_errorbar(aes(x = Month.ordered,ymin = Ymin,ymax = Ymax),data=df.RAIN.aggr,size = 0.85,width = 0.55) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 16.0,min.n = 16.0)) +
  theme_bw() +
  theme(axis.text.x = element_text(angle = 90.0),
        text=element_text(size=20,  family="serif")) +
  xlab("Month of the year") +
  ylab("Precipitation (mm/month)") 

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: HadGEM2-HadRM3P Q-Q plots generation
# Generates: Figure 10. Seasonal Q-Q plots of precipitation bias correction methods for HadGEM2-HadRM3P 
# during the validation period 1981-1995 for all climatic region of Costa Rica. 
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

# A HadGEM2-HadRM3P subset is executed
df.qq.base <- subset(df.taylor.01, Model == "EM")

# Loop vectors are defined
vector.region <- c("North_Pacific", "Central_Pacific", "South_Pacific", "Central_Valley", "North", "Caribbean")
vector.quarter <- c("DJF", "MAM", "JJA","SON")

# Vector/list containers are defined and reset
inner.list <- NULL
inner.list <- list()

outter.list <- NULL
outter.list<- list()

# region sorting loop is initiated
for (g in 1:6) {
  
  # Quarter sorting loop is initiated
  for (i in 1:4) {
    
    # seasonal subset is executed
    df.qq.ini <- subset(df.qq.base, quarter == vector.quarter[i])
    
    # A climatic region subset is executed
    df.qq <- subset(df.qq.ini, Region == vector.region[g])
    
    # BC-methods are isolated
    EQM <- subset(df.qq, UNION == "EM_EQM")
    GPQM <- subset(df.qq, UNION == "EM_GPQM")
    GQM <- subset(df.qq, UNION == "EM_GQM")
    PTR <- subset(df.qq, UNION == "EM_PTR")
    DELTA <- subset(df.qq, UNION == "EM_DT")
    RAW <- subset(df.qq, UNION == "EM_UC")
    SCALING <- subset(df.qq, UNION == "EM_LS")
    
    # Empirical quantiles boundaries are defined
    ps <- c(seq(0.025, 0.975, by = 0.020))
    
    # Empirical quantiles boundaries are applied to BC-methods
    Q.RAW <- quantile(RAW$Model_value, ps)
    Q.EQM <- quantile(EQM$Model_value, ps)            
    Q.GPQM <- quantile(GPQM$Model_value, ps)
    Q.GQM <- quantile(GQM$Model_value, ps)
    Q.DELTA <- quantile(DELTA$Model_value, ps)
    Q.PTR <- quantile(PTR$Model_value, ps)
    Q.SCALING <- quantile(SCALING$Model_value, ps)
    Q.OBS <- quantile(SCALING$Obs_value, ps)
    
    # A QQ data.frame is created
    df.qq.plot <- data.frame(Q.RAW, Q.EQM, Q.GPQM, Q.GQM, Q.DELTA, Q.PTR, Q.SCALING, Q.OBS)
    
    # data.frame variables are renamed
    names(df.qq.plot) <- c("UC", "EQM", "GPQM", "GQM", "DT", "PTR", "LS", "OBS")
    
    # data.frame is melted to long format
    df.qq.melt <- melt(df.qq.plot[, 1:7])
    
    # data.frame variables are repeated
    df.qq.melt$OBS <- rep(df.qq.plot$OBS,7)
    
    # A data.frame is compiled
    df.qq.final <- df.qq.melt
    
    # Character variables are included
    df.qq.final$quarter <- vector.quarter[i]
    df.qq.final$Region <- vector.region[g]
    
    # List container is filled
    inner.list[[i]] <- df.qq.final
    
  } # Inner loop is closed
  
  # List containing data frames are converted into one unique data.frame
  df.inner <- ldply(inner.list, data.frame)
  
  # List container is filled
  outter.list[[g]] <- df.inner
  
}

# List containing data frames are converted into one unique data.frame
df.outter <- ldply(outter.list, data.frame)

# data.frame variables are converted to factor class
df.outter$quarter <- as.factor(df.outter$quarter)
df.outter$Region <- as.factor(df.outter$Region)

# Regions are ordered to avoid ggplot plotting conflicts
df.outter$Region <- ordered(df.outter$Region, v.select.regioncr)
df.outter$quarter <- ordered(df.outter$quarter, v.select.name.02)

# A new scale08 color is defined
scale08 <- c("#E41A1C", "#1B9E77", "#66A61E", "#984EA3", "#377EB8", "#010101", "#FF7F00")

#----------------------------------------------------------------------------------------------------
# SUBBLOCK: ggplot2 HadGEM2-HadRM3P QQPLOTs configuration
#----------------------------------------------------------------------------------------------------

# A DJF quarter subset is executed
df.outter2 <- subset(df.outter, quarter == vector.quarter[1])

# A DJF QQ plot is created
ggplot() +
  #geom_line(aes(x = Q.OBS,y = value,colour = variable),data=df.outter2,size = 0.55) +
  theme_bw() +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), text=element_text(size=18,  family="serif")) +
  geom_abline(data=df.base,size = 0.95,linetype = 2,alpha = 0.7355) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0)) +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0)) +
  facet_wrap(facets = ~Region, scales = 'free', ncol = 2) +
  geom_point(aes(x = OBS,y = value,colour = variable, shape = variable),
             data=df.outter2,size = 2.0) +
  scale_shape_manual(values = c(15,16,17,10,13,8,11)) +
  scale_color_manual(values = scale08) +
  ggtitle("DJF") +
  xlab("Observed [mm/month]") +
  ylab("HadGEM2-HadRM3P [mm/month]") 

# A MAM quarter subset is executed
df.outter2 <- subset(df.outter, quarter == vector.quarter[2])

# A MAM QQ plot is created
ggplot() +
  #geom_line(aes(x = Q.OBS,y = value,colour = variable),data=df.outter2,size = 0.55) +
  theme_bw() +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), text=element_text(size=18,  family="serif")) +
  geom_abline(data=df.base,size = 0.95,linetype = 2,alpha = 0.7355) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0)) +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0)) +
  facet_wrap(facets = ~Region, scales = 'free', ncol = 2) +
  geom_point(aes(x = OBS,y = value,colour = variable, shape = variable),
             data=df.outter2,size = 2.5) +
  scale_shape_manual(values = c(15,16,17,10,13,8,11)) +
  scale_color_manual(values = scale08) +
  ggtitle("MAM") +
  xlab("Observed [mm/month]") +
  ylab("HadGEM2-HadRM3P [mm/month]") 

# A JJA quarter subset is executed
df.outter2 <- subset(df.outter, quarter == vector.quarter[3])

# A JJA QQ plot is created
ggplot() +
  #geom_line(aes(x = Q.OBS,y = value,colour = variable),data=df.outter2,size = 0.55) +
  theme_bw() +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), text=element_text(size=18,  family="serif")) +
  geom_abline(data=df.base,size = 0.95,linetype = 2,alpha = 0.7355) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0)) +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0)) +
  facet_wrap(facets = ~Region, scales = 'free', ncol = 2) +
  geom_point(aes(x = OBS,y = value,colour = variable, shape = variable),
             data=df.outter2,size = 2.5) +
  scale_shape_manual(values = c(15,16,17,10,13,8,11)) +
  scale_color_manual(values = scale08) +
  ggtitle("JJA") +
  xlab("Observed [mm/month]") +
  ylab("HadGEM2-HadRM3P [mm/month]") 

# A SON quarter subset is executed
df.outter2 <- subset(df.outter, quarter == vector.quarter[4])

# A SON QQ plot is created
ggplot() +
  #geom_line(aes(x = Q.OBS,y = value,colour = variable),data=df.outter2,size = 0.55) +
  theme_bw() +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), text=element_text(size=18,  family="serif")) +
  geom_abline(data=df.base,size = 0.95,linetype = 2,alpha = 0.7355) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0)) +
  scale_x_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0)) +
  facet_wrap(facets = ~Region, scales = 'free', ncol = 2) +
  geom_point(aes(x = OBS,y = value,colour = variable, shape = variable),
             data=df.outter2,size = 3.0) +
  scale_shape_manual(values = c(15,16,17,10,13,8,11)) +
  scale_color_manual(values = scale08) +
  ggtitle("SON") +
  xlab("Observed [mm/month]") +
  ylab("HadGEM2-HadRM3P [mm/month]") 

#-------------------------------------------------------------------------------------------------------------------
# END of code
#-------------------------------------------------------------------------------------------------------------------