# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# INSTITUTO TECNOLOGICO DE COSTA RICA
# Escuela de Ingenieria en Construccion
# Construction Engineering School
# https://www.tec.ac.cr
# Eng. MSc. Maikel Mendez Morales
# Email: maikel.mendez@gmail.com; mamendez@itcr.ac.cr
# https://orcid.org/0000-0003-1919-141X
# https://www.scopus.com/authid/detail.uri?authorId=51665581300
# https://scholar.google.com/citations?user=JnmSVFYAAAAJ&hl=en
# https://www.youtube.com/c/maikelmendez
# https://twitter.com/MaikelMendezM
# https://github.com/maikelonu
# Skype: maikel.mendez
# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#-----------------------------------------------------------------------------------------------------------------------------------------------------
# INFO: This script is intended for the execution of raster-based bias-corrected data using quantile mapping (EQM) by {R-Hyfo}
# and power transformation of precipitation (PTR) by {downscaleR} for RCPs 2.6, 4.5 and 8.5 for future scenarios
# 2011-2040, 2041-2070 and 2071-2100
#-----------------------------------------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------------------------------------
# MANUSCRIPT TITLE:
# "Performance Evaluation of Bias Correction Methods for Monthly Precipitation 
#  Projections over Costa Rica"
#-----------------------------------------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------------------------------------
# INPUT FILES:

# netCDF format: all *.nc files included in the following lists

# 1961-1990 period **
# hist_1961_1990_IMN.nc
# hist_1961_1990_PRECIS.nc

# RCP26 Time-series **
# RCP26_2011_2040.nc
# RCP26_2041_2070.nc
# RCP26_2071_2100.nc

# RCP45 Time-series **
# RCP45_2011_2040.nc
# RCP45_2041_2070.nc
# RCP45_2071_2100.nc

# RCP85 Time-series **
# RCP85_2011_2040.nc
# RCP85_2041_2070.nc
# RCP85_2071_2100.nc
#-----------------------------------------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------------------------------------
# MANUSCRIPT FIGURES:
# None
#-----------------------------------------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------------------------------------
# OUTPUT FILES:
# Individual netCDF files as listed per month:
#-----------------------------------------------------------------------------------------------------------------------------------------------------

#-----------------------------------------------------------------------------------------------------------------------------------------------------
# Workspace is cleared
#rm(list = ls())

# CRAN libraries are loaded
require(raster)
require(raster)
require(automap)
require(car)
require(caret)
require(chron)
require(compare)
require(Cubist)
require(DescTools)
require(devtools)
require(doParallel)
require(downscaleR)
require(dplyr)
require(FME)
require(forecast)
require(geoR)
require(ggmap)
require(ggplot2)
require(ggthemes)
require(glmulti)
require(gridExtra)
require(GSIF)
require(gstat)
require(hyfo)
require(intamap)
require(KScorrect)
require(lattice)
require(latticeExtra)
require(loadeR)
require(loadeR.2nc)
require(loadeR.java)
require(maps)
require(maptools)
require(MASS)
require(mlbench)
require(ncdf4)
require(ncdf4.helpers)
require(openair)
require(parallel)
require(pastecs)
require(PCICt)
require(plotrix)
require(plyr)
require(psych)
require(randomForest)
require(ranger)
require(rasterVis)
require(RColorBrewer)
require(rcompanion)
require(readxl)
require(reshape)
require(reshape2)
require(rgdal)
require(rgeos)
require(scales)
require(snow)
require(sp)
require(spatialEco)
require(tibble)
require(transformeR)
require(VSURF)

# Working directory is defined
setwd("/mnt/BACKUP/R_ITC/XLSX")                  # UBUNTU-LINUX
setwd("C:/DATOS/R_ITC/XLSX/FUTURE_ENSEMBLE")     # MS-WINDOWS

# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: PRECIS georeference is loaded
# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

# The blank map is transformed into SpatialGridDataFrame
raster.precis <- read.asciigrid("mod_region.asc", as.image = FALSE)

# The map is transformed into RasterLayer
raster.precis <- raster(raster.precis)

# CRTM05 projection for CR is created
CRTM05 <- CRS("+proj=tmerc +lat_0=0 +lon_0=-84 +k=0.9999 +x_0=500000 +y_0=0 +ellps=WGS84 +towgs84=0,0,0,0,0,0,0 +units=m +no_defs")

# CRTM05 projection is assigned to rasterbrick object
crs(raster.precis) <- CRTM05

# Object attributes are requested
# It can be seen that spa.res is different !!!!
image(raster.precis, col = topo.colors(64))

# A simple plot is requested
#plot(raster.precis)

# A summery is requested
summary(raster.precis)

# A dummy raster is created
raster.precis.dummy <- raster.precis

# Dummy NA values are introduced
values(raster.precis.dummy) <- NA

# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: ENSAMBLE SEASON DJF
# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# DJF i= 12, 1, 2
# DJF i= 3, 4, 5
# JJA i= 6, 7, 8
# SON i= 9, 10, 11

#----------------------------------------------------------------------------------------------------------------
# RCP26. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPraw_ENSEMBLE_rcp26_2011_2040_DEC <- stack("Mean_2011_2040_DELTA_BC_rcp26_DEC.tif")
TEMPraw_ENSEMBLE_rcp26_2041_2070_DEC <- stack("Mean_2041_2070_DELTA_BC_rcp26_DEC.tif")
TEMPraw_ENSEMBLE_rcp26_2071_2100_DEC <- stack("Mean_2071_2100_DELTA_BC_rcp26_DEC.tif")

TEMPraw_ENSEMBLE_rcp26_2011_2040_JAN <- stack("Mean_2011_2040_DELTA_BC_rcp26_JAN.tif")
TEMPraw_ENSEMBLE_rcp26_2041_2070_JAN <- stack("Mean_2041_2070_DELTA_BC_rcp26_JAN.tif")
TEMPraw_ENSEMBLE_rcp26_2071_2100_JAN <- stack("Mean_2071_2100_DELTA_BC_rcp26_JAN.tif")

TEMPraw_ENSEMBLE_rcp26_2011_2040_FEB <- stack("Mean_2011_2040_DELTA_BC_rcp26_FEB.tif")
TEMPraw_ENSEMBLE_rcp26_2041_2070_FEB <- stack("Mean_2041_2070_DELTA_BC_rcp26_FEB.tif")
TEMPraw_ENSEMBLE_rcp26_2071_2100_FEB <- stack("Mean_2071_2100_DELTA_BC_rcp26_FEB.tif")

crs(TEMPraw_ENSEMBLE_rcp26_2011_2040_DEC) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2041_2070_DEC) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2071_2100_DEC) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp26_2011_2040_JAN) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2041_2070_JAN) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2071_2100_JAN) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp26_2011_2040_FEB) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2041_2070_FEB) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2071_2100_FEB) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP45. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPraw_ENSEMBLE_rcp45_2011_2040_DEC <- stack("Mean_2011_2040_DELTA_BC_rcp45_DEC.tif")
TEMPraw_ENSEMBLE_rcp45_2041_2070_DEC <- stack("Mean_2041_2070_DELTA_BC_rcp45_DEC.tif")
TEMPraw_ENSEMBLE_rcp45_2071_2100_DEC <- stack("Mean_2071_2100_DELTA_BC_rcp45_DEC.tif")

TEMPraw_ENSEMBLE_rcp45_2011_2040_JAN <- stack("Mean_2011_2040_DELTA_BC_rcp45_JAN.tif")
TEMPraw_ENSEMBLE_rcp45_2041_2070_JAN <- stack("Mean_2041_2070_DELTA_BC_rcp45_JAN.tif")
TEMPraw_ENSEMBLE_rcp45_2071_2100_JAN <- stack("Mean_2071_2100_DELTA_BC_rcp45_JAN.tif")

TEMPraw_ENSEMBLE_rcp45_2011_2040_FEB <- stack("Mean_2011_2040_DELTA_BC_rcp45_FEB.tif")
TEMPraw_ENSEMBLE_rcp45_2041_2070_FEB <- stack("Mean_2041_2070_DELTA_BC_rcp45_FEB.tif")
TEMPraw_ENSEMBLE_rcp45_2071_2100_FEB <- stack("Mean_2071_2100_DELTA_BC_rcp45_FEB.tif")

crs(TEMPraw_ENSEMBLE_rcp45_2011_2040_DEC) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2041_2070_DEC) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2071_2100_DEC) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp45_2011_2040_JAN) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2041_2070_JAN) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2071_2100_JAN) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp45_2011_2040_FEB) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2041_2070_FEB) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2071_2100_FEB) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP85. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPraw_ENSEMBLE_rcp85_2011_2040_DEC <- stack("Mean_2011_2040_DELTA_BC_rcp85_DEC.tif")
TEMPraw_ENSEMBLE_rcp85_2041_2070_DEC <- stack("Mean_2041_2070_DELTA_BC_rcp85_DEC.tif")
TEMPraw_ENSEMBLE_rcp85_2071_2100_DEC <- stack("Mean_2071_2100_DELTA_BC_rcp85_DEC.tif")

TEMPraw_ENSEMBLE_rcp85_2011_2040_JAN <- stack("Mean_2011_2040_DELTA_BC_rcp85_JAN.tif")
TEMPraw_ENSEMBLE_rcp85_2041_2070_JAN <- stack("Mean_2041_2070_DELTA_BC_rcp85_JAN.tif")
TEMPraw_ENSEMBLE_rcp85_2071_2100_JAN <- stack("Mean_2071_2100_DELTA_BC_rcp85_JAN.tif")

TEMPraw_ENSEMBLE_rcp85_2011_2040_FEB <- stack("Mean_2011_2040_DELTA_BC_rcp85_FEB.tif")
TEMPraw_ENSEMBLE_rcp85_2041_2070_FEB <- stack("Mean_2041_2070_DELTA_BC_rcp85_FEB.tif")
TEMPraw_ENSEMBLE_rcp85_2071_2100_FEB <- stack("Mean_2071_2100_DELTA_BC_rcp85_FEB.tif")

crs(TEMPraw_ENSEMBLE_rcp85_2011_2040_DEC) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2041_2070_DEC) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2071_2100_DEC) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp85_2011_2040_JAN) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2041_2070_JAN) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2071_2100_JAN) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp85_2011_2040_FEB) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2041_2070_FEB) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2071_2100_FEB) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP26. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPbc_ENSEMBLE_rcp26_2011_2040_DEC <- stack("Mean_2011_2040_EQM_BC_rcp26_DEC.tif")
TEMPbc_ENSEMBLE_rcp26_2041_2070_DEC <- stack("Mean_2041_2070_EQM_BC_rcp26_DEC.tif")
TEMPbc_ENSEMBLE_rcp26_2071_2100_DEC <- stack("Mean_2071_2100_EQM_BC_rcp26_DEC.tif")

TEMPbc_ENSEMBLE_rcp26_2011_2040_JAN <- stack("Mean_2011_2040_EQM_BC_rcp26_JAN.tif")
TEMPbc_ENSEMBLE_rcp26_2041_2070_JAN <- stack("Mean_2041_2070_EQM_BC_rcp26_JAN.tif")
TEMPbc_ENSEMBLE_rcp26_2071_2100_JAN <- stack("Mean_2071_2100_EQM_BC_rcp26_JAN.tif")

TEMPbc_ENSEMBLE_rcp26_2011_2040_FEB <- stack("Mean_2011_2040_EQM_BC_rcp26_FEB.tif")
TEMPbc_ENSEMBLE_rcp26_2041_2070_FEB <- stack("Mean_2041_2070_EQM_BC_rcp26_FEB.tif")
TEMPbc_ENSEMBLE_rcp26_2071_2100_FEB <- stack("Mean_2071_2100_EQM_BC_rcp26_FEB.tif")

crs(TEMPbc_ENSEMBLE_rcp26_2011_2040_DEC) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2041_2070_DEC) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2071_2100_DEC) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp26_2011_2040_JAN) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2041_2070_JAN) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2071_2100_JAN) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp26_2011_2040_FEB) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2041_2070_FEB) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2071_2100_FEB) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP45. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPbc_ENSEMBLE_rcp45_2011_2040_DEC <- stack("Mean_2011_2040_EQM_BC_rcp45_DEC.tif")
TEMPbc_ENSEMBLE_rcp45_2041_2070_DEC <- stack("Mean_2041_2070_EQM_BC_rcp45_DEC.tif")
TEMPbc_ENSEMBLE_rcp45_2071_2100_DEC <- stack("Mean_2071_2100_EQM_BC_rcp45_DEC.tif")

TEMPbc_ENSEMBLE_rcp45_2011_2040_JAN <- stack("Mean_2011_2040_EQM_BC_rcp45_JAN.tif")
TEMPbc_ENSEMBLE_rcp45_2041_2070_JAN <- stack("Mean_2041_2070_EQM_BC_rcp45_JAN.tif")
TEMPbc_ENSEMBLE_rcp45_2071_2100_JAN <- stack("Mean_2071_2100_EQM_BC_rcp45_JAN.tif")

TEMPbc_ENSEMBLE_rcp45_2011_2040_FEB <- stack("Mean_2011_2040_EQM_BC_rcp45_FEB.tif")
TEMPbc_ENSEMBLE_rcp45_2041_2070_FEB <- stack("Mean_2041_2070_EQM_BC_rcp45_FEB.tif")
TEMPbc_ENSEMBLE_rcp45_2071_2100_FEB <- stack("Mean_2071_2100_EQM_BC_rcp45_FEB.tif")

crs(TEMPbc_ENSEMBLE_rcp45_2011_2040_DEC) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2041_2070_DEC) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2071_2100_DEC) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp45_2011_2040_JAN) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2041_2070_JAN) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2071_2100_JAN) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp45_2011_2040_FEB) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2041_2070_FEB) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2071_2100_FEB) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP85. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPbc_ENSEMBLE_rcp85_2011_2040_DEC <- stack("Mean_2011_2040_EQM_BC_rcp85_DEC.tif")
TEMPbc_ENSEMBLE_rcp85_2041_2070_DEC <- stack("Mean_2041_2070_EQM_BC_rcp85_DEC.tif")
TEMPbc_ENSEMBLE_rcp85_2071_2100_DEC <- stack("Mean_2071_2100_EQM_BC_rcp85_DEC.tif")

TEMPbc_ENSEMBLE_rcp85_2011_2040_JAN <- stack("Mean_2011_2040_EQM_BC_rcp85_JAN.tif")
TEMPbc_ENSEMBLE_rcp85_2041_2070_JAN <- stack("Mean_2041_2070_EQM_BC_rcp85_JAN.tif")
TEMPbc_ENSEMBLE_rcp85_2071_2100_JAN <- stack("Mean_2071_2100_EQM_BC_rcp85_JAN.tif")

TEMPbc_ENSEMBLE_rcp85_2011_2040_FEB <- stack("Mean_2011_2040_EQM_BC_rcp85_FEB.tif")
TEMPbc_ENSEMBLE_rcp85_2041_2070_FEB <- stack("Mean_2041_2070_EQM_BC_rcp85_FEB.tif")
TEMPbc_ENSEMBLE_rcp85_2071_2100_FEB <- stack("Mean_2071_2100_EQM_BC_rcp85_FEB.tif")

crs(TEMPbc_ENSEMBLE_rcp85_2011_2040_DEC) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2041_2070_DEC) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2071_2100_DEC) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp85_2011_2040_JAN) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2041_2070_JAN) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2071_2100_JAN) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp85_2011_2040_FEB) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2041_2070_FEB) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2071_2100_FEB) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# SUBBLOCK: ASC CR Climatic Regions Masking
#----------------------------------------------------------------------------------------------------------------
# North = 1
# Central_Pacific = 2 +++
# North_Pacific = 3 +++
# Central = 4
# South_Pacific = 5
# Caribbean = 6 +++
#----------------------------------------------------------------------------------------------------------------
CR <- c(1,2,3,4,5,6)
#----------------------------------------------------------------------------------------------------------------

# Climatic-regions CR blank raster is loaded
blank <- paste("mod_region.asc", sep = "/")

# The blank map is transformed into SpatialGridDataFrame
blank.grid <- read.asciigrid(blank, as.image = FALSE)

# The map is transformed into RasterLayer
blank.raster <- raster(blank.grid)

# CRTM05 projection is assigned to rasterbrick object
crs(blank.raster) <- CRTM05

# A simple plot is requested
#plot(blank.raster)

# Climatic-regions blank-maps are created
north.pacific.blank.raster <- blank.raster
central.pacific.blank.raster <- blank.raster
south.pacific.blank.raster <- blank.raster

# Climatic-regions to be Bias-Corrected are isolated
values(north.pacific.blank.raster)[values(north.pacific.blank.raster) != 3] <- NA
values(central.pacific.blank.raster)[values(central.pacific.blank.raster) != 2] <- NA
values(south.pacific.blank.raster)[values(south.pacific.blank.raster) != 5] <- NA

# A simple plot is requested
#plot(north.pacific.blank.raster)

# Climatic-regions to be Bias-Corrected are merged
blank.BC <- merge(north.pacific.blank.raster,
                  central.pacific.blank.raster,
                  south.pacific.blank.raster)

# A simple plot is requested
#plot(blank.BC)

# Climatic-regions blank-maps are created
caribbean.blank.raster <- blank.raster
north.blank.raster <- blank.raster
central.blank.raster <- blank.raster


# Climatic-regions to be left RAW are isolated
values(caribbean.blank.raster)[values(caribbean.blank.raster) != 6] <- NA
values(north.blank.raster)[values(north.blank.raster) != 1] <- NA
values(central.blank.raster)[values(central.blank.raster) != 4] <- NA


# A simple plot is requested
#plot(caribbean.blank.raster)
#plot(north.blank.raster)

# Climatic-regions to be to be left RAW are merged
blank.RAW <- merge(caribbean.blank.raster,
                   north.blank.raster,
                   central.blank.raster)

# A simple plot is requested
#plot(blank.RAW)

#----------------------------------------------------------------------------------------------------------------
# RCP26. Rasterbrick objects are defenetively merged
#----------------------------------------------------------------------------------------------------------------
test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2011_2040_DEC, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2011_2040_DEC, blank.BC)
DEF_ENSEMBLE_rcp26_2011_2040_DEC <- merge(test.RAW, test.BC, overlap=TRUE)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2011_2040_JAN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2011_2040_JAN, blank.BC)
DEF_ENSEMBLE_rcp26_2011_2040_JAN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2011_2040_FEB, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2011_2040_FEB, blank.BC)
DEF_ENSEMBLE_rcp26_2011_2040_FEB <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2041_2070_DEC, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2041_2070_DEC, blank.BC)
DEF_ENSEMBLE_rcp26_2041_2070_DEC <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2041_2070_JAN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2041_2070_JAN, blank.BC)
DEF_ENSEMBLE_rcp26_2041_2070_JAN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2041_2070_FEB, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2041_2070_FEB, blank.BC)
DEF_ENSEMBLE_rcp26_2041_2070_FEB <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2071_2100_DEC, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2071_2100_DEC, blank.BC)
DEF_ENSEMBLE_rcp26_2071_2100_DEC <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2071_2100_JAN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2071_2100_JAN, blank.BC)
DEF_ENSEMBLE_rcp26_2071_2100_JAN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2071_2100_FEB, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2071_2100_FEB, blank.BC)
DEF_ENSEMBLE_rcp26_2071_2100_FEB <- merge(test.RAW, test.BC)
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP45. Rasterbrick objects are defenetively merged
#----------------------------------------------------------------------------------------------------------------
test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2011_2040_DEC, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2011_2040_DEC, blank.BC)
DEF_ENSEMBLE_rcp45_2011_2040_DEC <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2011_2040_JAN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2011_2040_JAN, blank.BC)
DEF_ENSEMBLE_rcp45_2011_2040_JAN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2011_2040_FEB, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2011_2040_FEB, blank.BC)
DEF_ENSEMBLE_rcp45_2011_2040_FEB <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2041_2070_DEC, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2041_2070_DEC, blank.BC)
DEF_ENSEMBLE_rcp45_2041_2070_DEC <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2041_2070_JAN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2041_2070_JAN, blank.BC)
DEF_ENSEMBLE_rcp45_2041_2070_JAN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2041_2070_FEB, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2041_2070_FEB, blank.BC)
DEF_ENSEMBLE_rcp45_2041_2070_FEB <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2071_2100_DEC, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2071_2100_DEC, blank.BC)
DEF_ENSEMBLE_rcp45_2071_2100_DEC <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2071_2100_JAN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2071_2100_JAN, blank.BC)
DEF_ENSEMBLE_rcp45_2071_2100_JAN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2071_2100_FEB, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2071_2100_FEB, blank.BC)
DEF_ENSEMBLE_rcp45_2071_2100_FEB <- merge(test.RAW, test.BC)
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP85. Rasterbrick objects are defenetively merged
#----------------------------------------------------------------------------------------------------------------
test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2011_2040_DEC, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2011_2040_DEC, blank.BC)
DEF_ENSEMBLE_rcp85_2011_2040_DEC <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2011_2040_JAN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2011_2040_JAN, blank.BC)
DEF_ENSEMBLE_rcp85_2011_2040_JAN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2011_2040_FEB, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2011_2040_FEB, blank.BC)
DEF_ENSEMBLE_rcp85_2011_2040_FEB <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2041_2070_DEC, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2041_2070_DEC, blank.BC)
DEF_ENSEMBLE_rcp85_2041_2070_DEC <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2041_2070_JAN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2041_2070_JAN, blank.BC)
DEF_ENSEMBLE_rcp85_2041_2070_JAN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2041_2070_FEB, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2041_2070_FEB, blank.BC)
DEF_ENSEMBLE_rcp85_2041_2070_FEB <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2071_2100_DEC, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2071_2100_DEC, blank.BC)
DEF_ENSEMBLE_rcp85_2071_2100_DEC <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2071_2100_JAN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2071_2100_JAN, blank.BC)
DEF_ENSEMBLE_rcp85_2071_2100_JAN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2071_2100_FEB, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2071_2100_FEB, blank.BC)
DEF_ENSEMBLE_rcp85_2071_2100_FEB <- merge(test.RAW, test.BC)
#----------------------------------------------------------------------------------------------------------------

# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: ENSAMBLE SEASON MAM
# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# DJF i= 12, 1, 2
# MAM i= 3, 4, 5
# JJA i= 6, 7, 8
# SON i= 9, 10, 11

#----------------------------------------------------------------------------------------------------------------
# RCP26. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPraw_ENSEMBLE_rcp26_2011_2040_MAR <- stack("Mean_2011_2040_DELTA_BC_rcp26_MAR.tif")
TEMPraw_ENSEMBLE_rcp26_2041_2070_MAR <- stack("Mean_2041_2070_DELTA_BC_rcp26_MAR.tif")
TEMPraw_ENSEMBLE_rcp26_2071_2100_MAR <- stack("Mean_2071_2100_DELTA_BC_rcp26_MAR.tif")

TEMPraw_ENSEMBLE_rcp26_2011_2040_APR <- stack("Mean_2011_2040_DELTA_BC_rcp26_APR.tif")
TEMPraw_ENSEMBLE_rcp26_2041_2070_APR <- stack("Mean_2041_2070_DELTA_BC_rcp26_APR.tif")
TEMPraw_ENSEMBLE_rcp26_2071_2100_APR <- stack("Mean_2071_2100_DELTA_BC_rcp26_APR.tif")

TEMPraw_ENSEMBLE_rcp26_2011_2040_MAY <- stack("Mean_2011_2040_DELTA_BC_rcp26_MAY.tif")
TEMPraw_ENSEMBLE_rcp26_2041_2070_MAY <- stack("Mean_2041_2070_DELTA_BC_rcp26_MAY.tif")
TEMPraw_ENSEMBLE_rcp26_2071_2100_MAY <- stack("Mean_2071_2100_DELTA_BC_rcp26_MAY.tif")

crs(TEMPraw_ENSEMBLE_rcp26_2011_2040_MAR) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2041_2070_MAR) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2071_2100_MAR) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp26_2011_2040_APR) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2041_2070_APR) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2071_2100_APR) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp26_2011_2040_MAY) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2041_2070_MAY) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2071_2100_MAY) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP45. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPraw_ENSEMBLE_rcp45_2011_2040_MAR <- stack("Mean_2011_2040_DELTA_BC_rcp45_MAR.tif")
TEMPraw_ENSEMBLE_rcp45_2041_2070_MAR <- stack("Mean_2041_2070_DELTA_BC_rcp45_MAR.tif")
TEMPraw_ENSEMBLE_rcp45_2071_2100_MAR <- stack("Mean_2071_2100_DELTA_BC_rcp45_MAR.tif")

TEMPraw_ENSEMBLE_rcp45_2011_2040_APR <- stack("Mean_2011_2040_DELTA_BC_rcp45_APR.tif")
TEMPraw_ENSEMBLE_rcp45_2041_2070_APR <- stack("Mean_2041_2070_DELTA_BC_rcp45_APR.tif")
TEMPraw_ENSEMBLE_rcp45_2071_2100_APR <- stack("Mean_2071_2100_DELTA_BC_rcp45_APR.tif")

TEMPraw_ENSEMBLE_rcp45_2011_2040_MAY <- stack("Mean_2011_2040_DELTA_BC_rcp45_MAY.tif")
TEMPraw_ENSEMBLE_rcp45_2041_2070_MAY <- stack("Mean_2041_2070_DELTA_BC_rcp45_MAY.tif")
TEMPraw_ENSEMBLE_rcp45_2071_2100_MAY <- stack("Mean_2071_2100_DELTA_BC_rcp45_MAY.tif")

crs(TEMPraw_ENSEMBLE_rcp45_2011_2040_MAR) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2041_2070_MAR) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2071_2100_MAR) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp45_2011_2040_APR) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2041_2070_APR) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2071_2100_APR) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp45_2011_2040_MAY) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2041_2070_MAY) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2071_2100_MAY) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP85. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPraw_ENSEMBLE_rcp85_2011_2040_MAR <- stack("Mean_2011_2040_DELTA_BC_rcp85_MAR.tif")
TEMPraw_ENSEMBLE_rcp85_2041_2070_MAR <- stack("Mean_2041_2070_DELTA_BC_rcp85_MAR.tif")
TEMPraw_ENSEMBLE_rcp85_2071_2100_MAR <- stack("Mean_2071_2100_DELTA_BC_rcp85_MAR.tif")

TEMPraw_ENSEMBLE_rcp85_2011_2040_APR <- stack("Mean_2011_2040_DELTA_BC_rcp85_APR.tif")
TEMPraw_ENSEMBLE_rcp85_2041_2070_APR <- stack("Mean_2041_2070_DELTA_BC_rcp85_APR.tif")
TEMPraw_ENSEMBLE_rcp85_2071_2100_APR <- stack("Mean_2071_2100_DELTA_BC_rcp85_APR.tif")

TEMPraw_ENSEMBLE_rcp85_2011_2040_MAY <- stack("Mean_2011_2040_DELTA_BC_rcp85_MAY.tif")
TEMPraw_ENSEMBLE_rcp85_2041_2070_MAY <- stack("Mean_2041_2070_DELTA_BC_rcp85_MAY.tif")
TEMPraw_ENSEMBLE_rcp85_2071_2100_MAY <- stack("Mean_2071_2100_DELTA_BC_rcp85_MAY.tif")

crs(TEMPraw_ENSEMBLE_rcp85_2011_2040_MAR) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2041_2070_MAR) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2071_2100_MAR) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp85_2011_2040_APR) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2041_2070_APR) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2071_2100_APR) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp85_2011_2040_MAY) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2041_2070_MAY) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2071_2100_MAY) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP26. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPbc_ENSEMBLE_rcp26_2011_2040_MAR <- stack("Mean_2011_2040_EQM_BC_rcp26_MAR.tif")
TEMPbc_ENSEMBLE_rcp26_2041_2070_MAR <- stack("Mean_2041_2070_EQM_BC_rcp26_MAR.tif")
TEMPbc_ENSEMBLE_rcp26_2071_2100_MAR <- stack("Mean_2071_2100_EQM_BC_rcp26_MAR.tif")

TEMPbc_ENSEMBLE_rcp26_2011_2040_APR <- stack("Mean_2011_2040_EQM_BC_rcp26_APR.tif")
TEMPbc_ENSEMBLE_rcp26_2041_2070_APR <- stack("Mean_2041_2070_EQM_BC_rcp26_APR.tif")
TEMPbc_ENSEMBLE_rcp26_2071_2100_APR <- stack("Mean_2071_2100_EQM_BC_rcp26_APR.tif")

TEMPbc_ENSEMBLE_rcp26_2011_2040_MAY <- stack("Mean_2011_2040_EQM_BC_rcp26_MAY.tif")
TEMPbc_ENSEMBLE_rcp26_2041_2070_MAY <- stack("Mean_2041_2070_EQM_BC_rcp26_MAY.tif")
TEMPbc_ENSEMBLE_rcp26_2071_2100_MAY <- stack("Mean_2071_2100_EQM_BC_rcp26_MAY.tif")

crs(TEMPbc_ENSEMBLE_rcp26_2011_2040_MAR) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2041_2070_MAR) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2071_2100_MAR) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp26_2011_2040_APR) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2041_2070_APR) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2071_2100_APR) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp26_2011_2040_MAY) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2041_2070_MAY) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2071_2100_MAY) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP45. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPbc_ENSEMBLE_rcp45_2011_2040_MAR <- stack("Mean_2011_2040_EQM_BC_rcp45_MAR.tif")
TEMPbc_ENSEMBLE_rcp45_2041_2070_MAR <- stack("Mean_2041_2070_EQM_BC_rcp45_MAR.tif")
TEMPbc_ENSEMBLE_rcp45_2071_2100_MAR <- stack("Mean_2071_2100_EQM_BC_rcp45_MAR.tif")

TEMPbc_ENSEMBLE_rcp45_2011_2040_APR <- stack("Mean_2011_2040_EQM_BC_rcp45_APR.tif")
TEMPbc_ENSEMBLE_rcp45_2041_2070_APR <- stack("Mean_2041_2070_EQM_BC_rcp45_APR.tif")
TEMPbc_ENSEMBLE_rcp45_2071_2100_APR <- stack("Mean_2071_2100_EQM_BC_rcp45_APR.tif")

TEMPbc_ENSEMBLE_rcp45_2011_2040_MAY <- stack("Mean_2011_2040_EQM_BC_rcp45_MAY.tif")
TEMPbc_ENSEMBLE_rcp45_2041_2070_MAY <- stack("Mean_2041_2070_EQM_BC_rcp45_MAY.tif")
TEMPbc_ENSEMBLE_rcp45_2071_2100_MAY <- stack("Mean_2071_2100_EQM_BC_rcp45_MAY.tif")

crs(TEMPbc_ENSEMBLE_rcp45_2011_2040_MAR) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2041_2070_MAR) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2071_2100_MAR) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp45_2011_2040_APR) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2041_2070_APR) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2071_2100_APR) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp45_2011_2040_MAY) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2041_2070_MAY) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2071_2100_MAY) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP85. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPbc_ENSEMBLE_rcp85_2011_2040_MAR <- stack("Mean_2011_2040_EQM_BC_rcp85_MAR.tif")
TEMPbc_ENSEMBLE_rcp85_2041_2070_MAR <- stack("Mean_2041_2070_EQM_BC_rcp85_MAR.tif")
TEMPbc_ENSEMBLE_rcp85_2071_2100_MAR <- stack("Mean_2071_2100_EQM_BC_rcp85_MAR.tif")

TEMPbc_ENSEMBLE_rcp85_2011_2040_APR <- stack("Mean_2011_2040_EQM_BC_rcp85_APR.tif")
TEMPbc_ENSEMBLE_rcp85_2041_2070_APR <- stack("Mean_2041_2070_EQM_BC_rcp85_APR.tif")
TEMPbc_ENSEMBLE_rcp85_2071_2100_APR <- stack("Mean_2071_2100_EQM_BC_rcp85_APR.tif")

TEMPbc_ENSEMBLE_rcp85_2011_2040_MAY <- stack("Mean_2011_2040_EQM_BC_rcp85_MAY.tif")
TEMPbc_ENSEMBLE_rcp85_2041_2070_MAY <- stack("Mean_2041_2070_EQM_BC_rcp85_MAY.tif")
TEMPbc_ENSEMBLE_rcp85_2071_2100_MAY <- stack("Mean_2071_2100_EQM_BC_rcp85_MAY.tif")

crs(TEMPbc_ENSEMBLE_rcp85_2011_2040_MAR) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2041_2070_MAR) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2071_2100_MAR) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp85_2011_2040_APR) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2041_2070_APR) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2071_2100_APR) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp85_2011_2040_MAY) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2041_2070_MAY) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2071_2100_MAY) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# SUBBLOCK: ASC CR Climatic Regions Masking
#----------------------------------------------------------------------------------------------------------------
# North = 1
# Central_Pacific = 2 +++
# North_Pacific = 3 +++
# Central = 4
# South_Pacific = 5
# Caribbean = 6 +++
#----------------------------------------------------------------------------------------------------------------
CR <- c(1,2,3,4,5,6)
#----------------------------------------------------------------------------------------------------------------

# Climatic-regions CR blank raster is loaded
blank <- paste("mod_region.asc", sep = "/")

# The blank map is transformed into SpatialGridDataFrame
blank.grid <- read.asciigrid(blank, as.image = FALSE)

# The map is transformed into RasterLayer
blank.raster <- raster(blank.grid)

# CRTM05 projection is assigned to rasterbrick object
crs(blank.raster) <- CRTM05

# A simple plot is requested
#plot(blank.raster)

# Climatic-regions blank-maps are created
north.pacific.blank.raster <- blank.raster
central.pacific.blank.raster <- blank.raster
central.blank.raster <- blank.raster
south.pacific.blank.raster <- blank.raster

# Climatic-regions to be Bias-Corrected are isolated
values(north.pacific.blank.raster)[values(north.pacific.blank.raster) != 3] <- NA
values(central.pacific.blank.raster)[values(central.pacific.blank.raster) != 2] <- NA
values(central.blank.raster)[values(central.blank.raster) != 4] <- NA
values(south.pacific.blank.raster)[values(south.pacific.blank.raster) != 5] <- NA

# A simple plot is requested
#plot(north.pacific.blank.raster)
#plot(central.pacific.blank.raster)
#plot(central.blank.raster)
#plot(south.pacific.blank.raster)

# Climatic-regions to be Bias-Corrected are merged
blank.BC <- merge(north.pacific.blank.raster,
                  central.pacific.blank.raster,
                  central.blank.raster,
                  south.pacific.blank.raster)

# A simple plot is requested
#plot(blank.BC)

# Climatic-regions blank-maps are created
caribbean.blank.raster <- blank.raster
north.blank.raster <- blank.raster

# Climatic-regions to be left RAW are isolated
values(caribbean.blank.raster)[values(caribbean.blank.raster) != 6] <- NA
values(north.blank.raster)[values(north.blank.raster) != 1] <- NA

# A simple plot is requested
#plot(caribbean.blank.raster)
#plot(north.blank.raster)

# Climatic-regions to be to be left RAW are merged
blank.RAW <- merge(caribbean.blank.raster,
                   north.blank.raster)

# A simple plot is requested
#plot(blank.RAW)

#----------------------------------------------------------------------------------------------------------------
# RCP26. Rasterbrick objects are defenetively merged
#----------------------------------------------------------------------------------------------------------------
test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2011_2040_MAR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2011_2040_MAR, blank.BC)
DEF_ENSEMBLE_rcp26_2011_2040_MAR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2011_2040_APR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2011_2040_APR, blank.BC)
DEF_ENSEMBLE_rcp26_2011_2040_APR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2011_2040_MAY, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2011_2040_MAY, blank.BC)
DEF_ENSEMBLE_rcp26_2011_2040_MAY <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2041_2070_MAR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2041_2070_MAR, blank.BC)
DEF_ENSEMBLE_rcp26_2041_2070_MAR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2041_2070_APR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2041_2070_APR, blank.BC)
DEF_ENSEMBLE_rcp26_2041_2070_APR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2041_2070_MAY, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2041_2070_MAY, blank.BC)
DEF_ENSEMBLE_rcp26_2041_2070_MAY <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2071_2100_MAR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2071_2100_MAR, blank.BC)
DEF_ENSEMBLE_rcp26_2071_2100_MAR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2071_2100_APR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2071_2100_APR, blank.BC)
DEF_ENSEMBLE_rcp26_2071_2100_APR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2071_2100_MAY, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2071_2100_MAY, blank.BC)
DEF_ENSEMBLE_rcp26_2071_2100_MAY <- merge(test.RAW, test.BC)
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP45. Rasterbrick objects are defenetively merged
#----------------------------------------------------------------------------------------------------------------
test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2011_2040_MAR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2011_2040_MAR, blank.BC)
DEF_ENSEMBLE_rcp45_2011_2040_MAR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2011_2040_APR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2011_2040_APR, blank.BC)
DEF_ENSEMBLE_rcp45_2011_2040_APR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2011_2040_MAY, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2011_2040_MAY, blank.BC)
DEF_ENSEMBLE_rcp45_2011_2040_MAY <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2041_2070_MAR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2041_2070_MAR, blank.BC)
DEF_ENSEMBLE_rcp45_2041_2070_MAR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2041_2070_APR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2041_2070_APR, blank.BC)
DEF_ENSEMBLE_rcp45_2041_2070_APR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2041_2070_MAY, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2041_2070_MAY, blank.BC)
DEF_ENSEMBLE_rcp45_2041_2070_MAY <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2071_2100_MAR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2071_2100_MAR, blank.BC)
DEF_ENSEMBLE_rcp45_2071_2100_MAR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2071_2100_APR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2071_2100_APR, blank.BC)
DEF_ENSEMBLE_rcp45_2071_2100_APR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2071_2100_MAY, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2071_2100_MAY, blank.BC)
DEF_ENSEMBLE_rcp45_2071_2100_MAY <- merge(test.RAW, test.BC)
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP85. Rasterbrick objects are defenetively merged
#----------------------------------------------------------------------------------------------------------------
test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2011_2040_MAR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2011_2040_MAR, blank.BC)
DEF_ENSEMBLE_rcp85_2011_2040_MAR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2011_2040_APR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2011_2040_APR, blank.BC)
DEF_ENSEMBLE_rcp85_2011_2040_APR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2011_2040_MAY, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2011_2040_MAY, blank.BC)
DEF_ENSEMBLE_rcp85_2011_2040_MAY <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2041_2070_MAR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2041_2070_MAR, blank.BC)
DEF_ENSEMBLE_rcp85_2041_2070_MAR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2041_2070_APR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2041_2070_APR, blank.BC)
DEF_ENSEMBLE_rcp85_2041_2070_APR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2041_2070_MAY, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2041_2070_MAY, blank.BC)
DEF_ENSEMBLE_rcp85_2041_2070_MAY <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2071_2100_MAR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2071_2100_MAR, blank.BC)
DEF_ENSEMBLE_rcp85_2071_2100_MAR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2071_2100_APR, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2071_2100_APR, blank.BC)
DEF_ENSEMBLE_rcp85_2071_2100_APR <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2071_2100_MAY, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2071_2100_MAY, blank.BC)
DEF_ENSEMBLE_rcp85_2071_2100_MAY <- merge(test.RAW, test.BC)
#----------------------------------------------------------------------------------------------------------------

# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: ENSAMBLE SEASON JJA
# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# DJF i= 12, 1, 2
# MAM i= 3, 4, 5
# JJA i= 6, 7, 8
# SON i= 9, 10, 11

#----------------------------------------------------------------------------------------------------------------
# RCP26. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPraw_ENSEMBLE_rcp26_2011_2040_JUN <- stack("Mean_2011_2040_DELTA_BC_rcp26_JUN.tif")
TEMPraw_ENSEMBLE_rcp26_2041_2070_JUN <- stack("Mean_2041_2070_DELTA_BC_rcp26_JUN.tif")
TEMPraw_ENSEMBLE_rcp26_2071_2100_JUN <- stack("Mean_2071_2100_DELTA_BC_rcp26_JUN.tif")

TEMPraw_ENSEMBLE_rcp26_2011_2040_JUL <- stack("Mean_2011_2040_DELTA_BC_rcp26_JUL.tif")
TEMPraw_ENSEMBLE_rcp26_2041_2070_JUL <- stack("Mean_2041_2070_DELTA_BC_rcp26_JUL.tif")
TEMPraw_ENSEMBLE_rcp26_2071_2100_JUL <- stack("Mean_2071_2100_DELTA_BC_rcp26_JUL.tif")

TEMPraw_ENSEMBLE_rcp26_2011_2040_AUG <- stack("Mean_2011_2040_DELTA_BC_rcp26_AUG.tif")
TEMPraw_ENSEMBLE_rcp26_2041_2070_AUG <- stack("Mean_2041_2070_DELTA_BC_rcp26_AUG.tif")
TEMPraw_ENSEMBLE_rcp26_2071_2100_AUG <- stack("Mean_2071_2100_DELTA_BC_rcp26_AUG.tif")

crs(TEMPraw_ENSEMBLE_rcp26_2011_2040_JUN) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2041_2070_JUN) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2071_2100_JUN) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp26_2011_2040_JUL) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2041_2070_JUL) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2071_2100_JUL) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp26_2011_2040_AUG) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2041_2070_AUG) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2071_2100_AUG) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP45. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPraw_ENSEMBLE_rcp45_2011_2040_JUN <- stack("Mean_2011_2040_DELTA_BC_rcp45_JUN.tif")
TEMPraw_ENSEMBLE_rcp45_2041_2070_JUN <- stack("Mean_2041_2070_DELTA_BC_rcp45_JUN.tif")
TEMPraw_ENSEMBLE_rcp45_2071_2100_JUN <- stack("Mean_2071_2100_DELTA_BC_rcp45_JUN.tif")

TEMPraw_ENSEMBLE_rcp45_2011_2040_JUL <- stack("Mean_2011_2040_DELTA_BC_rcp45_JUL.tif")
TEMPraw_ENSEMBLE_rcp45_2041_2070_JUL <- stack("Mean_2041_2070_DELTA_BC_rcp45_JUL.tif")
TEMPraw_ENSEMBLE_rcp45_2071_2100_JUL <- stack("Mean_2071_2100_DELTA_BC_rcp45_JUL.tif")

TEMPraw_ENSEMBLE_rcp45_2011_2040_AUG <- stack("Mean_2011_2040_DELTA_BC_rcp45_AUG.tif")
TEMPraw_ENSEMBLE_rcp45_2041_2070_AUG <- stack("Mean_2041_2070_DELTA_BC_rcp45_AUG.tif")
TEMPraw_ENSEMBLE_rcp45_2071_2100_AUG <- stack("Mean_2071_2100_DELTA_BC_rcp45_AUG.tif")

crs(TEMPraw_ENSEMBLE_rcp45_2011_2040_JUN) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2041_2070_JUN) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2071_2100_JUN) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp45_2011_2040_JUL) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2041_2070_JUL) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2071_2100_JUL) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp45_2011_2040_AUG) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2041_2070_AUG) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2071_2100_AUG) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP85. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPraw_ENSEMBLE_rcp85_2011_2040_JUN <- stack("Mean_2011_2040_DELTA_BC_rcp85_JUN.tif")
TEMPraw_ENSEMBLE_rcp85_2041_2070_JUN <- stack("Mean_2041_2070_DELTA_BC_rcp85_JUN.tif")
TEMPraw_ENSEMBLE_rcp85_2071_2100_JUN <- stack("Mean_2071_2100_DELTA_BC_rcp85_JUN.tif")

TEMPraw_ENSEMBLE_rcp85_2011_2040_JUL <- stack("Mean_2011_2040_DELTA_BC_rcp85_JUL.tif")
TEMPraw_ENSEMBLE_rcp85_2041_2070_JUL <- stack("Mean_2041_2070_DELTA_BC_rcp85_JUL.tif")
TEMPraw_ENSEMBLE_rcp85_2071_2100_JUL <- stack("Mean_2071_2100_DELTA_BC_rcp85_JUL.tif")

TEMPraw_ENSEMBLE_rcp85_2011_2040_AUG <- stack("Mean_2011_2040_DELTA_BC_rcp85_AUG.tif")
TEMPraw_ENSEMBLE_rcp85_2041_2070_AUG <- stack("Mean_2041_2070_DELTA_BC_rcp85_AUG.tif")
TEMPraw_ENSEMBLE_rcp85_2071_2100_AUG <- stack("Mean_2071_2100_DELTA_BC_rcp85_AUG.tif")

crs(TEMPraw_ENSEMBLE_rcp85_2011_2040_JUN) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2041_2070_JUN) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2071_2100_JUN) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp85_2011_2040_JUL) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2041_2070_JUL) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2071_2100_JUL) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp85_2011_2040_AUG) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2041_2070_AUG) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2071_2100_AUG) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP26. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPbc_ENSEMBLE_rcp26_2011_2040_JUN <- stack("Mean_2011_2040_EQM_BC_rcp26_JUN.tif")
TEMPbc_ENSEMBLE_rcp26_2041_2070_JUN <- stack("Mean_2041_2070_EQM_BC_rcp26_JUN.tif")
TEMPbc_ENSEMBLE_rcp26_2071_2100_JUN <- stack("Mean_2071_2100_EQM_BC_rcp26_JUN.tif")

TEMPbc_ENSEMBLE_rcp26_2011_2040_JUL <- stack("Mean_2011_2040_EQM_BC_rcp26_JUL.tif")
TEMPbc_ENSEMBLE_rcp26_2041_2070_JUL <- stack("Mean_2041_2070_EQM_BC_rcp26_JUL.tif")
TEMPbc_ENSEMBLE_rcp26_2071_2100_JUL <- stack("Mean_2071_2100_EQM_BC_rcp26_JUL.tif")

TEMPbc_ENSEMBLE_rcp26_2011_2040_AUG <- stack("Mean_2011_2040_EQM_BC_rcp26_AUG.tif")
TEMPbc_ENSEMBLE_rcp26_2041_2070_AUG <- stack("Mean_2041_2070_EQM_BC_rcp26_AUG.tif")
TEMPbc_ENSEMBLE_rcp26_2071_2100_AUG <- stack("Mean_2071_2100_EQM_BC_rcp26_AUG.tif")

crs(TEMPbc_ENSEMBLE_rcp26_2011_2040_JUN) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2041_2070_JUN) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2071_2100_JUN) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp26_2011_2040_JUL) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2041_2070_JUL) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2071_2100_JUL) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp26_2011_2040_AUG) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2041_2070_AUG) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2071_2100_AUG) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP45. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPbc_ENSEMBLE_rcp45_2011_2040_JUN <- stack("Mean_2011_2040_EQM_BC_rcp45_JUN.tif")
TEMPbc_ENSEMBLE_rcp45_2041_2070_JUN <- stack("Mean_2041_2070_EQM_BC_rcp45_JUN.tif")
TEMPbc_ENSEMBLE_rcp45_2071_2100_JUN <- stack("Mean_2071_2100_EQM_BC_rcp45_JUN.tif")

TEMPbc_ENSEMBLE_rcp45_2011_2040_JUL <- stack("Mean_2011_2040_EQM_BC_rcp45_JUL.tif")
TEMPbc_ENSEMBLE_rcp45_2041_2070_JUL <- stack("Mean_2041_2070_EQM_BC_rcp45_JUL.tif")
TEMPbc_ENSEMBLE_rcp45_2071_2100_JUL <- stack("Mean_2071_2100_EQM_BC_rcp45_JUL.tif")

TEMPbc_ENSEMBLE_rcp45_2011_2040_AUG <- stack("Mean_2011_2040_EQM_BC_rcp45_AUG.tif")
TEMPbc_ENSEMBLE_rcp45_2041_2070_AUG <- stack("Mean_2041_2070_EQM_BC_rcp45_AUG.tif")
TEMPbc_ENSEMBLE_rcp45_2071_2100_AUG <- stack("Mean_2071_2100_EQM_BC_rcp45_AUG.tif")

crs(TEMPbc_ENSEMBLE_rcp45_2011_2040_JUN) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2041_2070_JUN) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2071_2100_JUN) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp45_2011_2040_JUL) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2041_2070_JUL) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2071_2100_JUL) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp45_2011_2040_AUG) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2041_2070_AUG) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2071_2100_AUG) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP85. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPbc_ENSEMBLE_rcp85_2011_2040_JUN <- stack("Mean_2011_2040_EQM_BC_rcp85_JUN.tif")
TEMPbc_ENSEMBLE_rcp85_2041_2070_JUN <- stack("Mean_2041_2070_EQM_BC_rcp85_JUN.tif")
TEMPbc_ENSEMBLE_rcp85_2071_2100_JUN <- stack("Mean_2071_2100_EQM_BC_rcp85_JUN.tif")

TEMPbc_ENSEMBLE_rcp85_2011_2040_JUL <- stack("Mean_2011_2040_EQM_BC_rcp85_JUL.tif")
TEMPbc_ENSEMBLE_rcp85_2041_2070_JUL <- stack("Mean_2041_2070_EQM_BC_rcp85_JUL.tif")
TEMPbc_ENSEMBLE_rcp85_2071_2100_JUL <- stack("Mean_2071_2100_EQM_BC_rcp85_JUL.tif")

TEMPbc_ENSEMBLE_rcp85_2011_2040_AUG <- stack("Mean_2011_2040_EQM_BC_rcp85_AUG.tif")
TEMPbc_ENSEMBLE_rcp85_2041_2070_AUG <- stack("Mean_2041_2070_EQM_BC_rcp85_AUG.tif")
TEMPbc_ENSEMBLE_rcp85_2071_2100_AUG <- stack("Mean_2071_2100_EQM_BC_rcp85_AUG.tif")

crs(TEMPbc_ENSEMBLE_rcp85_2011_2040_JUN) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2041_2070_JUN) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2071_2100_JUN) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp85_2011_2040_JUL) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2041_2070_JUL) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2071_2100_JUL) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp85_2011_2040_AUG) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2041_2070_AUG) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2071_2100_AUG) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# SUBBLOCK: ASC CR Climatic Regions Masking
#----------------------------------------------------------------------------------------------------------------
# North = 1
# Central_Pacific = 2 +++
# North_Pacific = 3 +++
# Central = 4
# South_Pacific = 5
# Caribbean = 6 +++
#----------------------------------------------------------------------------------------------------------------
CR <- c(1,2,3,4,5,6)
#----------------------------------------------------------------------------------------------------------------

# Climatic-regions CR blank raster is loaded
blank <- paste("mod_region.asc", sep = "/")

# The blank map is transformed into SpatialGridDataFrame
blank.grid <- read.asciigrid(blank, as.image = FALSE)

# The map is transformed into RasterLayer
blank.raster <- raster(blank.grid)

# CRTM05 projection is assigned to rasterbrick object
crs(blank.raster) <- CRTM05

# A simple plot is requested
#plot(blank.raster)

# Climatic-regions blank-maps are created
north.pacific.blank.raster <- blank.raster
central.pacific.blank.raster <- blank.raster
central.blank.raster <- blank.raster
south.pacific.blank.raster <- blank.raster
north.blank.raster <- blank.raster

# Climatic-regions to be Bias-Corrected are isolated
values(north.pacific.blank.raster)[values(north.pacific.blank.raster) != 3] <- NA
values(central.pacific.blank.raster)[values(central.pacific.blank.raster) != 2] <- NA
values(central.blank.raster)[values(central.blank.raster) != 4] <- NA
values(south.pacific.blank.raster)[values(south.pacific.blank.raster) != 5] <- NA
values(north.blank.raster)[values(north.blank.raster) != 1] <- NA

# A simple plot is requested
#plot(north.pacific.blank.raster)
#plot(central.pacific.blank.raster)
#plot(central.blank.raster)
#plot(south.pacific.blank.raster)

# Climatic-regions to be Bias-Corrected are merged
blank.BC <- merge(north.pacific.blank.raster,
                  central.pacific.blank.raster,
                  central.blank.raster,
                  south.pacific.blank.raster,
                  north.blank.raster)

# A simple plot is requested
#plot(blank.BC)

# Climatic-regions blank-maps are created
caribbean.blank.raster <- blank.raster

# Climatic-regions to be left RAW are isolated
values(caribbean.blank.raster)[values(caribbean.blank.raster) != 6] <- NA

# A simple plot is requested
#plot(caribbean.blank.raster)
#plot(north.blank.raster)

# Climatic-regions to be to be left RAW are merged
blank.RAW <- (caribbean.blank.raster)

# A simple plot is requested
#plot(blank.RAW)

#----------------------------------------------------------------------------------------------------------------
# RCP26. Rasterbrick objects are defenetively merged
#----------------------------------------------------------------------------------------------------------------
test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2011_2040_JUN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2011_2040_JUN, blank.BC)
DEF_ENSEMBLE_rcp26_2011_2040_JUN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2011_2040_JUL, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2011_2040_JUL, blank.BC)
DEF_ENSEMBLE_rcp26_2011_2040_JUL <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2011_2040_AUG, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2011_2040_AUG, blank.BC)
DEF_ENSEMBLE_rcp26_2011_2040_AUG <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2041_2070_JUN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2041_2070_JUN, blank.BC)
DEF_ENSEMBLE_rcp26_2041_2070_JUN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2041_2070_JUL, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2041_2070_JUL, blank.BC)
DEF_ENSEMBLE_rcp26_2041_2070_JUL <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2041_2070_AUG, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2041_2070_AUG, blank.BC)
DEF_ENSEMBLE_rcp26_2041_2070_AUG <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2071_2100_JUN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2071_2100_JUN, blank.BC)
DEF_ENSEMBLE_rcp26_2071_2100_JUN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2071_2100_JUL, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2071_2100_JUL, blank.BC)
DEF_ENSEMBLE_rcp26_2071_2100_JUL <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2071_2100_AUG, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2071_2100_AUG, blank.BC)
DEF_ENSEMBLE_rcp26_2071_2100_AUG <- merge(test.RAW, test.BC)
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP45. Rasterbrick objects are defenetively merged
#----------------------------------------------------------------------------------------------------------------
test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2011_2040_JUN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2011_2040_JUN, blank.BC)
DEF_ENSEMBLE_rcp45_2011_2040_JUN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2011_2040_JUL, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2011_2040_JUL, blank.BC)
DEF_ENSEMBLE_rcp45_2011_2040_JUL <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2011_2040_AUG, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2011_2040_AUG, blank.BC)
DEF_ENSEMBLE_rcp45_2011_2040_AUG <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2041_2070_JUN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2041_2070_JUN, blank.BC)
DEF_ENSEMBLE_rcp45_2041_2070_JUN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2041_2070_JUL, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2041_2070_JUL, blank.BC)
DEF_ENSEMBLE_rcp45_2041_2070_JUL <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2041_2070_AUG, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2041_2070_AUG, blank.BC)
DEF_ENSEMBLE_rcp45_2041_2070_AUG <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2071_2100_JUN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2071_2100_JUN, blank.BC)
DEF_ENSEMBLE_rcp45_2071_2100_JUN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2071_2100_JUL, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2071_2100_JUL, blank.BC)
DEF_ENSEMBLE_rcp45_2071_2100_JUL <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2071_2100_AUG, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2071_2100_AUG, blank.BC)
DEF_ENSEMBLE_rcp45_2071_2100_AUG <- merge(test.RAW, test.BC)
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP85. Rasterbrick objects are defenetively merged
#----------------------------------------------------------------------------------------------------------------
test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2011_2040_JUN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2011_2040_JUN, blank.BC)
DEF_ENSEMBLE_rcp85_2011_2040_JUN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2011_2040_JUL, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2011_2040_JUL, blank.BC)
DEF_ENSEMBLE_rcp85_2011_2040_JUL <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2011_2040_AUG, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2011_2040_AUG, blank.BC)
DEF_ENSEMBLE_rcp85_2011_2040_AUG <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2041_2070_JUN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2041_2070_JUN, blank.BC)
DEF_ENSEMBLE_rcp85_2041_2070_JUN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2041_2070_JUL, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2041_2070_JUL, blank.BC)
DEF_ENSEMBLE_rcp85_2041_2070_JUL <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2041_2070_AUG, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2041_2070_AUG, blank.BC)
DEF_ENSEMBLE_rcp85_2041_2070_AUG <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2071_2100_JUN, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2071_2100_JUN, blank.BC)
DEF_ENSEMBLE_rcp85_2071_2100_JUN <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2071_2100_JUL, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2071_2100_JUL, blank.BC)
DEF_ENSEMBLE_rcp85_2071_2100_JUL <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2071_2100_AUG, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2071_2100_AUG, blank.BC)
DEF_ENSEMBLE_rcp85_2071_2100_AUG <- merge(test.RAW, test.BC)
#----------------------------------------------------------------------------------------------------------------

# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: ENSAMBLE SEASON SON
# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# DJF i= 12, 1, 2
# MAM i= 3, 4, 5
# JJA i= 6, 7, 8
# SON i= 9, 10, 11

#----------------------------------------------------------------------------------------------------------------
# RCP26. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPraw_ENSEMBLE_rcp26_2011_2040_SEP <- stack("Mean_2011_2040_DELTA_BC_rcp26_SEP.tif")
TEMPraw_ENSEMBLE_rcp26_2041_2070_SEP <- stack("Mean_2041_2070_DELTA_BC_rcp26_SEP.tif")
TEMPraw_ENSEMBLE_rcp26_2071_2100_SEP <- stack("Mean_2071_2100_DELTA_BC_rcp26_SEP.tif")

TEMPraw_ENSEMBLE_rcp26_2011_2040_OCT <- stack("Mean_2011_2040_DELTA_BC_rcp26_OCT.tif")
TEMPraw_ENSEMBLE_rcp26_2041_2070_OCT <- stack("Mean_2041_2070_DELTA_BC_rcp26_OCT.tif")
TEMPraw_ENSEMBLE_rcp26_2071_2100_OCT <- stack("Mean_2071_2100_DELTA_BC_rcp26_OCT.tif")

TEMPraw_ENSEMBLE_rcp26_2011_2040_NOV <- stack("Mean_2011_2040_DELTA_BC_rcp26_NOV.tif")
TEMPraw_ENSEMBLE_rcp26_2041_2070_NOV <- stack("Mean_2041_2070_DELTA_BC_rcp26_NOV.tif")
TEMPraw_ENSEMBLE_rcp26_2071_2100_NOV <- stack("Mean_2071_2100_DELTA_BC_rcp26_NOV.tif")

crs(TEMPraw_ENSEMBLE_rcp26_2011_2040_SEP) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2041_2070_SEP) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2071_2100_SEP) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp26_2011_2040_OCT) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2041_2070_OCT) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2071_2100_OCT) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp26_2011_2040_NOV) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2041_2070_NOV) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp26_2071_2100_NOV) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP45. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPraw_ENSEMBLE_rcp45_2011_2040_SEP <- stack("Mean_2011_2040_DELTA_BC_rcp45_SEP.tif")
TEMPraw_ENSEMBLE_rcp45_2041_2070_SEP <- stack("Mean_2041_2070_DELTA_BC_rcp45_SEP.tif")
TEMPraw_ENSEMBLE_rcp45_2071_2100_SEP <- stack("Mean_2071_2100_DELTA_BC_rcp45_SEP.tif")

TEMPraw_ENSEMBLE_rcp45_2011_2040_OCT <- stack("Mean_2011_2040_DELTA_BC_rcp45_OCT.tif")
TEMPraw_ENSEMBLE_rcp45_2041_2070_OCT <- stack("Mean_2041_2070_DELTA_BC_rcp45_OCT.tif")
TEMPraw_ENSEMBLE_rcp45_2071_2100_OCT <- stack("Mean_2071_2100_DELTA_BC_rcp45_OCT.tif")

TEMPraw_ENSEMBLE_rcp45_2011_2040_NOV <- stack("Mean_2011_2040_DELTA_BC_rcp45_NOV.tif")
TEMPraw_ENSEMBLE_rcp45_2041_2070_NOV <- stack("Mean_2041_2070_DELTA_BC_rcp45_NOV.tif")
TEMPraw_ENSEMBLE_rcp45_2071_2100_NOV <- stack("Mean_2071_2100_DELTA_BC_rcp45_NOV.tif")

crs(TEMPraw_ENSEMBLE_rcp45_2011_2040_SEP) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2041_2070_SEP) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2071_2100_SEP) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp45_2011_2040_OCT) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2041_2070_OCT) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2071_2100_OCT) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp45_2011_2040_NOV) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2041_2070_NOV) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp45_2071_2100_NOV) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP85. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPraw_ENSEMBLE_rcp85_2011_2040_SEP <- stack("Mean_2011_2040_DELTA_BC_rcp85_SEP.tif")
TEMPraw_ENSEMBLE_rcp85_2041_2070_SEP <- stack("Mean_2041_2070_DELTA_BC_rcp85_SEP.tif")
TEMPraw_ENSEMBLE_rcp85_2071_2100_SEP <- stack("Mean_2071_2100_DELTA_BC_rcp85_SEP.tif")

TEMPraw_ENSEMBLE_rcp85_2011_2040_OCT <- stack("Mean_2011_2040_DELTA_BC_rcp85_OCT.tif")
TEMPraw_ENSEMBLE_rcp85_2041_2070_OCT <- stack("Mean_2041_2070_DELTA_BC_rcp85_OCT.tif")
TEMPraw_ENSEMBLE_rcp85_2071_2100_OCT <- stack("Mean_2071_2100_DELTA_BC_rcp85_OCT.tif")

TEMPraw_ENSEMBLE_rcp85_2011_2040_NOV <- stack("Mean_2011_2040_DELTA_BC_rcp85_NOV.tif")
TEMPraw_ENSEMBLE_rcp85_2041_2070_NOV <- stack("Mean_2041_2070_DELTA_BC_rcp85_NOV.tif")
TEMPraw_ENSEMBLE_rcp85_2071_2100_NOV <- stack("Mean_2071_2100_DELTA_BC_rcp85_NOV.tif")

crs(TEMPraw_ENSEMBLE_rcp85_2011_2040_SEP) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2041_2070_SEP) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2071_2100_SEP) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp85_2011_2040_OCT) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2041_2070_OCT) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2071_2100_OCT) <- CRTM05

crs(TEMPraw_ENSEMBLE_rcp85_2011_2040_NOV) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2041_2070_NOV) <- CRTM05
crs(TEMPraw_ENSEMBLE_rcp85_2071_2100_NOV) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP26. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPbc_ENSEMBLE_rcp26_2011_2040_SEP <- stack("Mean_2011_2040_EQM_BC_rcp26_SEP.tif")
TEMPbc_ENSEMBLE_rcp26_2041_2070_SEP <- stack("Mean_2041_2070_EQM_BC_rcp26_SEP.tif")
TEMPbc_ENSEMBLE_rcp26_2071_2100_SEP <- stack("Mean_2071_2100_EQM_BC_rcp26_SEP.tif")

TEMPbc_ENSEMBLE_rcp26_2011_2040_OCT <- stack("Mean_2011_2040_EQM_BC_rcp26_OCT.tif")
TEMPbc_ENSEMBLE_rcp26_2041_2070_OCT <- stack("Mean_2041_2070_EQM_BC_rcp26_OCT.tif")
TEMPbc_ENSEMBLE_rcp26_2071_2100_OCT <- stack("Mean_2071_2100_EQM_BC_rcp26_OCT.tif")

TEMPbc_ENSEMBLE_rcp26_2011_2040_NOV <- stack("Mean_2011_2040_EQM_BC_rcp26_NOV.tif")
TEMPbc_ENSEMBLE_rcp26_2041_2070_NOV <- stack("Mean_2041_2070_EQM_BC_rcp26_NOV.tif")
TEMPbc_ENSEMBLE_rcp26_2071_2100_NOV <- stack("Mean_2071_2100_EQM_BC_rcp26_NOV.tif")

crs(TEMPbc_ENSEMBLE_rcp26_2011_2040_SEP) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2041_2070_SEP) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2071_2100_SEP) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp26_2011_2040_OCT) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2041_2070_OCT) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2071_2100_OCT) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp26_2011_2040_NOV) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2041_2070_NOV) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp26_2071_2100_NOV) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP45. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPbc_ENSEMBLE_rcp45_2011_2040_SEP <- stack("Mean_2011_2040_EQM_BC_rcp45_SEP.tif")
TEMPbc_ENSEMBLE_rcp45_2041_2070_SEP <- stack("Mean_2041_2070_EQM_BC_rcp45_SEP.tif")
TEMPbc_ENSEMBLE_rcp45_2071_2100_SEP <- stack("Mean_2071_2100_EQM_BC_rcp45_SEP.tif")

TEMPbc_ENSEMBLE_rcp45_2011_2040_OCT <- stack("Mean_2011_2040_EQM_BC_rcp45_OCT.tif")
TEMPbc_ENSEMBLE_rcp45_2041_2070_OCT <- stack("Mean_2041_2070_EQM_BC_rcp45_OCT.tif")
TEMPbc_ENSEMBLE_rcp45_2071_2100_OCT <- stack("Mean_2071_2100_EQM_BC_rcp45_OCT.tif")

TEMPbc_ENSEMBLE_rcp45_2011_2040_NOV <- stack("Mean_2011_2040_EQM_BC_rcp45_NOV.tif")
TEMPbc_ENSEMBLE_rcp45_2041_2070_NOV <- stack("Mean_2041_2070_EQM_BC_rcp45_NOV.tif")
TEMPbc_ENSEMBLE_rcp45_2071_2100_NOV <- stack("Mean_2071_2100_EQM_BC_rcp45_NOV.tif")

crs(TEMPbc_ENSEMBLE_rcp45_2011_2040_SEP) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2041_2070_SEP) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2071_2100_SEP) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp45_2011_2040_OCT) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2041_2070_OCT) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2071_2100_OCT) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp45_2011_2040_NOV) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2041_2070_NOV) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp45_2071_2100_NOV) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP85. Rasterbrick objects are imported from netCDF/GeoTIFF formats
#----------------------------------------------------------------------------------------------------------------
TEMPbc_ENSEMBLE_rcp85_2011_2040_SEP <- stack("Mean_2011_2040_EQM_BC_rcp85_SEP.tif")
TEMPbc_ENSEMBLE_rcp85_2041_2070_SEP <- stack("Mean_2041_2070_EQM_BC_rcp85_SEP.tif")
TEMPbc_ENSEMBLE_rcp85_2071_2100_SEP <- stack("Mean_2071_2100_EQM_BC_rcp85_SEP.tif")

TEMPbc_ENSEMBLE_rcp85_2011_2040_OCT <- stack("Mean_2011_2040_EQM_BC_rcp85_OCT.tif")
TEMPbc_ENSEMBLE_rcp85_2041_2070_OCT <- stack("Mean_2041_2070_EQM_BC_rcp85_OCT.tif")
TEMPbc_ENSEMBLE_rcp85_2071_2100_OCT <- stack("Mean_2071_2100_EQM_BC_rcp85_OCT.tif")

TEMPbc_ENSEMBLE_rcp85_2011_2040_NOV <- stack("Mean_2011_2040_EQM_BC_rcp85_NOV.tif")
TEMPbc_ENSEMBLE_rcp85_2041_2070_NOV <- stack("Mean_2041_2070_EQM_BC_rcp85_NOV.tif")
TEMPbc_ENSEMBLE_rcp85_2071_2100_NOV <- stack("Mean_2071_2100_EQM_BC_rcp85_NOV.tif")

crs(TEMPbc_ENSEMBLE_rcp85_2011_2040_SEP) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2041_2070_SEP) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2071_2100_SEP) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp85_2011_2040_OCT) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2041_2070_OCT) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2071_2100_OCT) <- CRTM05

crs(TEMPbc_ENSEMBLE_rcp85_2011_2040_NOV) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2041_2070_NOV) <- CRTM05
crs(TEMPbc_ENSEMBLE_rcp85_2071_2100_NOV) <- CRTM05
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# SUBBLOCK: ASC CR Climatic Regions Masking
#----------------------------------------------------------------------------------------------------------------
# North = 1
# Central_Pacific = 2 +++
# North_Pacific = 3 +++
# Central = 4
# South_Pacific = 5
# Caribbean = 6 +++
#----------------------------------------------------------------------------------------------------------------
CR <- c(1,2,3,4,5,6)
#----------------------------------------------------------------------------------------------------------------

# Climatic-regions CR blank raster is loaded
blank <- paste("mod_region.asc", sep = "/")

# The blank map is transformed into SpatialGridDataFrame
blank.grid <- read.asciigrid(blank, as.image = FALSE)

# The map is transformed into RasterLayer
blank.raster <- raster(blank.grid)

# CRTM05 projection is assigned to rasterbrick object
crs(blank.raster) <- CRTM05

# A simple plot is requested
#plot(blank.raster)

# Climatic-regions blank-maps are created
north.pacific.blank.raster <- blank.raster
central.pacific.blank.raster <- blank.raster
central.blank.raster <- blank.raster
south.pacific.blank.raster <- blank.raster
north.blank.raster <- blank.raster

# Climatic-regions to be Bias-Corrected are isolated
values(north.pacific.blank.raster)[values(north.pacific.blank.raster) != 3] <- NA
values(central.pacific.blank.raster)[values(central.pacific.blank.raster) != 2] <- NA
values(central.blank.raster)[values(central.blank.raster) != 4] <- NA
values(south.pacific.blank.raster)[values(south.pacific.blank.raster) != 5] <- NA
values(north.blank.raster)[values(north.blank.raster) != 1] <- NA

# A simple plot is requested
#plot(north.pacific.blank.raster)
#plot(central.pacific.blank.raster)
#plot(central.blank.raster)
#plot(south.pacific.blank.raster)

# Climatic-regions to be Bias-Corrected are merged
blank.BC <- merge(north.pacific.blank.raster,
                  central.pacific.blank.raster,
                  central.blank.raster,
                  south.pacific.blank.raster,
                  north.blank.raster)

# A simple plot is requested
#plot(blank.BC)

# Climatic-regions blank-maps are created
caribbean.blank.raster <- blank.raster


# Climatic-regions to be left RAW are isolated
values(caribbean.blank.raster)[values(caribbean.blank.raster) != 6] <- NA

# A simple plot is requested
#plot(caribbean.blank.raster)
#plot(north.blank.raster)

# Climatic-regions to be to be left RAW are merged
blank.RAW <- (caribbean.blank.raster)

# A simple plot is requested
#plot(blank.RAW)

#----------------------------------------------------------------------------------------------------------------
# RCP26. Rasterbrick objects are defenetively merged
#----------------------------------------------------------------------------------------------------------------
test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2011_2040_SEP, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2011_2040_SEP, blank.BC)
DEF_ENSEMBLE_rcp26_2011_2040_SEP <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2011_2040_OCT, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2011_2040_OCT, blank.BC)
DEF_ENSEMBLE_rcp26_2011_2040_OCT <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2011_2040_NOV, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2011_2040_NOV, blank.BC)
DEF_ENSEMBLE_rcp26_2011_2040_NOV <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2041_2070_SEP, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2041_2070_SEP, blank.BC)
DEF_ENSEMBLE_rcp26_2041_2070_SEP <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2041_2070_OCT, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2041_2070_OCT, blank.BC)
DEF_ENSEMBLE_rcp26_2041_2070_OCT <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2041_2070_NOV, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2041_2070_NOV, blank.BC)
DEF_ENSEMBLE_rcp26_2041_2070_NOV <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2071_2100_SEP, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2071_2100_SEP, blank.BC)
DEF_ENSEMBLE_rcp26_2071_2100_SEP <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2071_2100_OCT, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2071_2100_OCT, blank.BC)
DEF_ENSEMBLE_rcp26_2071_2100_OCT <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp26_2071_2100_NOV, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp26_2071_2100_NOV, blank.BC)
DEF_ENSEMBLE_rcp26_2071_2100_NOV <- merge(test.RAW, test.BC)
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP45. Rasterbrick objects are defenetively merged
#----------------------------------------------------------------------------------------------------------------
test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2011_2040_SEP, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2011_2040_SEP, blank.BC)
DEF_ENSEMBLE_rcp45_2011_2040_SEP <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2011_2040_OCT, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2011_2040_OCT, blank.BC)
DEF_ENSEMBLE_rcp45_2011_2040_OCT <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2011_2040_NOV, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2011_2040_NOV, blank.BC)
DEF_ENSEMBLE_rcp45_2011_2040_NOV <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2041_2070_SEP, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2041_2070_SEP, blank.BC)
DEF_ENSEMBLE_rcp45_2041_2070_SEP <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2041_2070_OCT, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2041_2070_OCT, blank.BC)
DEF_ENSEMBLE_rcp45_2041_2070_OCT <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2041_2070_NOV, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2041_2070_NOV, blank.BC)
DEF_ENSEMBLE_rcp45_2041_2070_NOV <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2071_2100_SEP, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2071_2100_SEP, blank.BC)
DEF_ENSEMBLE_rcp45_2071_2100_SEP <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2071_2100_OCT, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2071_2100_OCT, blank.BC)
DEF_ENSEMBLE_rcp45_2071_2100_OCT <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp45_2071_2100_NOV, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp45_2071_2100_NOV, blank.BC)
DEF_ENSEMBLE_rcp45_2071_2100_NOV <- merge(test.RAW, test.BC)
#----------------------------------------------------------------------------------------------------------------

#----------------------------------------------------------------------------------------------------------------
# RCP85. Rasterbrick objects are defenetively merged
#----------------------------------------------------------------------------------------------------------------
test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2011_2040_SEP, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2011_2040_SEP, blank.BC)
DEF_ENSEMBLE_rcp85_2011_2040_SEP <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2011_2040_OCT, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2011_2040_OCT, blank.BC)
DEF_ENSEMBLE_rcp85_2011_2040_OCT <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2011_2040_NOV, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2011_2040_NOV, blank.BC)
DEF_ENSEMBLE_rcp85_2011_2040_NOV <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2041_2070_SEP, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2041_2070_SEP, blank.BC)
DEF_ENSEMBLE_rcp85_2041_2070_SEP <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2041_2070_OCT, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2041_2070_OCT, blank.BC)
DEF_ENSEMBLE_rcp85_2041_2070_OCT <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2041_2070_NOV, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2041_2070_NOV, blank.BC)
DEF_ENSEMBLE_rcp85_2041_2070_NOV <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2071_2100_SEP, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2071_2100_SEP, blank.BC)
DEF_ENSEMBLE_rcp85_2071_2100_SEP <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2071_2100_OCT, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2071_2100_OCT, blank.BC)
DEF_ENSEMBLE_rcp85_2071_2100_OCT <- merge(test.RAW, test.BC)

test.RAW <- mask(TEMPraw_ENSEMBLE_rcp85_2071_2100_NOV, blank.RAW)
test.BC <- mask(TEMPbc_ENSEMBLE_rcp85_2071_2100_NOV, blank.BC)
DEF_ENSEMBLE_rcp85_2071_2100_NOV <- merge(test.RAW, test.BC)
#----------------------------------------------------------------------------------------------------------------

# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Historical 1961-1990 IMN
# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

# Historical 1961-1990 IMN Rasterbrick object is imported from GeoTIFF format
hist_1961_1990_IMN <- stack("JAN_1961_DIC_1990_25x25km.tif") # BEWARE: CRTM05 !!!! This is for IMN !!!

# Monthly variables are loaded
vs.JAN <- c(1,13,25,37,49,61,73,85,97,109,121,133,145,157,169,181,193,205,217,229,241,253,265,277,289,301,313,325,337,349)
vs.FEB <- c(2,14,26,38,50,62,74,86,98,110,122,134,146,158,170,182,194,206,218,230,242,254,266,278,290,302,314,326,338,350)
vs.MAR <- c(3,15,27,39,51,63,75,87,99,111,123,135,147,159,171,183,195,207,219,231,243,255,267,279,291,303,315,327,339,351)
vs.APR <- c(4,16,28,40,52,64,76,88,100,112,124,136,148,160,172,184,196,208,220,232,244,256,268,280,292,304,316,328,340,352)
vs.MAY <- c(5,17,29,41,53,65,77,89,101,113,125,137,149,161,173,185,197,209,221,233,245,257,269,281,293,305,317,329,341,353)
vs.JUN <- c(6,18,30,42,54,66,78,90,102,114,126,138,150,162,174,186,198,210,222,234,246,258,270,282,294,306,318,330,342,354)
vs.JUL <- c(7,19,31,43,55,67,79,91,103,115,127,139,151,163,175,187,199,211,223,235,247,259,271,283,295,307,319,331,343,355)
vs.AUG <- c(8,20,32,44,56,68,80,92,104,116,128,140,152,164,176,188,200,212,224,236,248,260,272,284,296,308,320,332,344,356)
vs.SEP <- c(9,21,33,45,57,69,81,93,105,117,129,141,153,165,177,189,201,213,225,237,249,261,273,285,297,309,321,333,345,357)
vs.OCT <- c(10,22,34,46,58,70,82,94,106,118,130,142,154,166,178,190,202,214,226,238,250,262,274,286,298,310,322,334,346,358)
vs.NOV <- c(11,23,35,47,59,71,83,95,107,119,131,143,155,167,179,191,203,215,227,239,251,263,275,287,299,311,323,335,347,359)
vs.DEC <- c(12,24,36,48,60,72,84,96,108,120,132,144,156,168,180,192,204,216,228,240,252,264,276,288,300,312,324,336,348,360)

# Observed values are defined
l.observed <- hist_1961_1990_IMN

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# JAN
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IMN_1961_1990_JAN_mean <- calc(l.observed[[vs.JAN]], mean, na.rm=TRUE)
writeRaster(IMN_1961_1990_JAN_mean, "test_IMN_1961_1990_JAN_mean.tif", format="GTiff", overwrite = TRUE)

IMN_1961_1990_JAN_sd <- calc(l.observed[[vs.JAN]], sd, na.rm=TRUE)
writeRaster(IMN_1961_1990_JAN_sd, "test_IMN_1961_1990_JAN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_JAN_mean <- calc(DEF_ENSEMBLE_rcp26_2011_2040_JAN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_JAN_mean, "DEF_ENSEMBLE_rcp26_2011_2040_JAN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_JAN_mean <- calc(DEF_ENSEMBLE_rcp45_2011_2040_JAN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_JAN_mean, "DEF_ENSEMBLE_rcp45_2011_2040_JAN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_JAN_mean <- calc(DEF_ENSEMBLE_rcp85_2011_2040_JAN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_JAN_mean, "DEF_ENSEMBLE_rcp85_2011_2040_JAN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_JAN_sd <- calc(DEF_ENSEMBLE_rcp26_2011_2040_JAN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_JAN_sd, "DEF_ENSEMBLE_rcp26_2011_2040_JAN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_JAN_sd <- calc(DEF_ENSEMBLE_rcp45_2011_2040_JAN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_JAN_sd, "DEF_ENSEMBLE_rcp45_2011_2040_JAN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_JAN_sd <- calc(DEF_ENSEMBLE_rcp85_2011_2040_JAN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_JAN_sd, "DEF_ENSEMBLE_rcp85_2011_2040_JAN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_JAN_mean <- calc(DEF_ENSEMBLE_rcp26_2041_2070_JAN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_JAN_mean, "DEF_ENSEMBLE_rcp26_2041_2070_JAN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_JAN_mean <- calc(DEF_ENSEMBLE_rcp45_2041_2070_JAN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_JAN_mean, "DEF_ENSEMBLE_rcp45_2041_2070_JAN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_JAN_mean <- calc(DEF_ENSEMBLE_rcp85_2041_2070_JAN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_JAN_mean, "DEF_ENSEMBLE_rcp85_2041_2070_JAN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_JAN_sd <- calc(DEF_ENSEMBLE_rcp26_2041_2070_JAN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_JAN_sd, "DEF_ENSEMBLE_rcp26_2041_2070_JAN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_JAN_sd <- calc(DEF_ENSEMBLE_rcp45_2041_2070_JAN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_JAN_sd, "DEF_ENSEMBLE_rcp45_2041_2070_JAN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_JAN_sd <- calc(DEF_ENSEMBLE_rcp85_2041_2070_JAN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_JAN_sd, "DEF_ENSEMBLE_rcp85_2041_2070_JAN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_JAN_mean <- calc(DEF_ENSEMBLE_rcp26_2071_2100_JAN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_JAN_mean, "DEF_ENSEMBLE_rcp26_2071_2100_JAN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_JAN_mean <- calc(DEF_ENSEMBLE_rcp45_2071_2100_JAN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_JAN_mean, "DEF_ENSEMBLE_rcp45_2071_2100_JAN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_JAN_mean <- calc(DEF_ENSEMBLE_rcp85_2071_2100_JAN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_JAN_mean, "DEF_ENSEMBLE_rcp85_2071_2100_JAN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_JAN_sd <- calc(DEF_ENSEMBLE_rcp26_2071_2100_JAN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_JAN_sd, "DEF_ENSEMBLE_rcp26_2071_2100_JAN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_JAN_sd <- calc(DEF_ENSEMBLE_rcp45_2071_2100_JAN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_JAN_sd, "DEF_ENSEMBLE_rcp45_2071_2100_JAN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_JAN_sd <- calc(DEF_ENSEMBLE_rcp85_2071_2100_JAN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_JAN_sd, "DEF_ENSEMBLE_rcp85_2071_2100_JAN_sd.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# JAN
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_JAN <- (((IMN_1961_1990_JAN_mean - DEF_ENSEMBLE_rcp26_2011_2040_JAN_mean) / IMN_1961_1990_JAN_mean)*-100)
writeRaster(PC_rcp26_2011_2040_JAN, "PC_rcp26_2011_2040_JAN.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2011_2040_JAN <- (((IMN_1961_1990_JAN_mean - DEF_ENSEMBLE_rcp45_2011_2040_JAN_mean) / IMN_1961_1990_JAN_mean)*-100)
writeRaster(PC_rcp45_2011_2040_JAN, "PC_rcp45_2011_2040_JAN.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2011_2040_JAN <- (((IMN_1961_1990_JAN_mean - DEF_ENSEMBLE_rcp85_2011_2040_JAN_mean) / IMN_1961_1990_JAN_mean)*-100)
writeRaster(PC_rcp85_2011_2040_JAN, "PC_rcp85_2011_2040_JAN.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_JAN <- (((IMN_1961_1990_JAN_mean - DEF_ENSEMBLE_rcp26_2041_2070_JAN_mean) / IMN_1961_1990_JAN_mean)*-100)
writeRaster(PC_rcp26_2041_2070_JAN, "PC_rcp26_2041_2070_JAN.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2041_2070_JAN <- (((IMN_1961_1990_JAN_mean - DEF_ENSEMBLE_rcp45_2041_2070_JAN_mean) / IMN_1961_1990_JAN_mean)*-100)
writeRaster(PC_rcp45_2041_2070_JAN, "PC_rcp45_2041_2070_JAN.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2041_2070_JAN <- (((IMN_1961_1990_JAN_mean - DEF_ENSEMBLE_rcp85_2041_2070_JAN_mean) / IMN_1961_1990_JAN_mean)*-100)
writeRaster(PC_rcp85_2041_2070_JAN, "PC_rcp85_2041_2070_JAN.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_JAN <- (((IMN_1961_1990_JAN_mean - DEF_ENSEMBLE_rcp26_2071_2100_JAN_mean) / IMN_1961_1990_JAN_mean)*-100)
writeRaster(PC_rcp26_2071_2100_JAN, "PC_rcp26_2071_2100_JAN.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2071_2100_JAN <- (((IMN_1961_1990_JAN_mean - DEF_ENSEMBLE_rcp45_2071_2100_JAN_mean) / IMN_1961_1990_JAN_mean)*-100)
writeRaster(PC_rcp45_2071_2100_JAN, "PC_rcp45_2071_2100_JAN.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2071_2100_JAN <- (((IMN_1961_1990_JAN_mean - DEF_ENSEMBLE_rcp85_2071_2100_JAN_mean) / IMN_1961_1990_JAN_mean)*-100)
writeRaster(PC_rcp85_2071_2100_JAN, "PC_rcp85_2071_2100_JAN.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# FEB
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IMN_1961_1990_FEB_mean <- calc(l.observed[[vs.FEB]], mean, na.rm=TRUE)
writeRaster(IMN_1961_1990_FEB_mean, "test_IMN_1961_1990_FEB_mean.tif", format="GTiff", overwrite = TRUE)

IMN_1961_1990_FEB_sd <- calc(l.observed[[vs.FEB]], sd, na.rm=TRUE)
writeRaster(IMN_1961_1990_FEB_sd, "test_IMN_1961_1990_FEB_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_FEB_mean <- calc(DEF_ENSEMBLE_rcp26_2011_2040_FEB, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_FEB_mean, "DEF_ENSEMBLE_rcp26_2011_2040_FEB_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_FEB_mean <- calc(DEF_ENSEMBLE_rcp45_2011_2040_FEB, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_FEB_mean, "DEF_ENSEMBLE_rcp45_2011_2040_FEB_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_FEB_mean <- calc(DEF_ENSEMBLE_rcp85_2011_2040_FEB, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_FEB_mean, "DEF_ENSEMBLE_rcp85_2011_2040_FEB_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_FEB_sd <- calc(DEF_ENSEMBLE_rcp26_2011_2040_FEB, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_FEB_sd, "DEF_ENSEMBLE_rcp26_2011_2040_FEB_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_FEB_sd <- calc(DEF_ENSEMBLE_rcp45_2011_2040_FEB, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_FEB_sd, "DEF_ENSEMBLE_rcp45_2011_2040_FEB_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_FEB_sd <- calc(DEF_ENSEMBLE_rcp85_2011_2040_FEB, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_FEB_sd, "DEF_ENSEMBLE_rcp85_2011_2040_FEB_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_FEB_mean <- calc(DEF_ENSEMBLE_rcp26_2041_2070_FEB, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_FEB_mean, "DEF_ENSEMBLE_rcp26_2041_2070_FEB_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_FEB_mean <- calc(DEF_ENSEMBLE_rcp45_2041_2070_FEB, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_FEB_mean, "DEF_ENSEMBLE_rcp45_2041_2070_FEB_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_FEB_mean <- calc(DEF_ENSEMBLE_rcp85_2041_2070_FEB, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_FEB_mean, "DEF_ENSEMBLE_rcp85_2041_2070_FEB_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_FEB_sd <- calc(DEF_ENSEMBLE_rcp26_2041_2070_FEB, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_FEB_sd, "DEF_ENSEMBLE_rcp26_2041_2070_FEB_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_FEB_sd <- calc(DEF_ENSEMBLE_rcp45_2041_2070_FEB, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_FEB_sd, "DEF_ENSEMBLE_rcp45_2041_2070_FEB_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_FEB_sd <- calc(DEF_ENSEMBLE_rcp85_2041_2070_FEB, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_FEB_sd, "DEF_ENSEMBLE_rcp85_2041_2070_FEB_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_FEB_mean <- calc(DEF_ENSEMBLE_rcp26_2071_2100_FEB, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_FEB_mean, "DEF_ENSEMBLE_rcp26_2071_2100_FEB_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_FEB_mean <- calc(DEF_ENSEMBLE_rcp45_2071_2100_FEB, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_FEB_mean, "DEF_ENSEMBLE_rcp45_2071_2100_FEB_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_FEB_mean <- calc(DEF_ENSEMBLE_rcp85_2071_2100_FEB, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_FEB_mean, "DEF_ENSEMBLE_rcp85_2071_2100_FEB_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_FEB_sd <- calc(DEF_ENSEMBLE_rcp26_2071_2100_FEB, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_FEB_sd, "DEF_ENSEMBLE_rcp26_2071_2100_FEB_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_FEB_sd <- calc(DEF_ENSEMBLE_rcp45_2071_2100_FEB, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_FEB_sd, "DEF_ENSEMBLE_rcp45_2071_2100_FEB_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_FEB_sd <- calc(DEF_ENSEMBLE_rcp85_2071_2100_FEB, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_FEB_sd, "DEF_ENSEMBLE_rcp85_2071_2100_FEB_sd.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# FEB
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_FEB <- (((IMN_1961_1990_FEB_mean - DEF_ENSEMBLE_rcp26_2011_2040_FEB_mean) / IMN_1961_1990_FEB_mean)*-100)
writeRaster(PC_rcp26_2011_2040_FEB, "PC_rcp26_2011_2040_FEB.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2011_2040_FEB <- (((IMN_1961_1990_FEB_mean - DEF_ENSEMBLE_rcp45_2011_2040_FEB_mean) / IMN_1961_1990_FEB_mean)*-100)
writeRaster(PC_rcp45_2011_2040_FEB, "PC_rcp45_2011_2040_FEB.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2011_2040_FEB <- (((IMN_1961_1990_FEB_mean - DEF_ENSEMBLE_rcp85_2011_2040_FEB_mean) / IMN_1961_1990_FEB_mean)*-100)
writeRaster(PC_rcp85_2011_2040_FEB, "PC_rcp85_2011_2040_FEB.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_FEB <- (((IMN_1961_1990_FEB_mean - DEF_ENSEMBLE_rcp26_2041_2070_FEB_mean) / IMN_1961_1990_FEB_mean)*-100)
writeRaster(PC_rcp26_2041_2070_FEB, "PC_rcp26_2041_2070_FEB.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2041_2070_FEB <- (((IMN_1961_1990_FEB_mean - DEF_ENSEMBLE_rcp45_2041_2070_FEB_mean) / IMN_1961_1990_FEB_mean)*-100)
writeRaster(PC_rcp45_2041_2070_FEB, "PC_rcp45_2041_2070_FEB.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2041_2070_FEB <- (((IMN_1961_1990_FEB_mean - DEF_ENSEMBLE_rcp85_2041_2070_FEB_mean) / IMN_1961_1990_FEB_mean)*-100)
writeRaster(PC_rcp85_2041_2070_FEB, "PC_rcp85_2041_2070_FEB.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_FEB <- (((IMN_1961_1990_FEB_mean - DEF_ENSEMBLE_rcp26_2071_2100_FEB_mean) / IMN_1961_1990_FEB_mean)*-100)
writeRaster(PC_rcp26_2071_2100_FEB, "PC_rcp26_2071_2100_FEB.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2071_2100_FEB <- (((IMN_1961_1990_FEB_mean - DEF_ENSEMBLE_rcp45_2071_2100_FEB_mean) / IMN_1961_1990_FEB_mean)*-100)
writeRaster(PC_rcp45_2071_2100_FEB, "PC_rcp45_2071_2100_FEB.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2071_2100_FEB <- (((IMN_1961_1990_FEB_mean - DEF_ENSEMBLE_rcp85_2071_2100_FEB_mean) / IMN_1961_1990_FEB_mean)*-100)
writeRaster(PC_rcp85_2071_2100_FEB, "PC_rcp85_2071_2100_FEB.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# MAR
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IMN_1961_1990_MAR_mean <- calc(l.observed[[vs.MAR]], mean, na.rm=TRUE)
writeRaster(IMN_1961_1990_MAR_mean, "test_IMN_1961_1990_MAR_mean.tif", format="GTiff", overwrite = TRUE)

IMN_1961_1990_MAR_sd <- calc(l.observed[[vs.MAR]], sd, na.rm=TRUE)
writeRaster(IMN_1961_1990_MAR_sd, "test_IMN_1961_1990_MAR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_MAR_mean <- calc(DEF_ENSEMBLE_rcp26_2011_2040_MAR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_MAR_mean, "DEF_ENSEMBLE_rcp26_2011_2040_MAR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_MAR_mean <- calc(DEF_ENSEMBLE_rcp45_2011_2040_MAR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_MAR_mean, "DEF_ENSEMBLE_rcp45_2011_2040_MAR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_MAR_mean <- calc(DEF_ENSEMBLE_rcp85_2011_2040_MAR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_MAR_mean, "DEF_ENSEMBLE_rcp85_2011_2040_MAR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_MAR_sd <- calc(DEF_ENSEMBLE_rcp26_2011_2040_MAR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_MAR_sd, "DEF_ENSEMBLE_rcp26_2011_2040_MAR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_MAR_sd <- calc(DEF_ENSEMBLE_rcp45_2011_2040_MAR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_MAR_sd, "DEF_ENSEMBLE_rcp45_2011_2040_MAR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_MAR_sd <- calc(DEF_ENSEMBLE_rcp85_2011_2040_MAR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_MAR_sd, "DEF_ENSEMBLE_rcp85_2011_2040_MAR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_MAR_mean <- calc(DEF_ENSEMBLE_rcp26_2041_2070_MAR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_MAR_mean, "DEF_ENSEMBLE_rcp26_2041_2070_MAR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_MAR_mean <- calc(DEF_ENSEMBLE_rcp45_2041_2070_MAR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_MAR_mean, "DEF_ENSEMBLE_rcp45_2041_2070_MAR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_MAR_mean <- calc(DEF_ENSEMBLE_rcp85_2041_2070_MAR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_MAR_mean, "DEF_ENSEMBLE_rcp85_2041_2070_MAR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_MAR_sd <- calc(DEF_ENSEMBLE_rcp26_2041_2070_MAR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_MAR_sd, "DEF_ENSEMBLE_rcp26_2041_2070_MAR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_MAR_sd <- calc(DEF_ENSEMBLE_rcp45_2041_2070_MAR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_MAR_sd, "DEF_ENSEMBLE_rcp45_2041_2070_MAR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_MAR_sd <- calc(DEF_ENSEMBLE_rcp85_2041_2070_MAR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_MAR_sd, "DEF_ENSEMBLE_rcp85_2041_2070_MAR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_MAR_mean <- calc(DEF_ENSEMBLE_rcp26_2071_2100_MAR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_MAR_mean, "DEF_ENSEMBLE_rcp26_2071_2100_MAR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_MAR_mean <- calc(DEF_ENSEMBLE_rcp45_2071_2100_MAR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_MAR_mean, "DEF_ENSEMBLE_rcp45_2071_2100_MAR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_MAR_mean <- calc(DEF_ENSEMBLE_rcp85_2071_2100_MAR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_MAR_mean, "DEF_ENSEMBLE_rcp85_2071_2100_MAR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_MAR_sd <- calc(DEF_ENSEMBLE_rcp26_2071_2100_MAR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_MAR_sd, "DEF_ENSEMBLE_rcp26_2071_2100_MAR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_MAR_sd <- calc(DEF_ENSEMBLE_rcp45_2071_2100_MAR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_MAR_sd, "DEF_ENSEMBLE_rcp45_2071_2100_MAR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_MAR_sd <- calc(DEF_ENSEMBLE_rcp85_2071_2100_MAR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_MAR_sd, "DEF_ENSEMBLE_rcp85_2071_2100_MAR_sd.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# MAR
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_MAR <- (((IMN_1961_1990_MAR_mean - DEF_ENSEMBLE_rcp26_2011_2040_MAR_mean) / IMN_1961_1990_MAR_mean)*-100)
writeRaster(PC_rcp26_2011_2040_MAR, "PC_rcp26_2011_2040_MAR.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2011_2040_MAR <- (((IMN_1961_1990_MAR_mean - DEF_ENSEMBLE_rcp45_2011_2040_MAR_mean) / IMN_1961_1990_MAR_mean)*-100)
writeRaster(PC_rcp45_2011_2040_MAR, "PC_rcp45_2011_2040_MAR.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2011_2040_MAR <- (((IMN_1961_1990_MAR_mean - DEF_ENSEMBLE_rcp85_2011_2040_MAR_mean) / IMN_1961_1990_MAR_mean)*-100)
writeRaster(PC_rcp85_2011_2040_MAR, "PC_rcp85_2011_2040_MAR.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_MAR <- (((IMN_1961_1990_MAR_mean - DEF_ENSEMBLE_rcp26_2041_2070_MAR_mean) / IMN_1961_1990_MAR_mean)*-100)
writeRaster(PC_rcp26_2041_2070_MAR, "PC_rcp26_2041_2070_MAR.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2041_2070_MAR <- (((IMN_1961_1990_MAR_mean - DEF_ENSEMBLE_rcp45_2041_2070_MAR_mean) / IMN_1961_1990_MAR_mean)*-100)
writeRaster(PC_rcp45_2041_2070_MAR, "PC_rcp45_2041_2070_MAR.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2041_2070_MAR <- (((IMN_1961_1990_MAR_mean - DEF_ENSEMBLE_rcp85_2041_2070_MAR_mean) / IMN_1961_1990_MAR_mean)*-100)
writeRaster(PC_rcp85_2041_2070_MAR, "PC_rcp85_2041_2070_MAR.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_MAR <- (((IMN_1961_1990_MAR_mean - DEF_ENSEMBLE_rcp26_2071_2100_MAR_mean) / IMN_1961_1990_MAR_mean)*-100)
writeRaster(PC_rcp26_2071_2100_MAR, "PC_rcp26_2071_2100_MAR.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2071_2100_MAR <- (((IMN_1961_1990_MAR_mean - DEF_ENSEMBLE_rcp45_2071_2100_MAR_mean) / IMN_1961_1990_MAR_mean)*-100)
writeRaster(PC_rcp45_2071_2100_MAR, "PC_rcp45_2071_2100_MAR.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2071_2100_MAR <- (((IMN_1961_1990_MAR_mean - DEF_ENSEMBLE_rcp85_2071_2100_MAR_mean) / IMN_1961_1990_MAR_mean)*-100)
writeRaster(PC_rcp85_2071_2100_MAR, "PC_rcp85_2071_2100_MAR.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# APR
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IMN_1961_1990_APR_mean <- calc(l.observed[[vs.APR]], mean, na.rm=TRUE)
writeRaster(IMN_1961_1990_APR_mean, "test_IMN_1961_1990_APR_mean.tif", format="GTiff", overwrite = TRUE)

IMN_1961_1990_APR_sd <- calc(l.observed[[vs.APR]], sd, na.rm=TRUE)
writeRaster(IMN_1961_1990_APR_sd, "test_IMN_1961_1990_APR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_APR_mean <- calc(DEF_ENSEMBLE_rcp26_2011_2040_APR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_APR_mean, "DEF_ENSEMBLE_rcp26_2011_2040_APR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_APR_mean <- calc(DEF_ENSEMBLE_rcp45_2011_2040_APR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_APR_mean, "DEF_ENSEMBLE_rcp45_2011_2040_APR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_APR_mean <- calc(DEF_ENSEMBLE_rcp85_2011_2040_APR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_APR_mean, "DEF_ENSEMBLE_rcp85_2011_2040_APR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_APR_sd <- calc(DEF_ENSEMBLE_rcp26_2011_2040_APR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_APR_sd, "DEF_ENSEMBLE_rcp26_2011_2040_APR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_APR_sd <- calc(DEF_ENSEMBLE_rcp45_2011_2040_APR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_APR_sd, "DEF_ENSEMBLE_rcp45_2011_2040_APR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_APR_sd <- calc(DEF_ENSEMBLE_rcp85_2011_2040_APR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_APR_sd, "DEF_ENSEMBLE_rcp85_2011_2040_APR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_APR_mean <- calc(DEF_ENSEMBLE_rcp26_2041_2070_APR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_APR_mean, "DEF_ENSEMBLE_rcp26_2041_2070_APR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_APR_mean <- calc(DEF_ENSEMBLE_rcp45_2041_2070_APR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_APR_mean, "DEF_ENSEMBLE_rcp45_2041_2070_APR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_APR_mean <- calc(DEF_ENSEMBLE_rcp85_2041_2070_APR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_APR_mean, "DEF_ENSEMBLE_rcp85_2041_2070_APR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_APR_sd <- calc(DEF_ENSEMBLE_rcp26_2041_2070_APR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_APR_sd, "DEF_ENSEMBLE_rcp26_2041_2070_APR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_APR_sd <- calc(DEF_ENSEMBLE_rcp45_2041_2070_APR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_APR_sd, "DEF_ENSEMBLE_rcp45_2041_2070_APR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_APR_sd <- calc(DEF_ENSEMBLE_rcp85_2041_2070_APR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_APR_sd, "DEF_ENSEMBLE_rcp85_2041_2070_APR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_APR_mean <- calc(DEF_ENSEMBLE_rcp26_2071_2100_APR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_APR_mean, "DEF_ENSEMBLE_rcp26_2071_2100_APR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_APR_mean <- calc(DEF_ENSEMBLE_rcp45_2071_2100_APR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_APR_mean, "DEF_ENSEMBLE_rcp45_2071_2100_APR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_APR_mean <- calc(DEF_ENSEMBLE_rcp85_2071_2100_APR, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_APR_mean, "DEF_ENSEMBLE_rcp85_2071_2100_APR_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_APR_sd <- calc(DEF_ENSEMBLE_rcp26_2071_2100_APR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_APR_sd, "DEF_ENSEMBLE_rcp26_2071_2100_APR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_APR_sd <- calc(DEF_ENSEMBLE_rcp45_2071_2100_APR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_APR_sd, "DEF_ENSEMBLE_rcp45_2071_2100_APR_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_APR_sd <- calc(DEF_ENSEMBLE_rcp85_2071_2100_APR, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_APR_sd, "DEF_ENSEMBLE_rcp85_2071_2100_APR_sd.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# APR
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_APR <- (((IMN_1961_1990_APR_mean - DEF_ENSEMBLE_rcp26_2011_2040_APR_mean) / IMN_1961_1990_APR_mean)*-100)
writeRaster(PC_rcp26_2011_2040_APR, "PC_rcp26_2011_2040_APR.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2011_2040_APR <- (((IMN_1961_1990_APR_mean - DEF_ENSEMBLE_rcp45_2011_2040_APR_mean) / IMN_1961_1990_APR_mean)*-100)
writeRaster(PC_rcp45_2011_2040_APR, "PC_rcp45_2011_2040_APR.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2011_2040_APR <- (((IMN_1961_1990_APR_mean - DEF_ENSEMBLE_rcp85_2011_2040_APR_mean) / IMN_1961_1990_APR_mean)*-100)
writeRaster(PC_rcp85_2011_2040_APR, "PC_rcp85_2011_2040_APR.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_APR <- (((IMN_1961_1990_APR_mean - DEF_ENSEMBLE_rcp26_2041_2070_APR_mean) / IMN_1961_1990_APR_mean)*-100)
writeRaster(PC_rcp26_2041_2070_APR, "PC_rcp26_2041_2070_APR.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2041_2070_APR <- (((IMN_1961_1990_APR_mean - DEF_ENSEMBLE_rcp45_2041_2070_APR_mean) / IMN_1961_1990_APR_mean)*-100)
writeRaster(PC_rcp45_2041_2070_APR, "PC_rcp45_2041_2070_APR.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2041_2070_APR <- (((IMN_1961_1990_APR_mean - DEF_ENSEMBLE_rcp85_2041_2070_APR_mean) / IMN_1961_1990_APR_mean)*-100)
writeRaster(PC_rcp85_2041_2070_APR, "PC_rcp85_2041_2070_APR.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_APR <- (((IMN_1961_1990_APR_mean - DEF_ENSEMBLE_rcp26_2071_2100_APR_mean) / IMN_1961_1990_APR_mean)*-100)
writeRaster(PC_rcp26_2071_2100_APR, "PC_rcp26_2071_2100_APR.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2071_2100_APR <- (((IMN_1961_1990_APR_mean - DEF_ENSEMBLE_rcp45_2071_2100_APR_mean) / IMN_1961_1990_APR_mean)*-100)
writeRaster(PC_rcp45_2071_2100_APR, "PC_rcp45_2071_2100_APR.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2071_2100_APR <- (((IMN_1961_1990_APR_mean - DEF_ENSEMBLE_rcp85_2071_2100_APR_mean) / IMN_1961_1990_APR_mean)*-100)
writeRaster(PC_rcp85_2071_2100_APR, "PC_rcp85_2071_2100_APR.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# MAY
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IMN_1961_1990_MAY_mean <- calc(l.observed[[vs.MAY]], mean, na.rm=TRUE)
writeRaster(IMN_1961_1990_MAY_mean, "test_IMN_1961_1990_MAY_mean.tif", format="GTiff", overwrite = TRUE)

IMN_1961_1990_MAY_sd <- calc(l.observed[[vs.MAY]], sd, na.rm=TRUE)
writeRaster(IMN_1961_1990_MAY_sd, "test_IMN_1961_1990_MAY_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_MAY_mean <- calc(DEF_ENSEMBLE_rcp26_2011_2040_MAY, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_MAY_mean, "DEF_ENSEMBLE_rcp26_2011_2040_MAY_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_MAY_mean <- calc(DEF_ENSEMBLE_rcp45_2011_2040_MAY, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_MAY_mean, "DEF_ENSEMBLE_rcp45_2011_2040_MAY_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_MAY_mean <- calc(DEF_ENSEMBLE_rcp85_2011_2040_MAY, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_MAY_mean, "DEF_ENSEMBLE_rcp85_2011_2040_MAY_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_MAY_sd <- calc(DEF_ENSEMBLE_rcp26_2011_2040_MAY, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_MAY_sd, "DEF_ENSEMBLE_rcp26_2011_2040_MAY_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_MAY_sd <- calc(DEF_ENSEMBLE_rcp45_2011_2040_MAY, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_MAY_sd, "DEF_ENSEMBLE_rcp45_2011_2040_MAY_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_MAY_sd <- calc(DEF_ENSEMBLE_rcp85_2011_2040_MAY, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_MAY_sd, "DEF_ENSEMBLE_rcp85_2011_2040_MAY_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_MAY_mean <- calc(DEF_ENSEMBLE_rcp26_2041_2070_MAY, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_MAY_mean, "DEF_ENSEMBLE_rcp26_2041_2070_MAY_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_MAY_mean <- calc(DEF_ENSEMBLE_rcp45_2041_2070_MAY, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_MAY_mean, "DEF_ENSEMBLE_rcp45_2041_2070_MAY_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_MAY_mean <- calc(DEF_ENSEMBLE_rcp85_2041_2070_MAY, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_MAY_mean, "DEF_ENSEMBLE_rcp85_2041_2070_MAY_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_MAY_sd <- calc(DEF_ENSEMBLE_rcp26_2041_2070_MAY, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_MAY_sd, "DEF_ENSEMBLE_rcp26_2041_2070_MAY_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_MAY_sd <- calc(DEF_ENSEMBLE_rcp45_2041_2070_MAY, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_MAY_sd, "DEF_ENSEMBLE_rcp45_2041_2070_MAY_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_MAY_sd <- calc(DEF_ENSEMBLE_rcp85_2041_2070_MAY, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_MAY_sd, "DEF_ENSEMBLE_rcp85_2041_2070_MAY_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_MAY_mean <- calc(DEF_ENSEMBLE_rcp26_2071_2100_MAY, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_MAY_mean, "DEF_ENSEMBLE_rcp26_2071_2100_MAY_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_MAY_mean <- calc(DEF_ENSEMBLE_rcp45_2071_2100_MAY, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_MAY_mean, "DEF_ENSEMBLE_rcp45_2071_2100_MAY_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_MAY_mean <- calc(DEF_ENSEMBLE_rcp85_2071_2100_MAY, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_MAY_mean, "DEF_ENSEMBLE_rcp85_2071_2100_MAY_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_MAY_sd <- calc(DEF_ENSEMBLE_rcp26_2071_2100_MAY, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_MAY_sd, "DEF_ENSEMBLE_rcp26_2071_2100_MAY_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_MAY_sd <- calc(DEF_ENSEMBLE_rcp45_2071_2100_MAY, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_MAY_sd, "DEF_ENSEMBLE_rcp45_2071_2100_MAY_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_MAY_sd <- calc(DEF_ENSEMBLE_rcp85_2071_2100_MAY, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_MAY_sd, "DEF_ENSEMBLE_rcp85_2071_2100_MAY_sd.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# MAY
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_MAY <- (((IMN_1961_1990_MAY_mean - DEF_ENSEMBLE_rcp26_2011_2040_MAY_mean) / IMN_1961_1990_MAY_mean)*-100)
writeRaster(PC_rcp26_2011_2040_MAY, "PC_rcp26_2011_2040_MAY.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2011_2040_MAY <- (((IMN_1961_1990_MAY_mean - DEF_ENSEMBLE_rcp45_2011_2040_MAY_mean) / IMN_1961_1990_MAY_mean)*-100)
writeRaster(PC_rcp45_2011_2040_MAY, "PC_rcp45_2011_2040_MAY.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2011_2040_MAY <- (((IMN_1961_1990_MAY_mean - DEF_ENSEMBLE_rcp85_2011_2040_MAY_mean) / IMN_1961_1990_MAY_mean)*-100)
writeRaster(PC_rcp85_2011_2040_MAY, "PC_rcp85_2011_2040_MAY.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_MAY <- (((IMN_1961_1990_MAY_mean - DEF_ENSEMBLE_rcp26_2041_2070_MAY_mean) / IMN_1961_1990_MAY_mean)*-100)
writeRaster(PC_rcp26_2041_2070_MAY, "PC_rcp26_2041_2070_MAY.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2041_2070_MAY <- (((IMN_1961_1990_MAY_mean - DEF_ENSEMBLE_rcp45_2041_2070_MAY_mean) / IMN_1961_1990_MAY_mean)*-100)
writeRaster(PC_rcp45_2041_2070_MAY, "PC_rcp45_2041_2070_MAY.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2041_2070_MAY <- (((IMN_1961_1990_MAY_mean - DEF_ENSEMBLE_rcp85_2041_2070_MAY_mean) / IMN_1961_1990_MAY_mean)*-100)
writeRaster(PC_rcp85_2041_2070_MAY, "PC_rcp85_2041_2070_MAY.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_MAY <- (((IMN_1961_1990_MAY_mean - DEF_ENSEMBLE_rcp26_2071_2100_MAY_mean) / IMN_1961_1990_MAY_mean)*-100)
writeRaster(PC_rcp26_2071_2100_MAY, "PC_rcp26_2071_2100_MAY.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2071_2100_MAY <- (((IMN_1961_1990_MAY_mean - DEF_ENSEMBLE_rcp45_2071_2100_MAY_mean) / IMN_1961_1990_MAY_mean)*-100)
writeRaster(PC_rcp45_2071_2100_MAY, "PC_rcp45_2071_2100_MAY.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2071_2100_MAY <- (((IMN_1961_1990_MAY_mean - DEF_ENSEMBLE_rcp85_2071_2100_MAY_mean) / IMN_1961_1990_MAY_mean)*-100)
writeRaster(PC_rcp85_2071_2100_MAY, "PC_rcp85_2071_2100_MAY.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# JUN
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IMN_1961_1990_JUN_mean <- calc(l.observed[[vs.JUN]], mean, na.rm=TRUE)
writeRaster(IMN_1961_1990_JUN_mean, "test_IMN_1961_1990_JUN_mean.tif", format="GTiff", overwrite = TRUE)

IMN_1961_1990_JUN_sd <- calc(l.observed[[vs.JUN]], sd, na.rm=TRUE)
writeRaster(IMN_1961_1990_JUN_sd, "test_IMN_1961_1990_JUN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_JUN_mean <- calc(DEF_ENSEMBLE_rcp26_2011_2040_JUN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_JUN_mean, "DEF_ENSEMBLE_rcp26_2011_2040_JUN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_JUN_mean <- calc(DEF_ENSEMBLE_rcp45_2011_2040_JUN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_JUN_mean, "DEF_ENSEMBLE_rcp45_2011_2040_JUN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_JUN_mean <- calc(DEF_ENSEMBLE_rcp85_2011_2040_JUN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_JUN_mean, "DEF_ENSEMBLE_rcp85_2011_2040_JUN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_JUN_sd <- calc(DEF_ENSEMBLE_rcp26_2011_2040_JUN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_JUN_sd, "DEF_ENSEMBLE_rcp26_2011_2040_JUN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_JUN_sd <- calc(DEF_ENSEMBLE_rcp45_2011_2040_JUN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_JUN_sd, "DEF_ENSEMBLE_rcp45_2011_2040_JUN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_JUN_sd <- calc(DEF_ENSEMBLE_rcp85_2011_2040_JUN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_JUN_sd, "DEF_ENSEMBLE_rcp85_2011_2040_JUN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_JUN_mean <- calc(DEF_ENSEMBLE_rcp26_2041_2070_JUN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_JUN_mean, "DEF_ENSEMBLE_rcp26_2041_2070_JUN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_JUN_mean <- calc(DEF_ENSEMBLE_rcp45_2041_2070_JUN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_JUN_mean, "DEF_ENSEMBLE_rcp45_2041_2070_JUN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_JUN_mean <- calc(DEF_ENSEMBLE_rcp85_2041_2070_JUN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_JUN_mean, "DEF_ENSEMBLE_rcp85_2041_2070_JUN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_JUN_sd <- calc(DEF_ENSEMBLE_rcp26_2041_2070_JUN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_JUN_sd, "DEF_ENSEMBLE_rcp26_2041_2070_JUN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_JUN_sd <- calc(DEF_ENSEMBLE_rcp45_2041_2070_JUN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_JUN_sd, "DEF_ENSEMBLE_rcp45_2041_2070_JUN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_JUN_sd <- calc(DEF_ENSEMBLE_rcp85_2041_2070_JUN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_JUN_sd, "DEF_ENSEMBLE_rcp85_2041_2070_JUN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_JUN_mean <- calc(DEF_ENSEMBLE_rcp26_2071_2100_JUN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_JUN_mean, "DEF_ENSEMBLE_rcp26_2071_2100_JUN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_JUN_mean <- calc(DEF_ENSEMBLE_rcp45_2071_2100_JUN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_JUN_mean, "DEF_ENSEMBLE_rcp45_2071_2100_JUN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_JUN_mean <- calc(DEF_ENSEMBLE_rcp85_2071_2100_JUN, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_JUN_mean, "DEF_ENSEMBLE_rcp85_2071_2100_JUN_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_JUN_sd <- calc(DEF_ENSEMBLE_rcp26_2071_2100_JUN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_JUN_sd, "DEF_ENSEMBLE_rcp26_2071_2100_JUN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_JUN_sd <- calc(DEF_ENSEMBLE_rcp45_2071_2100_JUN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_JUN_sd, "DEF_ENSEMBLE_rcp45_2071_2100_JUN_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_JUN_sd <- calc(DEF_ENSEMBLE_rcp85_2071_2100_JUN, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_JUN_sd, "DEF_ENSEMBLE_rcp85_2071_2100_JUN_sd.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# JUN
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_JUN <- (((IMN_1961_1990_JUN_mean - DEF_ENSEMBLE_rcp26_2011_2040_JUN_mean) / IMN_1961_1990_JUN_mean)*-100)
writeRaster(PC_rcp26_2011_2040_JUN, "PC_rcp26_2011_2040_JUN.nc", format="GTiff", overwrite = TRUE)
PC_rcp45_2011_2040_JUN <- (((IMN_1961_1990_JUN_mean - DEF_ENSEMBLE_rcp45_2011_2040_JUN_mean) / IMN_1961_1990_JUN_mean)*-100)
writeRaster(PC_rcp45_2011_2040_JUN, "PC_rcp45_2011_2040_JUN.nc", format="GTiff", overwrite = TRUE)
PC_rcp85_2011_2040_JUN <- (((IMN_1961_1990_JUN_mean - DEF_ENSEMBLE_rcp85_2011_2040_JUN_mean) / IMN_1961_1990_JUN_mean)*-100)
writeRaster(PC_rcp85_2011_2040_JUN, "PC_rcp85_2011_2040_JUN.nc", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_JUN <- (((IMN_1961_1990_JUN_mean - DEF_ENSEMBLE_rcp26_2041_2070_JUN_mean) / IMN_1961_1990_JUN_mean)*-100)
writeRaster(PC_rcp26_2041_2070_JUN, "PC_rcp26_2041_2070_JUN.nc", format="GTiff", overwrite = TRUE)
PC_rcp45_2041_2070_JUN <- (((IMN_1961_1990_JUN_mean - DEF_ENSEMBLE_rcp45_2041_2070_JUN_mean) / IMN_1961_1990_JUN_mean)*-100)
writeRaster(PC_rcp45_2041_2070_JUN, "PC_rcp45_2041_2070_JUN.nc", format="GTiff", overwrite = TRUE)
PC_rcp85_2041_2070_JUN <- (((IMN_1961_1990_JUN_mean - DEF_ENSEMBLE_rcp85_2041_2070_JUN_mean) / IMN_1961_1990_JUN_mean)*-100)
writeRaster(PC_rcp85_2041_2070_JUN, "PC_rcp85_2041_2070_JUN.nc", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_JUN <- (((IMN_1961_1990_JUN_mean - DEF_ENSEMBLE_rcp26_2071_2100_JUN_mean) / IMN_1961_1990_JUN_mean)*-100)
writeRaster(PC_rcp26_2071_2100_JUN, "PC_rcp26_2071_2100_JUN.nc", format="GTiff", overwrite = TRUE)
PC_rcp45_2071_2100_JUN <- (((IMN_1961_1990_JUN_mean - DEF_ENSEMBLE_rcp45_2071_2100_JUN_mean) / IMN_1961_1990_JUN_mean)*-100)
writeRaster(PC_rcp45_2071_2100_JUN, "PC_rcp45_2071_2100_JUN.nc", format="GTiff", overwrite = TRUE)
PC_rcp85_2071_2100_JUN <- (((IMN_1961_1990_JUN_mean - DEF_ENSEMBLE_rcp85_2071_2100_JUN_mean) / IMN_1961_1990_JUN_mean)*-100)
writeRaster(PC_rcp85_2071_2100_JUN, "PC_rcp85_2071_2100_JUN.nc", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# JUL
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IMN_1961_1990_JUL_mean <- calc(l.observed[[vs.JUL]], mean, na.rm=TRUE)
writeRaster(IMN_1961_1990_JUL_mean, "test_IMN_1961_1990_JUL_mean.tif", format="GTiff", overwrite = TRUE)

IMN_1961_1990_JUL_sd <- calc(l.observed[[vs.JUL]], sd, na.rm=TRUE)
writeRaster(IMN_1961_1990_JUL_sd, "test_IMN_1961_1990_JUL_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_JUL_mean <- calc(DEF_ENSEMBLE_rcp26_2011_2040_JUL, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_JUL_mean, "DEF_ENSEMBLE_rcp26_2011_2040_JUL_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_JUL_mean <- calc(DEF_ENSEMBLE_rcp45_2011_2040_JUL, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_JUL_mean, "DEF_ENSEMBLE_rcp45_2011_2040_JUL_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_JUL_mean <- calc(DEF_ENSEMBLE_rcp85_2011_2040_JUL, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_JUL_mean, "DEF_ENSEMBLE_rcp85_2011_2040_JUL_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_JUL_sd <- calc(DEF_ENSEMBLE_rcp26_2011_2040_JUL, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_JUL_sd, "DEF_ENSEMBLE_rcp26_2011_2040_JUL_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_JUL_sd <- calc(DEF_ENSEMBLE_rcp45_2011_2040_JUL, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_JUL_sd, "DEF_ENSEMBLE_rcp45_2011_2040_JUL_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_JUL_sd <- calc(DEF_ENSEMBLE_rcp85_2011_2040_JUL, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_JUL_sd, "DEF_ENSEMBLE_rcp85_2011_2040_JUL_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_JUL_mean <- calc(DEF_ENSEMBLE_rcp26_2041_2070_JUL, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_JUL_mean, "DEF_ENSEMBLE_rcp26_2041_2070_JUL_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_JUL_mean <- calc(DEF_ENSEMBLE_rcp45_2041_2070_JUL, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_JUL_mean, "DEF_ENSEMBLE_rcp45_2041_2070_JUL_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_JUL_mean <- calc(DEF_ENSEMBLE_rcp85_2041_2070_JUL, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_JUL_mean, "DEF_ENSEMBLE_rcp85_2041_2070_JUL_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_JUL_sd <- calc(DEF_ENSEMBLE_rcp26_2041_2070_JUL, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_JUL_sd, "DEF_ENSEMBLE_rcp26_2041_2070_JUL_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_JUL_sd <- calc(DEF_ENSEMBLE_rcp45_2041_2070_JUL, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_JUL_sd, "DEF_ENSEMBLE_rcp45_2041_2070_JUL_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_JUL_sd <- calc(DEF_ENSEMBLE_rcp85_2041_2070_JUL, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_JUL_sd, "DEF_ENSEMBLE_rcp85_2041_2070_JUL_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_JUL_mean <- calc(DEF_ENSEMBLE_rcp26_2071_2100_JUL, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_JUL_mean, "DEF_ENSEMBLE_rcp26_2071_2100_JUL_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_JUL_mean <- calc(DEF_ENSEMBLE_rcp45_2071_2100_JUL, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_JUL_mean, "DEF_ENSEMBLE_rcp45_2071_2100_JUL_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_JUL_mean <- calc(DEF_ENSEMBLE_rcp85_2071_2100_JUL, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_JUL_mean, "DEF_ENSEMBLE_rcp85_2071_2100_JUL_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_JUL_sd <- calc(DEF_ENSEMBLE_rcp26_2071_2100_JUL, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_JUL_sd, "DEF_ENSEMBLE_rcp26_2071_2100_JUL_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_JUL_sd <- calc(DEF_ENSEMBLE_rcp45_2071_2100_JUL, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_JUL_sd, "DEF_ENSEMBLE_rcp45_2071_2100_JUL_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_JUL_sd <- calc(DEF_ENSEMBLE_rcp85_2071_2100_JUL, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_JUL_sd, "DEF_ENSEMBLE_rcp85_2071_2100_JUL_sd.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# JUL
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_JUL <- (((IMN_1961_1990_JUL_mean - DEF_ENSEMBLE_rcp26_2011_2040_JUL_mean) / IMN_1961_1990_JUL_mean)*-100)
writeRaster(PC_rcp26_2011_2040_JUL, "PC_rcp26_2011_2040_JUL.nc", format="GTiff", overwrite = TRUE)
PC_rcp45_2011_2040_JUL <- (((IMN_1961_1990_JUL_mean - DEF_ENSEMBLE_rcp45_2011_2040_JUL_mean) / IMN_1961_1990_JUL_mean)*-100)
writeRaster(PC_rcp45_2011_2040_JUL, "PC_rcp45_2011_2040_JUL.nc", format="GTiff", overwrite = TRUE)
PC_rcp85_2011_2040_JUL <- (((IMN_1961_1990_JUL_mean - DEF_ENSEMBLE_rcp85_2011_2040_JUL_mean) / IMN_1961_1990_JUL_mean)*-100)
writeRaster(PC_rcp85_2011_2040_JUL, "PC_rcp85_2011_2040_JUL.nc", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_JUL <- (((IMN_1961_1990_JUL_mean - DEF_ENSEMBLE_rcp26_2041_2070_JUL_mean) / IMN_1961_1990_JUL_mean)*-100)
writeRaster(PC_rcp26_2041_2070_JUL, "PC_rcp26_2041_2070_JUL.nc", format="GTiff", overwrite = TRUE)
PC_rcp45_2041_2070_JUL <- (((IMN_1961_1990_JUL_mean - DEF_ENSEMBLE_rcp45_2041_2070_JUL_mean) / IMN_1961_1990_JUL_mean)*-100)
writeRaster(PC_rcp45_2041_2070_JUL, "PC_rcp45_2041_2070_JUL.nc", format="GTiff", overwrite = TRUE)
PC_rcp85_2041_2070_JUL <- (((IMN_1961_1990_JUL_mean - DEF_ENSEMBLE_rcp85_2041_2070_JUL_mean) / IMN_1961_1990_JUL_mean)*-100)
writeRaster(PC_rcp85_2041_2070_JUL, "PC_rcp85_2041_2070_JUL.nc", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_JUL <- (((IMN_1961_1990_JUL_mean - DEF_ENSEMBLE_rcp26_2071_2100_JUL_mean) / IMN_1961_1990_JUL_mean)*-100)
writeRaster(PC_rcp26_2071_2100_JUL, "PC_rcp26_2071_2100_JUL.nc", format="GTiff", overwrite = TRUE)
PC_rcp45_2071_2100_JUL <- (((IMN_1961_1990_JUL_mean - DEF_ENSEMBLE_rcp45_2071_2100_JUL_mean) / IMN_1961_1990_JUL_mean)*-100)
writeRaster(PC_rcp45_2071_2100_JUL, "PC_rcp45_2071_2100_JUL.nc", format="GTiff", overwrite = TRUE)
PC_rcp85_2071_2100_JUL <- (((IMN_1961_1990_JUL_mean - DEF_ENSEMBLE_rcp85_2071_2100_JUL_mean) / IMN_1961_1990_JUL_mean)*-100)
writeRaster(PC_rcp85_2071_2100_JUL, "PC_rcp85_2071_2100_JUL.nc", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# AUG
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IMN_1961_1990_AUG_mean <- calc(l.observed[[vs.AUG]], mean, na.rm=TRUE)
writeRaster(IMN_1961_1990_AUG_mean, "test_IMN_1961_1990_AUG_mean.tif", format="GTiff", overwrite = TRUE)

IMN_1961_1990_AUG_sd <- calc(l.observed[[vs.AUG]], sd, na.rm=TRUE)
writeRaster(IMN_1961_1990_AUG_sd, "test_IMN_1961_1990_AUG_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_AUG_mean <- calc(DEF_ENSEMBLE_rcp26_2011_2040_AUG, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_AUG_mean, "DEF_ENSEMBLE_rcp26_2011_2040_AUG_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_AUG_mean <- calc(DEF_ENSEMBLE_rcp45_2011_2040_AUG, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_AUG_mean, "DEF_ENSEMBLE_rcp45_2011_2040_AUG_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_AUG_mean <- calc(DEF_ENSEMBLE_rcp85_2011_2040_AUG, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_AUG_mean, "DEF_ENSEMBLE_rcp85_2011_2040_AUG_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_AUG_sd <- calc(DEF_ENSEMBLE_rcp26_2011_2040_AUG, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_AUG_sd, "DEF_ENSEMBLE_rcp26_2011_2040_AUG_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_AUG_sd <- calc(DEF_ENSEMBLE_rcp45_2011_2040_AUG, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_AUG_sd, "DEF_ENSEMBLE_rcp45_2011_2040_AUG_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_AUG_sd <- calc(DEF_ENSEMBLE_rcp85_2011_2040_AUG, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_AUG_sd, "DEF_ENSEMBLE_rcp85_2011_2040_AUG_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_AUG_mean <- calc(DEF_ENSEMBLE_rcp26_2041_2070_AUG, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_AUG_mean, "DEF_ENSEMBLE_rcp26_2041_2070_AUG_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_AUG_mean <- calc(DEF_ENSEMBLE_rcp45_2041_2070_AUG, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_AUG_mean, "DEF_ENSEMBLE_rcp45_2041_2070_AUG_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_AUG_mean <- calc(DEF_ENSEMBLE_rcp85_2041_2070_AUG, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_AUG_mean, "DEF_ENSEMBLE_rcp85_2041_2070_AUG_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_AUG_sd <- calc(DEF_ENSEMBLE_rcp26_2041_2070_AUG, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_AUG_sd, "DEF_ENSEMBLE_rcp26_2041_2070_AUG_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_AUG_sd <- calc(DEF_ENSEMBLE_rcp45_2041_2070_AUG, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_AUG_sd, "DEF_ENSEMBLE_rcp45_2041_2070_AUG_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_AUG_sd <- calc(DEF_ENSEMBLE_rcp85_2041_2070_AUG, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_AUG_sd, "DEF_ENSEMBLE_rcp85_2041_2070_AUG_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_AUG_mean <- calc(DEF_ENSEMBLE_rcp26_2071_2100_AUG, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_AUG_mean, "DEF_ENSEMBLE_rcp26_2071_2100_AUG_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_AUG_mean <- calc(DEF_ENSEMBLE_rcp45_2071_2100_AUG, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_AUG_mean, "DEF_ENSEMBLE_rcp45_2071_2100_AUG_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_AUG_mean <- calc(DEF_ENSEMBLE_rcp85_2071_2100_AUG, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_AUG_mean, "DEF_ENSEMBLE_rcp85_2071_2100_AUG_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_AUG_sd <- calc(DEF_ENSEMBLE_rcp26_2071_2100_AUG, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_AUG_sd, "DEF_ENSEMBLE_rcp26_2071_2100_AUG_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_AUG_sd <- calc(DEF_ENSEMBLE_rcp45_2071_2100_AUG, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_AUG_sd, "DEF_ENSEMBLE_rcp45_2071_2100_AUG_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_AUG_sd <- calc(DEF_ENSEMBLE_rcp85_2071_2100_AUG, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_AUG_sd, "DEF_ENSEMBLE_rcp85_2071_2100_AUG_sd.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# AUG
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_AUG <- (((IMN_1961_1990_AUG_mean - DEF_ENSEMBLE_rcp26_2011_2040_AUG_mean) / IMN_1961_1990_AUG_mean)*-100)
writeRaster(PC_rcp26_2011_2040_AUG, "PC_rcp26_2011_2040_AUG.nc", format="GTiff", overwrite = TRUE)
PC_rcp45_2011_2040_AUG <- (((IMN_1961_1990_AUG_mean - DEF_ENSEMBLE_rcp45_2011_2040_AUG_mean) / IMN_1961_1990_AUG_mean)*-100)
writeRaster(PC_rcp45_2011_2040_AUG, "PC_rcp45_2011_2040_AUG.nc", format="GTiff", overwrite = TRUE)
PC_rcp85_2011_2040_AUG <- (((IMN_1961_1990_AUG_mean - DEF_ENSEMBLE_rcp85_2011_2040_AUG_mean) / IMN_1961_1990_AUG_mean)*-100)
writeRaster(PC_rcp85_2011_2040_AUG, "PC_rcp85_2011_2040_AUG.nc", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_AUG <- (((IMN_1961_1990_AUG_mean - DEF_ENSEMBLE_rcp26_2041_2070_AUG_mean) / IMN_1961_1990_AUG_mean)*-100)
writeRaster(PC_rcp26_2041_2070_AUG, "PC_rcp26_2041_2070_AUG.nc", format="GTiff", overwrite = TRUE)
PC_rcp45_2041_2070_AUG <- (((IMN_1961_1990_AUG_mean - DEF_ENSEMBLE_rcp45_2041_2070_AUG_mean) / IMN_1961_1990_AUG_mean)*-100)
writeRaster(PC_rcp45_2041_2070_AUG, "PC_rcp45_2041_2070_AUG.nc", format="GTiff", overwrite = TRUE)
PC_rcp85_2041_2070_AUG <- (((IMN_1961_1990_AUG_mean - DEF_ENSEMBLE_rcp85_2041_2070_AUG_mean) / IMN_1961_1990_AUG_mean)*-100)
writeRaster(PC_rcp85_2041_2070_AUG, "PC_rcp85_2041_2070_AUG.nc", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_AUG <- (((IMN_1961_1990_AUG_mean - DEF_ENSEMBLE_rcp26_2071_2100_AUG_mean) / IMN_1961_1990_AUG_mean)*-100)
writeRaster(PC_rcp26_2071_2100_AUG, "PC_rcp26_2071_2100_AUG.nc", format="GTiff", overwrite = TRUE)
PC_rcp45_2071_2100_AUG <- (((IMN_1961_1990_AUG_mean - DEF_ENSEMBLE_rcp45_2071_2100_AUG_mean) / IMN_1961_1990_AUG_mean)*-100)
writeRaster(PC_rcp45_2071_2100_AUG, "PC_rcp45_2071_2100_AUG.nc", format="GTiff", overwrite = TRUE)
PC_rcp85_2071_2100_AUG <- (((IMN_1961_1990_AUG_mean - DEF_ENSEMBLE_rcp85_2071_2100_AUG_mean) / IMN_1961_1990_AUG_mean)*-100)
writeRaster(PC_rcp85_2071_2100_AUG, "PC_rcp85_2071_2100_AUG.nc", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# SEP
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IMN_1961_1990_SEP_mean <- calc(l.observed[[vs.SEP]], mean, na.rm=TRUE)
writeRaster(IMN_1961_1990_SEP_mean, "test_IMN_1961_1990_SEP_mean.tif", format="GTiff", overwrite = TRUE)

IMN_1961_1990_SEP_sd <- calc(l.observed[[vs.SEP]], sd, na.rm=TRUE)
writeRaster(IMN_1961_1990_SEP_sd, "test_IMN_1961_1990_SEP_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_SEP_mean <- calc(DEF_ENSEMBLE_rcp26_2011_2040_SEP, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_SEP_mean, "DEF_ENSEMBLE_rcp26_2011_2040_SEP_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_SEP_mean <- calc(DEF_ENSEMBLE_rcp45_2011_2040_SEP, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_SEP_mean, "DEF_ENSEMBLE_rcp45_2011_2040_SEP_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_SEP_mean <- calc(DEF_ENSEMBLE_rcp85_2011_2040_SEP, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_SEP_mean, "DEF_ENSEMBLE_rcp85_2011_2040_SEP_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_SEP_sd <- calc(DEF_ENSEMBLE_rcp26_2011_2040_SEP, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_SEP_sd, "DEF_ENSEMBLE_rcp26_2011_2040_SEP_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_SEP_sd <- calc(DEF_ENSEMBLE_rcp45_2011_2040_SEP, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_SEP_sd, "DEF_ENSEMBLE_rcp45_2011_2040_SEP_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_SEP_sd <- calc(DEF_ENSEMBLE_rcp85_2011_2040_SEP, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_SEP_sd, "DEF_ENSEMBLE_rcp85_2011_2040_SEP_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_SEP_mean <- calc(DEF_ENSEMBLE_rcp26_2041_2070_SEP, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_SEP_mean, "DEF_ENSEMBLE_rcp26_2041_2070_SEP_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_SEP_mean <- calc(DEF_ENSEMBLE_rcp45_2041_2070_SEP, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_SEP_mean, "DEF_ENSEMBLE_rcp45_2041_2070_SEP_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_SEP_mean <- calc(DEF_ENSEMBLE_rcp85_2041_2070_SEP, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_SEP_mean, "DEF_ENSEMBLE_rcp85_2041_2070_SEP_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_SEP_sd <- calc(DEF_ENSEMBLE_rcp26_2041_2070_SEP, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_SEP_sd, "DEF_ENSEMBLE_rcp26_2041_2070_SEP_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_SEP_sd <- calc(DEF_ENSEMBLE_rcp45_2041_2070_SEP, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_SEP_sd, "DEF_ENSEMBLE_rcp45_2041_2070_SEP_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_SEP_sd <- calc(DEF_ENSEMBLE_rcp85_2041_2070_SEP, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_SEP_sd, "DEF_ENSEMBLE_rcp85_2041_2070_SEP_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_SEP_mean <- calc(DEF_ENSEMBLE_rcp26_2071_2100_SEP, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_SEP_mean, "DEF_ENSEMBLE_rcp26_2071_2100_SEP_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_SEP_mean <- calc(DEF_ENSEMBLE_rcp45_2071_2100_SEP, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_SEP_mean, "DEF_ENSEMBLE_rcp45_2071_2100_SEP_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_SEP_mean <- calc(DEF_ENSEMBLE_rcp85_2071_2100_SEP, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_SEP_mean, "DEF_ENSEMBLE_rcp85_2071_2100_SEP_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_SEP_sd <- calc(DEF_ENSEMBLE_rcp26_2071_2100_SEP, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_SEP_sd, "DEF_ENSEMBLE_rcp26_2071_2100_SEP_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_SEP_sd <- calc(DEF_ENSEMBLE_rcp45_2071_2100_SEP, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_SEP_sd, "DEF_ENSEMBLE_rcp45_2071_2100_SEP_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_SEP_sd <- calc(DEF_ENSEMBLE_rcp85_2071_2100_SEP, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_SEP_sd, "DEF_ENSEMBLE_rcp85_2071_2100_SEP_sd.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# SEP
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_SEP <- (((IMN_1961_1990_SEP_mean - DEF_ENSEMBLE_rcp26_2011_2040_SEP_mean) / IMN_1961_1990_SEP_mean)*-100)
writeRaster(PC_rcp26_2011_2040_SEP, "PC_rcp26_2011_2040_SEP.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2011_2040_SEP <- (((IMN_1961_1990_SEP_mean - DEF_ENSEMBLE_rcp45_2011_2040_SEP_mean) / IMN_1961_1990_SEP_mean)*-100)
writeRaster(PC_rcp45_2011_2040_SEP, "PC_rcp45_2011_2040_SEP.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2011_2040_SEP <- (((IMN_1961_1990_SEP_mean - DEF_ENSEMBLE_rcp85_2011_2040_SEP_mean) / IMN_1961_1990_SEP_mean)*-100)
writeRaster(PC_rcp85_2011_2040_SEP, "PC_rcp85_2011_2040_SEP.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_SEP <- (((IMN_1961_1990_SEP_mean - DEF_ENSEMBLE_rcp26_2041_2070_SEP_mean) / IMN_1961_1990_SEP_mean)*-100)
writeRaster(PC_rcp26_2041_2070_SEP, "PC_rcp26_2041_2070_SEP.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2041_2070_SEP <- (((IMN_1961_1990_SEP_mean - DEF_ENSEMBLE_rcp45_2041_2070_SEP_mean) / IMN_1961_1990_SEP_mean)*-100)
writeRaster(PC_rcp45_2041_2070_SEP, "PC_rcp45_2041_2070_SEP.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2041_2070_SEP <- (((IMN_1961_1990_SEP_mean - DEF_ENSEMBLE_rcp85_2041_2070_SEP_mean) / IMN_1961_1990_SEP_mean)*-100)
writeRaster(PC_rcp85_2041_2070_SEP, "PC_rcp85_2041_2070_SEP.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_SEP <- (((IMN_1961_1990_SEP_mean - DEF_ENSEMBLE_rcp26_2071_2100_SEP_mean) / IMN_1961_1990_SEP_mean)*-100)
writeRaster(PC_rcp26_2071_2100_SEP, "PC_rcp26_2071_2100_SEP.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2071_2100_SEP <- (((IMN_1961_1990_SEP_mean - DEF_ENSEMBLE_rcp45_2071_2100_SEP_mean) / IMN_1961_1990_SEP_mean)*-100)
writeRaster(PC_rcp45_2071_2100_SEP, "PC_rcp45_2071_2100_SEP.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2071_2100_SEP <- (((IMN_1961_1990_SEP_mean - DEF_ENSEMBLE_rcp85_2071_2100_SEP_mean) / IMN_1961_1990_SEP_mean)*-100)
writeRaster(PC_rcp85_2071_2100_SEP, "PC_rcp85_2071_2100_SEP.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# OCT
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IMN_1961_1990_OCT_mean <- calc(l.observed[[vs.OCT]], mean, na.rm=TRUE)
writeRaster(IMN_1961_1990_OCT_mean, "test_IMN_1961_1990_OCT_mean.tif", format="GTiff", overwrite = TRUE)

IMN_1961_1990_OCT_sd <- calc(l.observed[[vs.OCT]], sd, na.rm=TRUE)
writeRaster(IMN_1961_1990_OCT_sd, "test_IMN_1961_1990_OCT_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_OCT_mean <- calc(DEF_ENSEMBLE_rcp26_2011_2040_OCT, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_OCT_mean, "DEF_ENSEMBLE_rcp26_2011_2040_OCT_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_OCT_mean <- calc(DEF_ENSEMBLE_rcp45_2011_2040_OCT, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_OCT_mean, "DEF_ENSEMBLE_rcp45_2011_2040_OCT_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_OCT_mean <- calc(DEF_ENSEMBLE_rcp85_2011_2040_OCT, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_OCT_mean, "DEF_ENSEMBLE_rcp85_2011_2040_OCT_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_OCT_sd <- calc(DEF_ENSEMBLE_rcp26_2011_2040_OCT, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_OCT_sd, "DEF_ENSEMBLE_rcp26_2011_2040_OCT_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_OCT_sd <- calc(DEF_ENSEMBLE_rcp45_2011_2040_OCT, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_OCT_sd, "DEF_ENSEMBLE_rcp45_2011_2040_OCT_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_OCT_sd <- calc(DEF_ENSEMBLE_rcp85_2011_2040_OCT, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_OCT_sd, "DEF_ENSEMBLE_rcp85_2011_2040_OCT_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_OCT_mean <- calc(DEF_ENSEMBLE_rcp26_2041_2070_OCT, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_OCT_mean, "DEF_ENSEMBLE_rcp26_2041_2070_OCT_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_OCT_mean <- calc(DEF_ENSEMBLE_rcp45_2041_2070_OCT, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_OCT_mean, "DEF_ENSEMBLE_rcp45_2041_2070_OCT_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_OCT_mean <- calc(DEF_ENSEMBLE_rcp85_2041_2070_OCT, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_OCT_mean, "DEF_ENSEMBLE_rcp85_2041_2070_OCT_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_OCT_sd <- calc(DEF_ENSEMBLE_rcp26_2041_2070_OCT, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_OCT_sd, "DEF_ENSEMBLE_rcp26_2041_2070_OCT_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_OCT_sd <- calc(DEF_ENSEMBLE_rcp45_2041_2070_OCT, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_OCT_sd, "DEF_ENSEMBLE_rcp45_2041_2070_OCT_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_OCT_sd <- calc(DEF_ENSEMBLE_rcp85_2041_2070_OCT, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_OCT_sd, "DEF_ENSEMBLE_rcp85_2041_2070_OCT_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_OCT_mean <- calc(DEF_ENSEMBLE_rcp26_2071_2100_OCT, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_OCT_mean, "DEF_ENSEMBLE_rcp26_2071_2100_OCT_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_OCT_mean <- calc(DEF_ENSEMBLE_rcp45_2071_2100_OCT, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_OCT_mean, "DEF_ENSEMBLE_rcp45_2071_2100_OCT_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_OCT_mean <- calc(DEF_ENSEMBLE_rcp85_2071_2100_OCT, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_OCT_mean, "DEF_ENSEMBLE_rcp85_2071_2100_OCT_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_OCT_sd <- calc(DEF_ENSEMBLE_rcp26_2071_2100_OCT, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_OCT_sd, "DEF_ENSEMBLE_rcp26_2071_2100_OCT_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_OCT_sd <- calc(DEF_ENSEMBLE_rcp45_2071_2100_OCT, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_OCT_sd, "DEF_ENSEMBLE_rcp45_2071_2100_OCT_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_OCT_sd <- calc(DEF_ENSEMBLE_rcp85_2071_2100_OCT, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_OCT_sd, "DEF_ENSEMBLE_rcp85_2071_2100_OCT_sd.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# OCT
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_OCT <- (((IMN_1961_1990_OCT_mean - DEF_ENSEMBLE_rcp26_2011_2040_OCT_mean) / IMN_1961_1990_OCT_mean)*-100)
writeRaster(PC_rcp26_2011_2040_OCT, "PC_rcp26_2011_2040_OCT.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2011_2040_OCT <- (((IMN_1961_1990_OCT_mean - DEF_ENSEMBLE_rcp45_2011_2040_OCT_mean) / IMN_1961_1990_OCT_mean)*-100)
writeRaster(PC_rcp45_2011_2040_OCT, "PC_rcp45_2011_2040_OCT.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2011_2040_OCT <- (((IMN_1961_1990_OCT_mean - DEF_ENSEMBLE_rcp85_2011_2040_OCT_mean) / IMN_1961_1990_OCT_mean)*-100)
writeRaster(PC_rcp85_2011_2040_OCT, "PC_rcp85_2011_2040_OCT.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_OCT <- (((IMN_1961_1990_OCT_mean - DEF_ENSEMBLE_rcp26_2041_2070_OCT_mean) / IMN_1961_1990_OCT_mean)*-100)
writeRaster(PC_rcp26_2041_2070_OCT, "PC_rcp26_2041_2070_OCT.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2041_2070_OCT <- (((IMN_1961_1990_OCT_mean - DEF_ENSEMBLE_rcp45_2041_2070_OCT_mean) / IMN_1961_1990_OCT_mean)*-100)
writeRaster(PC_rcp45_2041_2070_OCT, "PC_rcp45_2041_2070_OCT.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2041_2070_OCT <- (((IMN_1961_1990_OCT_mean - DEF_ENSEMBLE_rcp85_2041_2070_OCT_mean) / IMN_1961_1990_OCT_mean)*-100)
writeRaster(PC_rcp85_2041_2070_OCT, "PC_rcp85_2041_2070_OCT.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_OCT <- (((IMN_1961_1990_OCT_mean - DEF_ENSEMBLE_rcp26_2071_2100_OCT_mean) / IMN_1961_1990_OCT_mean)*-100)
writeRaster(PC_rcp26_2071_2100_OCT, "PC_rcp26_2071_2100_OCT.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2071_2100_OCT <- (((IMN_1961_1990_OCT_mean - DEF_ENSEMBLE_rcp45_2071_2100_OCT_mean) / IMN_1961_1990_OCT_mean)*-100)
writeRaster(PC_rcp45_2071_2100_OCT, "PC_rcp45_2071_2100_OCT.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2071_2100_OCT <- (((IMN_1961_1990_OCT_mean - DEF_ENSEMBLE_rcp85_2071_2100_OCT_mean) / IMN_1961_1990_OCT_mean)*-100)
writeRaster(PC_rcp85_2071_2100_OCT, "PC_rcp85_2071_2100_OCT.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# NOV
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IMN_1961_1990_NOV_mean <- calc(l.observed[[vs.NOV]], mean, na.rm=TRUE)
writeRaster(IMN_1961_1990_NOV_mean, "test_IMN_1961_1990_NOV_mean.tif", format="GTiff", overwrite = TRUE)

IMN_1961_1990_NOV_sd <- calc(l.observed[[vs.NOV]], sd, na.rm=TRUE)
writeRaster(IMN_1961_1990_NOV_sd, "test_IMN_1961_1990_NOV_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_NOV_mean <- calc(DEF_ENSEMBLE_rcp26_2011_2040_NOV, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_NOV_mean, "DEF_ENSEMBLE_rcp26_2011_2040_NOV_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_NOV_mean <- calc(DEF_ENSEMBLE_rcp45_2011_2040_NOV, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_NOV_mean, "DEF_ENSEMBLE_rcp45_2011_2040_NOV_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_NOV_mean <- calc(DEF_ENSEMBLE_rcp85_2011_2040_NOV, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_NOV_mean, "DEF_ENSEMBLE_rcp85_2011_2040_NOV_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_NOV_sd <- calc(DEF_ENSEMBLE_rcp26_2011_2040_NOV, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_NOV_sd, "DEF_ENSEMBLE_rcp26_2011_2040_NOV_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_NOV_sd <- calc(DEF_ENSEMBLE_rcp45_2011_2040_NOV, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_NOV_sd, "DEF_ENSEMBLE_rcp45_2011_2040_NOV_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_NOV_sd <- calc(DEF_ENSEMBLE_rcp85_2011_2040_NOV, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_NOV_sd, "DEF_ENSEMBLE_rcp85_2011_2040_NOV_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_NOV_mean <- calc(DEF_ENSEMBLE_rcp26_2041_2070_NOV, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_NOV_mean, "DEF_ENSEMBLE_rcp26_2041_2070_NOV_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_NOV_mean <- calc(DEF_ENSEMBLE_rcp45_2041_2070_NOV, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_NOV_mean, "DEF_ENSEMBLE_rcp45_2041_2070_NOV_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_NOV_mean <- calc(DEF_ENSEMBLE_rcp85_2041_2070_NOV, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_NOV_mean, "DEF_ENSEMBLE_rcp85_2041_2070_NOV_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_NOV_sd <- calc(DEF_ENSEMBLE_rcp26_2041_2070_NOV, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_NOV_sd, "DEF_ENSEMBLE_rcp26_2041_2070_NOV_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_NOV_sd <- calc(DEF_ENSEMBLE_rcp45_2041_2070_NOV, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_NOV_sd, "DEF_ENSEMBLE_rcp45_2041_2070_NOV_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_NOV_sd <- calc(DEF_ENSEMBLE_rcp85_2041_2070_NOV, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_NOV_sd, "DEF_ENSEMBLE_rcp85_2041_2070_NOV_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_NOV_mean <- calc(DEF_ENSEMBLE_rcp26_2071_2100_NOV, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_NOV_mean, "DEF_ENSEMBLE_rcp26_2071_2100_NOV_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_NOV_mean <- calc(DEF_ENSEMBLE_rcp45_2071_2100_NOV, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_NOV_mean, "DEF_ENSEMBLE_rcp45_2071_2100_NOV_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_NOV_mean <- calc(DEF_ENSEMBLE_rcp85_2071_2100_NOV, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_NOV_mean, "DEF_ENSEMBLE_rcp85_2071_2100_NOV_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_NOV_sd <- calc(DEF_ENSEMBLE_rcp26_2071_2100_NOV, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_NOV_sd, "DEF_ENSEMBLE_rcp26_2071_2100_NOV_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_NOV_sd <- calc(DEF_ENSEMBLE_rcp45_2071_2100_NOV, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_NOV_sd, "DEF_ENSEMBLE_rcp45_2071_2100_NOV_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_NOV_sd <- calc(DEF_ENSEMBLE_rcp85_2071_2100_NOV, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_NOV_sd, "DEF_ENSEMBLE_rcp85_2071_2100_NOV_sd.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# NOV
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_NOV <- (((IMN_1961_1990_NOV_mean - DEF_ENSEMBLE_rcp26_2011_2040_NOV_mean) / IMN_1961_1990_NOV_mean)*-100)
writeRaster(PC_rcp26_2011_2040_NOV, "PC_rcp26_2011_2040_NOV.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2011_2040_NOV <- (((IMN_1961_1990_NOV_mean - DEF_ENSEMBLE_rcp45_2011_2040_NOV_mean) / IMN_1961_1990_NOV_mean)*-100)
writeRaster(PC_rcp45_2011_2040_NOV, "PC_rcp45_2011_2040_NOV.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2011_2040_NOV <- (((IMN_1961_1990_NOV_mean - DEF_ENSEMBLE_rcp85_2011_2040_NOV_mean) / IMN_1961_1990_NOV_mean)*-100)
writeRaster(PC_rcp85_2011_2040_NOV, "PC_rcp85_2011_2040_NOV.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_NOV <- (((IMN_1961_1990_NOV_mean - DEF_ENSEMBLE_rcp26_2041_2070_NOV_mean) / IMN_1961_1990_NOV_mean)*-100)
writeRaster(PC_rcp26_2041_2070_NOV, "PC_rcp26_2041_2070_NOV.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2041_2070_NOV <- (((IMN_1961_1990_NOV_mean - DEF_ENSEMBLE_rcp45_2041_2070_NOV_mean) / IMN_1961_1990_NOV_mean)*-100)
writeRaster(PC_rcp45_2041_2070_NOV, "PC_rcp45_2041_2070_NOV.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2041_2070_NOV <- (((IMN_1961_1990_NOV_mean - DEF_ENSEMBLE_rcp85_2041_2070_NOV_mean) / IMN_1961_1990_NOV_mean)*-100)
writeRaster(PC_rcp85_2041_2070_NOV, "PC_rcp85_2041_2070_NOV.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_NOV <- (((IMN_1961_1990_NOV_mean - DEF_ENSEMBLE_rcp26_2071_2100_NOV_mean) / IMN_1961_1990_NOV_mean)*-100)
writeRaster(PC_rcp26_2071_2100_NOV, "PC_rcp26_2071_2100_NOV.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2071_2100_NOV <- (((IMN_1961_1990_NOV_mean - DEF_ENSEMBLE_rcp45_2071_2100_NOV_mean) / IMN_1961_1990_NOV_mean)*-100)
writeRaster(PC_rcp45_2071_2100_NOV, "PC_rcp45_2071_2100_NOV.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2071_2100_NOV <- (((IMN_1961_1990_NOV_mean - DEF_ENSEMBLE_rcp85_2071_2100_NOV_mean) / IMN_1961_1990_NOV_mean)*-100)
writeRaster(PC_rcp85_2071_2100_NOV, "PC_rcp85_2071_2100_NOV.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# DEC
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
IMN_1961_1990_DEC_mean <- calc(l.observed[[vs.DEC]], mean, na.rm=TRUE)
writeRaster(IMN_1961_1990_DEC_mean, "test_IMN_1961_1990_DEC_mean.tif", format="GTiff", overwrite = TRUE)

IMN_1961_1990_DEC_sd <- calc(l.observed[[vs.DEC]], sd, na.rm=TRUE)
writeRaster(IMN_1961_1990_DEC_sd, "test_IMN_1961_1990_DEC_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_DEC_mean <- calc(DEF_ENSEMBLE_rcp26_2011_2040_DEC, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_DEC_mean, "DEF_ENSEMBLE_rcp26_2011_2040_DEC_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_DEC_mean <- calc(DEF_ENSEMBLE_rcp45_2011_2040_DEC, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_DEC_mean, "DEF_ENSEMBLE_rcp45_2011_2040_DEC_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_DEC_mean <- calc(DEF_ENSEMBLE_rcp85_2011_2040_DEC, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_DEC_mean, "DEF_ENSEMBLE_rcp85_2011_2040_DEC_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2011_2040_DEC_sd <- calc(DEF_ENSEMBLE_rcp26_2011_2040_DEC, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2011_2040_DEC_sd, "DEF_ENSEMBLE_rcp26_2011_2040_DEC_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2011_2040_DEC_sd <- calc(DEF_ENSEMBLE_rcp45_2011_2040_DEC, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2011_2040_DEC_sd, "DEF_ENSEMBLE_rcp45_2011_2040_DEC_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2011_2040_DEC_sd <- calc(DEF_ENSEMBLE_rcp85_2011_2040_DEC, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2011_2040_DEC_sd, "DEF_ENSEMBLE_rcp85_2011_2040_DEC_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_DEC_mean <- calc(DEF_ENSEMBLE_rcp26_2041_2070_DEC, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_DEC_mean, "DEF_ENSEMBLE_rcp26_2041_2070_DEC_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_DEC_mean <- calc(DEF_ENSEMBLE_rcp45_2041_2070_DEC, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_DEC_mean, "DEF_ENSEMBLE_rcp45_2041_2070_DEC_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_DEC_mean <- calc(DEF_ENSEMBLE_rcp85_2041_2070_DEC, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_DEC_mean, "DEF_ENSEMBLE_rcp85_2041_2070_DEC_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2041_2070_DEC_sd <- calc(DEF_ENSEMBLE_rcp26_2041_2070_DEC, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2041_2070_DEC_sd, "DEF_ENSEMBLE_rcp26_2041_2070_DEC_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2041_2070_DEC_sd <- calc(DEF_ENSEMBLE_rcp45_2041_2070_DEC, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2041_2070_DEC_sd, "DEF_ENSEMBLE_rcp45_2041_2070_DEC_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2041_2070_DEC_sd <- calc(DEF_ENSEMBLE_rcp85_2041_2070_DEC, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2041_2070_DEC_sd, "DEF_ENSEMBLE_rcp85_2041_2070_DEC_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_DEC_mean <- calc(DEF_ENSEMBLE_rcp26_2071_2100_DEC, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_DEC_mean, "DEF_ENSEMBLE_rcp26_2071_2100_DEC_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_DEC_mean <- calc(DEF_ENSEMBLE_rcp45_2071_2100_DEC, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_DEC_mean, "DEF_ENSEMBLE_rcp45_2071_2100_DEC_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_DEC_mean <- calc(DEF_ENSEMBLE_rcp85_2071_2100_DEC, mean, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_DEC_mean, "DEF_ENSEMBLE_rcp85_2071_2100_DEC_mean.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp26_2071_2100_DEC_sd <- calc(DEF_ENSEMBLE_rcp26_2071_2100_DEC, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp26_2071_2100_DEC_sd, "DEF_ENSEMBLE_rcp26_2071_2100_DEC_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp45_2071_2100_DEC_sd <- calc(DEF_ENSEMBLE_rcp45_2071_2100_DEC, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp45_2071_2100_DEC_sd, "DEF_ENSEMBLE_rcp45_2071_2100_DEC_sd.tif", format="GTiff", overwrite = TRUE)

DEF_ENSEMBLE_rcp85_2071_2100_DEC_sd <- calc(DEF_ENSEMBLE_rcp85_2071_2100_DEC, sd, na.rm=TRUE)
writeRaster(DEF_ENSEMBLE_rcp85_2071_2100_DEC_sd, "DEF_ENSEMBLE_rcp85_2071_2100_DEC_sd.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# DEC
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_DEC <- (((IMN_1961_1990_DEC_mean - DEF_ENSEMBLE_rcp26_2011_2040_DEC_mean) / IMN_1961_1990_DEC_mean)*-100)
writeRaster(PC_rcp26_2011_2040_DEC, "PC_rcp26_2011_2040_DEC.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2011_2040_DEC <- (((IMN_1961_1990_DEC_mean - DEF_ENSEMBLE_rcp45_2011_2040_DEC_mean) / IMN_1961_1990_DEC_mean)*-100)
writeRaster(PC_rcp45_2011_2040_DEC, "PC_rcp45_2011_2040_DEC.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2011_2040_DEC <- (((IMN_1961_1990_DEC_mean - DEF_ENSEMBLE_rcp85_2011_2040_DEC_mean) / IMN_1961_1990_DEC_mean)*-100)
writeRaster(PC_rcp85_2011_2040_DEC, "PC_rcp85_2011_2040_DEC.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_DEC <- (((IMN_1961_1990_DEC_mean - DEF_ENSEMBLE_rcp26_2041_2070_DEC_mean) / IMN_1961_1990_DEC_mean)*-100)
writeRaster(PC_rcp26_2041_2070_DEC, "PC_rcp26_2041_2070_DEC.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2041_2070_DEC <- (((IMN_1961_1990_DEC_mean - DEF_ENSEMBLE_rcp45_2041_2070_DEC_mean) / IMN_1961_1990_DEC_mean)*-100)
writeRaster(PC_rcp45_2041_2070_DEC, "PC_rcp45_2041_2070_DEC.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2041_2070_DEC <- (((IMN_1961_1990_DEC_mean - DEF_ENSEMBLE_rcp85_2041_2070_DEC_mean) / IMN_1961_1990_DEC_mean)*-100)
writeRaster(PC_rcp85_2041_2070_DEC, "PC_rcp85_2041_2070_DEC.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_DEC <- (((IMN_1961_1990_DEC_mean - DEF_ENSEMBLE_rcp26_2071_2100_DEC_mean) / IMN_1961_1990_DEC_mean)*-100)
writeRaster(PC_rcp26_2071_2100_DEC, "PC_rcp26_2071_2100_DEC.tif", format="GTiff", overwrite = TRUE)
PC_rcp45_2071_2100_DEC <- (((IMN_1961_1990_DEC_mean - DEF_ENSEMBLE_rcp45_2071_2100_DEC_mean) / IMN_1961_1990_DEC_mean)*-100)
writeRaster(PC_rcp45_2071_2100_DEC, "PC_rcp45_2071_2100_DEC.tif", format="GTiff", overwrite = TRUE)
PC_rcp85_2071_2100_DEC <- (((IMN_1961_1990_DEC_mean - DEF_ENSEMBLE_rcp85_2071_2100_DEC_mean) / IMN_1961_1990_DEC_mean)*-100)
writeRaster(PC_rcp85_2071_2100_DEC, "PC_rcp85_2071_2100_DEC.tif", format="GTiff", overwrite = TRUE)
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------

# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Seasonal Analysis
# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# Seasonal DJF
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_DJF <- stack(PC_rcp26_2011_2040_DEC, PC_rcp26_2011_2040_JAN, PC_rcp26_2011_2040_FEB)
PC_rcp26_2011_2040_DJF <- calc(PC_rcp26_2011_2040_DJF, mean, na.rm=TRUE)
writeRaster(PC_rcp26_2011_2040_DJF, "PC_rcp26_2011_2040_DJF.tif", format="GTiff", overwrite = TRUE)

PC_rcp45_2011_2040_DJF <- stack(PC_rcp45_2011_2040_DEC, PC_rcp45_2011_2040_JAN, PC_rcp45_2011_2040_FEB)
PC_rcp45_2011_2040_DJF <- calc(PC_rcp45_2011_2040_DJF, mean, na.rm=TRUE)
writeRaster(PC_rcp45_2011_2040_DJF, "PC_rcp45_2011_2040_DJF.tif", format="GTiff", overwrite = TRUE)

PC_rcp85_2011_2040_DJF <- stack(PC_rcp85_2011_2040_DEC, PC_rcp85_2011_2040_JAN, PC_rcp85_2011_2040_FEB)
PC_rcp85_2011_2040_DJF <- calc(PC_rcp85_2011_2040_DJF, mean, na.rm=TRUE)
writeRaster(PC_rcp85_2011_2040_DJF, "PC_rcp85_2011_2040_DJF.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_DJF <- stack(PC_rcp26_2041_2070_DEC, PC_rcp26_2041_2070_JAN, PC_rcp26_2041_2070_FEB)
PC_rcp26_2041_2070_DJF <- calc(PC_rcp26_2041_2070_DJF, mean, na.rm=TRUE)
writeRaster(PC_rcp26_2041_2070_DJF, "PC_rcp26_2041_2070_DJF.tif", format="GTiff", overwrite = TRUE)

PC_rcp45_2041_2070_DJF <- stack(PC_rcp45_2041_2070_DEC, PC_rcp45_2041_2070_JAN, PC_rcp45_2041_2070_FEB)
PC_rcp45_2041_2070_DJF <- calc(PC_rcp45_2041_2070_DJF, mean, na.rm=TRUE)
writeRaster(PC_rcp45_2041_2070_DJF, "PC_rcp45_2041_2070_DJF.tif", format="GTiff", overwrite = TRUE)

PC_rcp85_2041_2070_DJF <- stack(PC_rcp85_2041_2070_DEC, PC_rcp85_2041_2070_JAN, PC_rcp85_2041_2070_FEB)
PC_rcp85_2041_2070_DJF <- calc(PC_rcp85_2041_2070_DJF, mean, na.rm=TRUE)
writeRaster(PC_rcp85_2041_2070_DJF, "PC_rcp85_2041_2070_DJF.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_DJF <- stack(PC_rcp26_2071_2100_DEC, PC_rcp26_2071_2100_JAN, PC_rcp26_2071_2100_FEB)
PC_rcp26_2071_2100_DJF <- calc(PC_rcp26_2071_2100_DJF, mean, na.rm=TRUE)
writeRaster(PC_rcp26_2071_2100_DJF, "PC_rcp26_2071_2100_DJF.tif", format="GTiff", overwrite = TRUE)

PC_rcp45_2071_2100_DJF <- stack(PC_rcp45_2071_2100_DEC, PC_rcp45_2071_2100_JAN, PC_rcp45_2071_2100_FEB)
PC_rcp45_2071_2100_DJF <- calc(PC_rcp45_2071_2100_DJF, mean, na.rm=TRUE)
writeRaster(PC_rcp45_2071_2100_DJF, "PC_rcp45_2071_2100_DJF.tif", format="GTiff", overwrite = TRUE)

PC_rcp85_2071_2100_DJF <- stack(PC_rcp85_2071_2100_DEC, PC_rcp85_2071_2100_JAN, PC_rcp85_2071_2100_FEB)
PC_rcp85_2071_2100_DJF <- calc(PC_rcp85_2071_2100_DJF, mean, na.rm=TRUE)
writeRaster(PC_rcp85_2071_2100_DJF, "PC_rcp85_2071_2100_DJF.tif", format="GTiff", overwrite = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# Seasonal MAM
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_MAM <- stack(PC_rcp26_2011_2040_MAR, PC_rcp26_2011_2040_APR, PC_rcp26_2011_2040_MAY)
PC_rcp26_2011_2040_MAM <- calc(PC_rcp26_2011_2040_MAM, mean, na.rm=TRUE)
writeRaster(PC_rcp26_2011_2040_MAM, "PC_rcp26_2011_2040_MAM.tif", format="GTiff", overwrite = TRUE)

PC_rcp45_2011_2040_MAM <- stack(PC_rcp45_2011_2040_MAR, PC_rcp45_2011_2040_APR, PC_rcp45_2011_2040_MAY)
PC_rcp45_2011_2040_MAM <- calc(PC_rcp45_2011_2040_MAM, mean, na.rm=TRUE)
writeRaster(PC_rcp45_2011_2040_MAM, "PC_rcp45_2011_2040_MAM.tif", format="GTiff", overwrite = TRUE)

PC_rcp85_2011_2040_MAM <- stack(PC_rcp85_2011_2040_MAR, PC_rcp85_2011_2040_APR, PC_rcp85_2011_2040_MAY)
PC_rcp85_2011_2040_MAM <- calc(PC_rcp85_2011_2040_MAM, mean, na.rm=TRUE)
writeRaster(PC_rcp85_2011_2040_MAM, "PC_rcp85_2011_2040_MAM.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_MAM <- stack(PC_rcp26_2041_2070_MAR, PC_rcp26_2041_2070_APR, PC_rcp26_2041_2070_MAY)
PC_rcp26_2041_2070_MAM <- calc(PC_rcp26_2041_2070_MAM, mean, na.rm=TRUE)
writeRaster(PC_rcp26_2041_2070_MAM, "PC_rcp26_2041_2070_MAM.tif", format="GTiff", overwrite = TRUE)

PC_rcp45_2041_2070_MAM <- stack(PC_rcp45_2041_2070_MAR, PC_rcp45_2041_2070_APR, PC_rcp45_2041_2070_MAY)
PC_rcp45_2041_2070_MAM <- calc(PC_rcp45_2041_2070_MAM, mean, na.rm=TRUE)
writeRaster(PC_rcp45_2041_2070_MAM, "PC_rcp45_2041_2070_MAM.tif", format="GTiff", overwrite = TRUE)

PC_rcp85_2041_2070_MAM <- stack(PC_rcp85_2041_2070_MAR, PC_rcp85_2041_2070_APR, PC_rcp85_2041_2070_MAY)
PC_rcp85_2041_2070_MAM <- calc(PC_rcp85_2041_2070_MAM, mean, na.rm=TRUE)
writeRaster(PC_rcp85_2041_2070_MAM, "PC_rcp85_2041_2070_MAM.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_MAM <- stack(PC_rcp26_2071_2100_MAR, PC_rcp26_2071_2100_APR, PC_rcp26_2071_2100_MAY)
PC_rcp26_2071_2100_MAM <- calc(PC_rcp26_2071_2100_MAM, mean, na.rm=TRUE)
writeRaster(PC_rcp26_2071_2100_MAM, "PC_rcp26_2071_2100_MAM.tif", format="GTiff", overwrite = TRUE)

PC_rcp45_2071_2100_MAM <- stack(PC_rcp45_2071_2100_MAR, PC_rcp45_2071_2100_APR, PC_rcp45_2071_2100_MAY)
PC_rcp45_2071_2100_MAM <- calc(PC_rcp45_2071_2100_MAM, mean, na.rm=TRUE)
writeRaster(PC_rcp45_2071_2100_MAM, "PC_rcp45_2071_2100_MAM.tif", format="GTiff", overwrite = TRUE)

PC_rcp85_2071_2100_MAM <- stack(PC_rcp85_2071_2100_MAR, PC_rcp85_2071_2100_APR, PC_rcp85_2071_2100_MAY)
PC_rcp85_2071_2100_MAM <- calc(PC_rcp85_2071_2100_MAM, mean, na.rm=TRUE)
writeRaster(PC_rcp85_2071_2100_MAM, "PC_rcp85_2071_2100_MAM.tif", format="GTiff", overwrite = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# Seasonal JJA
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_JJA <- stack(PC_rcp26_2011_2040_JUN, PC_rcp26_2011_2040_JUL, PC_rcp26_2011_2040_AUG)
PC_rcp26_2011_2040_JJA <- calc(PC_rcp26_2011_2040_JJA, mean, na.rm=TRUE)
writeRaster(PC_rcp26_2011_2040_JJA, "PC_rcp26_2011_2040_JJA.tif", format="GTiff", overwrite = TRUE)

PC_rcp45_2011_2040_JJA <- stack(PC_rcp45_2011_2040_JUN, PC_rcp45_2011_2040_JUL, PC_rcp45_2011_2040_AUG)
PC_rcp45_2011_2040_JJA <- calc(PC_rcp45_2011_2040_JJA, mean, na.rm=TRUE)
writeRaster(PC_rcp45_2011_2040_JJA, "PC_rcp45_2011_2040_JJA.tif", format="GTiff", overwrite = TRUE)

PC_rcp85_2011_2040_JJA <- stack(PC_rcp85_2011_2040_JUN, PC_rcp85_2011_2040_JUL, PC_rcp85_2011_2040_AUG)
PC_rcp85_2011_2040_JJA <- calc(PC_rcp85_2011_2040_JJA, mean, na.rm=TRUE)
writeRaster(PC_rcp85_2011_2040_JJA, "PC_rcp85_2011_2040_JJA.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_JJA <- stack(PC_rcp26_2041_2070_JUN, PC_rcp26_2041_2070_JUL, PC_rcp26_2041_2070_AUG)
PC_rcp26_2041_2070_JJA <- calc(PC_rcp26_2041_2070_JJA, mean, na.rm=TRUE)
writeRaster(PC_rcp26_2041_2070_JJA, "PC_rcp26_2041_2070_JJA.tif", format="GTiff", overwrite = TRUE)

PC_rcp45_2041_2070_JJA <- stack(PC_rcp45_2041_2070_JUN, PC_rcp45_2041_2070_JUL, PC_rcp45_2041_2070_AUG)
PC_rcp45_2041_2070_JJA <- calc(PC_rcp45_2041_2070_JJA, mean, na.rm=TRUE)
writeRaster(PC_rcp45_2041_2070_JJA, "PC_rcp45_2041_2070_JJA.tif", format="GTiff", overwrite = TRUE)

PC_rcp85_2041_2070_JJA <- stack(PC_rcp85_2041_2070_JUN, PC_rcp85_2041_2070_JUL, PC_rcp85_2041_2070_AUG)
PC_rcp85_2041_2070_JJA <- calc(PC_rcp85_2041_2070_JJA, mean, na.rm=TRUE)
writeRaster(PC_rcp85_2041_2070_JJA, "PC_rcp85_2041_2070_JJA.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_JJA <- stack(PC_rcp26_2071_2100_JUN, PC_rcp26_2071_2100_JUL, PC_rcp26_2071_2100_AUG)
PC_rcp26_2071_2100_JJA <- calc(PC_rcp26_2071_2100_JJA, mean, na.rm=TRUE)
writeRaster(PC_rcp26_2071_2100_JJA, "PC_rcp26_2071_2100_JJA.tif", format="GTiff", overwrite = TRUE)

PC_rcp45_2071_2100_JJA <- stack(PC_rcp45_2071_2100_JUN, PC_rcp45_2071_2100_JUL, PC_rcp45_2071_2100_AUG)
PC_rcp45_2071_2100_JJA <- calc(PC_rcp45_2071_2100_JJA, mean, na.rm=TRUE)
writeRaster(PC_rcp45_2071_2100_JJA, "PC_rcp45_2071_2100_JJA.tif", format="GTiff", overwrite = TRUE)

PC_rcp85_2071_2100_JJA <- stack(PC_rcp85_2071_2100_JUN, PC_rcp85_2071_2100_JUL, PC_rcp85_2071_2100_AUG)
PC_rcp85_2071_2100_JJA <- calc(PC_rcp85_2071_2100_JJA, mean, na.rm=TRUE)
writeRaster(PC_rcp85_2071_2100_JJA, "PC_rcp85_2071_2100_JJA.tif", format="GTiff", overwrite = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# Seasonal SON
# Percent-Change (PC%) is calculated (modelled - observed)
# Positive values indicate WET change, whereas negative values indicate DRY change
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_rcp26_2011_2040_SON <- stack(PC_rcp26_2011_2040_SEP, PC_rcp26_2011_2040_OCT, PC_rcp26_2011_2040_NOV)
PC_rcp26_2011_2040_SON <- calc(PC_rcp26_2011_2040_SON, mean, na.rm=TRUE)
writeRaster(PC_rcp26_2011_2040_SON, "PC_rcp26_2011_2040_SON.tif", format="GTiff", overwrite = TRUE)

PC_rcp45_2011_2040_SON <- stack(PC_rcp45_2011_2040_SEP, PC_rcp45_2011_2040_OCT, PC_rcp45_2011_2040_NOV)
PC_rcp45_2011_2040_SON <- calc(PC_rcp45_2011_2040_SON, mean, na.rm=TRUE)
writeRaster(PC_rcp45_2011_2040_SON, "PC_rcp45_2011_2040_SON.tif", format="GTiff", overwrite = TRUE)

PC_rcp85_2011_2040_SON <- stack(PC_rcp85_2011_2040_SEP, PC_rcp85_2011_2040_OCT, PC_rcp85_2011_2040_NOV)
PC_rcp85_2011_2040_SON <- calc(PC_rcp85_2011_2040_SON, mean, na.rm=TRUE)
writeRaster(PC_rcp85_2011_2040_SON, "PC_rcp85_2011_2040_SON.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2041_2070_SON <- stack(PC_rcp26_2041_2070_SEP, PC_rcp26_2041_2070_OCT, PC_rcp26_2041_2070_NOV)
PC_rcp26_2041_2070_SON <- calc(PC_rcp26_2041_2070_SON, mean, na.rm=TRUE)
writeRaster(PC_rcp26_2041_2070_SON, "PC_rcp26_2041_2070_SON.tif", format="GTiff", overwrite = TRUE)

PC_rcp45_2041_2070_SON <- stack(PC_rcp45_2041_2070_SEP, PC_rcp45_2041_2070_OCT, PC_rcp45_2041_2070_NOV)
PC_rcp45_2041_2070_SON <- calc(PC_rcp45_2041_2070_SON, mean, na.rm=TRUE)
writeRaster(PC_rcp45_2041_2070_SON, "PC_rcp45_2041_2070_SON.tif", format="GTiff", overwrite = TRUE)

PC_rcp85_2041_2070_SON <- stack(PC_rcp85_2041_2070_SEP, PC_rcp85_2041_2070_OCT, PC_rcp85_2041_2070_NOV)
PC_rcp85_2041_2070_SON <- calc(PC_rcp85_2041_2070_SON, mean, na.rm=TRUE)
writeRaster(PC_rcp85_2041_2070_SON, "PC_rcp85_2041_2070_SON.tif", format="GTiff", overwrite = TRUE)

PC_rcp26_2071_2100_SON <- stack(PC_rcp26_2071_2100_SEP, PC_rcp26_2071_2100_OCT, PC_rcp26_2071_2100_NOV)
PC_rcp26_2071_2100_SON <- calc(PC_rcp26_2071_2100_SON, mean, na.rm=TRUE)
writeRaster(PC_rcp26_2071_2100_SON, "PC_rcp26_2071_2100_SON.tif", format="GTiff", overwrite = TRUE)

PC_rcp45_2071_2100_SON <- stack(PC_rcp45_2071_2100_SEP, PC_rcp45_2071_2100_OCT, PC_rcp45_2071_2100_NOV)
PC_rcp45_2071_2100_SON <- calc(PC_rcp45_2071_2100_SON, mean, na.rm=TRUE)
writeRaster(PC_rcp45_2071_2100_SON, "PC_rcp45_2071_2100_SON.tif", format="GTiff", overwrite = TRUE)

PC_rcp85_2071_2100_SON <- stack(PC_rcp85_2071_2100_SEP, PC_rcp85_2071_2100_OCT, PC_rcp85_2071_2100_NOV)
PC_rcp85_2071_2100_SON <- calc(PC_rcp85_2071_2100_SON, mean, na.rm=TRUE)
writeRaster(PC_rcp85_2071_2100_SON, "PC_rcp85_2071_2100_SON.tif", format="GTiff", overwrite = TRUE)

# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Seasonal Climatic Regions Masking
# ///////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

#----------------------------------------------------------------------------------------------------------------
# SUBBLOCK: ASC CR Climatic Regions Masking
#----------------------------------------------------------------------------------------------------------------
# North_Pacific = 3
# Central_Pacific = 2
# South_Pacific = 5
# Central = 4
# North = 1
# Caribbean = 6
#----------------------------------------------------------------------------------------------------------------

# A list container is created
list.CRCR <- NULL
list.CRCR <- list()

# Main loop is initiated
for (w in 1:6){
  
  # Climatic-regions CR blank raster is loaded
  blank <- paste("mod_region.asc", sep = "/")
  
  # The blank map is transformed into SpatialGridDataFrame
  blank.grid <- read.asciigrid(blank, as.image = FALSE)
  
  # The map is transformed into RasterLayer
  blank.raster <- raster(blank.grid)
  
  # CRTM05 projection is assigned to rasterbrick object
  crs(blank.raster) <- CRTM05
  
  # Climatic region is selected
  values(blank.raster)[values(blank.raster) != w] <- NA
  
  # A simple plot is requested
  #plot(blank.raster)
  
  #----------------------------------------------------------------------------------------------------------------
  # SUBBLOCK: Spatially-Averaged Mean Value
  #----------------------------------------------------------------------------------------------------------------
  
  #----------------------------------------------------------------------------------------------------------------
  # Season DJF
  #----------------------------------------------------------------------------------------------------------------
  rt.PC_rcp26_2011_2040_DJF <- mask(PC_rcp26_2011_2040_DJF, blank.raster)
  rt.PC_rcp26_2011_2040_DJF <- cellStats(rt.PC_rcp26_2011_2040_DJF, mean, na.rm = TRUE)
  
  rt.PC_rcp26_2041_2070_DJF <- mask(PC_rcp26_2041_2070_DJF, blank.raster)
  rt.PC_rcp26_2041_2070_DJF <- cellStats(rt.PC_rcp26_2041_2070_DJF, mean, na.rm = TRUE)
  
  rt.PC_rcp26_2071_2100_DJF <- mask(PC_rcp26_2071_2100_DJF, blank.raster)
  rt.PC_rcp26_2071_2100_DJF <- cellStats(rt.PC_rcp26_2071_2100_DJF, mean, na.rm = TRUE)
  
  rt.PC_rcp45_2011_2040_DJF <- mask(PC_rcp45_2011_2040_DJF, blank.raster)
  rt.PC_rcp45_2011_2040_DJF <- cellStats(rt.PC_rcp45_2011_2040_DJF, mean, na.rm = TRUE)
  
  rt.PC_rcp45_2041_2070_DJF <- mask(PC_rcp45_2041_2070_DJF, blank.raster)
  rt.PC_rcp45_2041_2070_DJF <- cellStats(rt.PC_rcp45_2041_2070_DJF, mean, na.rm = TRUE)
  
  rt.PC_rcp45_2071_2100_DJF <- mask(PC_rcp45_2071_2100_DJF, blank.raster)
  rt.PC_rcp45_2071_2100_DJF <- cellStats(rt.PC_rcp45_2071_2100_DJF, mean, na.rm = TRUE)
  
  rt.PC_rcp85_2011_2040_DJF <- mask(PC_rcp85_2011_2040_DJF, blank.raster)
  rt.PC_rcp85_2011_2040_DJF <- cellStats(rt.PC_rcp85_2011_2040_DJF, mean, na.rm = TRUE)
  
  rt.PC_rcp85_2041_2070_DJF <- mask(PC_rcp85_2041_2070_DJF, blank.raster)
  rt.PC_rcp85_2041_2070_DJF <- cellStats(rt.PC_rcp85_2041_2070_DJF, mean, na.rm = TRUE)
  
  rt.PC_rcp85_2071_2100_DJF <- mask(PC_rcp85_2071_2100_DJF, blank.raster)
  rt.PC_rcp85_2071_2100_DJF <- cellStats(rt.PC_rcp85_2071_2100_DJF, mean, na.rm = TRUE)
  
  #----------------------------------------------------------------------------------------------------------------
  # Season MAM
  #----------------------------------------------------------------------------------------------------------------
  rt.PC_rcp26_2011_2040_MAM <- mask(PC_rcp26_2011_2040_MAM, blank.raster)
  rt.PC_rcp26_2011_2040_MAM <- cellStats(rt.PC_rcp26_2011_2040_MAM, mean, na.rm = TRUE)
  
  rt.PC_rcp26_2041_2070_MAM <- mask(PC_rcp26_2041_2070_MAM, blank.raster)
  rt.PC_rcp26_2041_2070_MAM <- cellStats(rt.PC_rcp26_2041_2070_MAM, mean, na.rm = TRUE)
  
  rt.PC_rcp26_2071_2100_MAM <- mask(PC_rcp26_2071_2100_MAM, blank.raster)
  rt.PC_rcp26_2071_2100_MAM <- cellStats(rt.PC_rcp26_2071_2100_MAM, mean, na.rm = TRUE)
  
  rt.PC_rcp45_2011_2040_MAM <- mask(PC_rcp45_2011_2040_MAM, blank.raster)
  rt.PC_rcp45_2011_2040_MAM <- cellStats(rt.PC_rcp45_2011_2040_MAM, mean, na.rm = TRUE)
  
  rt.PC_rcp45_2041_2070_MAM <- mask(PC_rcp45_2041_2070_MAM, blank.raster)
  rt.PC_rcp45_2041_2070_MAM <- cellStats(rt.PC_rcp45_2041_2070_MAM, mean, na.rm = TRUE)
  
  rt.PC_rcp45_2071_2100_MAM <- mask(PC_rcp45_2071_2100_MAM, blank.raster)
  rt.PC_rcp45_2071_2100_MAM <- cellStats(rt.PC_rcp45_2071_2100_MAM, mean, na.rm = TRUE)
  
  rt.PC_rcp85_2011_2040_MAM <- mask(PC_rcp85_2011_2040_MAM, blank.raster)
  rt.PC_rcp85_2011_2040_MAM <- cellStats(rt.PC_rcp85_2011_2040_MAM, mean, na.rm = TRUE)
  
  rt.PC_rcp85_2041_2070_MAM <- mask(PC_rcp85_2041_2070_MAM, blank.raster)
  rt.PC_rcp85_2041_2070_MAM <- cellStats(rt.PC_rcp85_2041_2070_MAM, mean, na.rm = TRUE)
  
  rt.PC_rcp85_2071_2100_MAM <- mask(PC_rcp85_2071_2100_MAM, blank.raster)
  rt.PC_rcp85_2071_2100_MAM <- cellStats(rt.PC_rcp85_2071_2100_MAM, mean, na.rm = TRUE)
  
  #----------------------------------------------------------------------------------------------------------------
  # Season JJA
  #----------------------------------------------------------------------------------------------------------------
  rt.PC_rcp26_2011_2040_JJA <- mask(PC_rcp26_2011_2040_JJA, blank.raster)
  rt.PC_rcp26_2011_2040_JJA <- cellStats(rt.PC_rcp26_2011_2040_JJA, mean, na.rm = TRUE)
  
  rt.PC_rcp26_2041_2070_JJA <- mask(PC_rcp26_2041_2070_JJA, blank.raster)
  rt.PC_rcp26_2041_2070_JJA <- cellStats(rt.PC_rcp26_2041_2070_JJA, mean, na.rm = TRUE)
  
  rt.PC_rcp26_2071_2100_JJA <- mask(PC_rcp26_2071_2100_JJA, blank.raster)
  rt.PC_rcp26_2071_2100_JJA <- cellStats(rt.PC_rcp26_2071_2100_JJA, mean, na.rm = TRUE)
  
  rt.PC_rcp45_2011_2040_JJA <- mask(PC_rcp45_2011_2040_JJA, blank.raster)
  rt.PC_rcp45_2011_2040_JJA <- cellStats(rt.PC_rcp45_2011_2040_JJA, mean, na.rm = TRUE)
  
  rt.PC_rcp45_2041_2070_JJA <- mask(PC_rcp45_2041_2070_JJA, blank.raster)
  rt.PC_rcp45_2041_2070_JJA <- cellStats(rt.PC_rcp45_2041_2070_JJA, mean, na.rm = TRUE)
  
  rt.PC_rcp45_2071_2100_JJA <- mask(PC_rcp45_2071_2100_JJA, blank.raster)
  rt.PC_rcp45_2071_2100_JJA <- cellStats(rt.PC_rcp45_2071_2100_JJA, mean, na.rm = TRUE)
  
  rt.PC_rcp85_2011_2040_JJA <- mask(PC_rcp85_2011_2040_JJA, blank.raster)
  rt.PC_rcp85_2011_2040_JJA <- cellStats(rt.PC_rcp85_2011_2040_JJA, mean, na.rm = TRUE)
  
  rt.PC_rcp85_2041_2070_JJA <- mask(PC_rcp85_2041_2070_JJA, blank.raster)
  rt.PC_rcp85_2041_2070_JJA <- cellStats(rt.PC_rcp85_2041_2070_JJA, mean, na.rm = TRUE)
  
  rt.PC_rcp85_2071_2100_JJA <- mask(PC_rcp85_2071_2100_JJA, blank.raster)
  rt.PC_rcp85_2071_2100_JJA <- cellStats(rt.PC_rcp85_2071_2100_JJA, mean, na.rm = TRUE)
  
  #----------------------------------------------------------------------------------------------------------------
  # Season SON
  #----------------------------------------------------------------------------------------------------------------
  rt.PC_rcp26_2011_2040_SON <- mask(PC_rcp26_2011_2040_SON, blank.raster)
  rt.PC_rcp26_2011_2040_SON <- cellStats(rt.PC_rcp26_2011_2040_SON, mean, na.rm = TRUE)
  
  rt.PC_rcp26_2041_2070_SON <- mask(PC_rcp26_2041_2070_SON, blank.raster)
  rt.PC_rcp26_2041_2070_SON <- cellStats(rt.PC_rcp26_2041_2070_SON, mean, na.rm = TRUE)
  
  rt.PC_rcp26_2071_2100_SON <- mask(PC_rcp26_2071_2100_SON, blank.raster)
  rt.PC_rcp26_2071_2100_SON <- cellStats(rt.PC_rcp26_2071_2100_SON, mean, na.rm = TRUE)
  
  rt.PC_rcp45_2011_2040_SON <- mask(PC_rcp45_2011_2040_SON, blank.raster)
  rt.PC_rcp45_2011_2040_SON <- cellStats(rt.PC_rcp45_2011_2040_SON, mean, na.rm = TRUE)
  
  rt.PC_rcp45_2041_2070_SON <- mask(PC_rcp45_2041_2070_SON, blank.raster)
  rt.PC_rcp45_2041_2070_SON <- cellStats(rt.PC_rcp45_2041_2070_SON, mean, na.rm = TRUE)
  
  rt.PC_rcp45_2071_2100_SON <- mask(PC_rcp45_2071_2100_SON, blank.raster)
  rt.PC_rcp45_2071_2100_SON <- cellStats(rt.PC_rcp45_2071_2100_SON, mean, na.rm = TRUE)
  
  rt.PC_rcp85_2011_2040_SON <- mask(PC_rcp85_2011_2040_SON, blank.raster)
  rt.PC_rcp85_2011_2040_SON <- cellStats(rt.PC_rcp85_2011_2040_SON, mean, na.rm = TRUE)
  
  rt.PC_rcp85_2041_2070_SON <- mask(PC_rcp85_2041_2070_SON, blank.raster)
  rt.PC_rcp85_2041_2070_SON <- cellStats(rt.PC_rcp85_2041_2070_SON, mean, na.rm = TRUE)
  
  rt.PC_rcp85_2071_2100_SON <- mask(PC_rcp85_2071_2100_SON, blank.raster)
  rt.PC_rcp85_2071_2100_SON <- cellStats(rt.PC_rcp85_2071_2100_SON, mean, na.rm = TRUE)
  
  # Seasonal vectors are created
  GENERIC_DJF <- c(rt.PC_rcp26_2011_2040_DJF,rt.PC_rcp26_2041_2070_DJF,rt.PC_rcp26_2071_2100_DJF,rt.PC_rcp45_2011_2040_DJF,rt.PC_rcp45_2041_2070_DJF,rt.PC_rcp45_2071_2100_DJF,rt.PC_rcp85_2011_2040_DJF,rt.PC_rcp85_2041_2070_DJF,rt.PC_rcp85_2071_2100_DJF)
  GENERIC_MAM <- c(rt.PC_rcp26_2011_2040_MAM,rt.PC_rcp26_2041_2070_MAM,rt.PC_rcp26_2071_2100_MAM,rt.PC_rcp45_2011_2040_MAM,rt.PC_rcp45_2041_2070_MAM,rt.PC_rcp45_2071_2100_MAM,rt.PC_rcp85_2011_2040_MAM,rt.PC_rcp85_2041_2070_MAM,rt.PC_rcp85_2071_2100_MAM)
  GENERIC_JJA <- c(rt.PC_rcp26_2011_2040_JJA,rt.PC_rcp26_2041_2070_JJA,rt.PC_rcp26_2071_2100_JJA,rt.PC_rcp45_2011_2040_JJA,rt.PC_rcp45_2041_2070_JJA,rt.PC_rcp45_2071_2100_JJA,rt.PC_rcp85_2011_2040_JJA,rt.PC_rcp85_2041_2070_JJA,rt.PC_rcp85_2071_2100_JJA)
  GENERIC_SON <- c(rt.PC_rcp26_2011_2040_SON,rt.PC_rcp26_2041_2070_SON,rt.PC_rcp26_2071_2100_SON,rt.PC_rcp45_2011_2040_SON,rt.PC_rcp45_2041_2070_SON,rt.PC_rcp45_2071_2100_SON,rt.PC_rcp85_2011_2040_SON,rt.PC_rcp85_2041_2070_SON,rt.PC_rcp85_2071_2100_SON)
  
  # Seasonal vectors are renamed
  names(GENERIC_DJF) <- c("PC_rcp26_2011_2040_DJF","PC_rcp26_2041_2070_DJF","PC_rcp26_2071_2100_DJF","PC_rcp45_2011_2040_DJF","PC_rcp45_2041_2070_DJF","PC_rcp45_2071_2100_DJF","PC_rcp85_2011_2040_DJF","PC_rcp85_2041_2070_DJF","PC_rcp85_2071_2100_DJF")
  names(GENERIC_MAM) <- c("PC_rcp26_2011_2040_MAM","PC_rcp26_2041_2070_MAM","PC_rcp26_2071_2100_MAM","PC_rcp45_2011_2040_MAM","PC_rcp45_2041_2070_MAM","PC_rcp45_2071_2100_MAM","PC_rcp85_2011_2040_MAM","PC_rcp85_2041_2070_MAM","PC_rcp85_2071_2100_MAM")
  names(GENERIC_JJA) <- c("PC_rcp26_2011_2040_JJA","PC_rcp26_2041_2070_JJA","PC_rcp26_2071_2100_JJA","PC_rcp45_2011_2040_JJA","PC_rcp45_2041_2070_JJA","PC_rcp45_2071_2100_JJA","PC_rcp85_2011_2040_JJA","PC_rcp85_2041_2070_JJA","PC_rcp85_2071_2100_JJA")
  names(GENERIC_SON) <- c("PC_rcp26_2011_2040_SON","PC_rcp26_2041_2070_SON","PC_rcp26_2071_2100_SON","PC_rcp45_2011_2040_SON","PC_rcp45_2041_2070_SON","PC_rcp45_2071_2100_SON","PC_rcp85_2011_2040_SON","PC_rcp85_2041_2070_SON","PC_rcp85_2071_2100_SON")
  
  # A compiled vector is created
  GENERIC_SEASONS <- c(GENERIC_DJF, GENERIC_MAM, GENERIC_JJA, GENERIC_SON)
  
  # A compiled seasonal data.frame is created
  df.compiled <- data.frame(GENERIC_SEASONS)
  
  # A character variable is created
  #df.compiled$metric <- c("mean")
  
  # data.frame is requested
  #View(df.compiled)
  
  #----------------------------------------------------------------------------------------------------------------
  # SUBBLOCK: Spatially-Averaged Median Value
  #----------------------------------------------------------------------------------------------------------------
  
  #----------------------------------------------------------------------------------------------------------------
  # Season DJF
  #----------------------------------------------------------------------------------------------------------------
  rt.PC_rcp26_2011_2040_DJF <- mask(PC_rcp26_2011_2040_DJF, blank.raster)
  rt.PC_rcp26_2011_2040_DJF <- cellStats(rt.PC_rcp26_2011_2040_DJF, median, na.rm = TRUE)
  
  rt.PC_rcp26_2041_2070_DJF <- mask(PC_rcp26_2041_2070_DJF, blank.raster)
  rt.PC_rcp26_2041_2070_DJF <- cellStats(rt.PC_rcp26_2041_2070_DJF, median, na.rm = TRUE)
  
  rt.PC_rcp26_2071_2100_DJF <- mask(PC_rcp26_2071_2100_DJF, blank.raster)
  rt.PC_rcp26_2071_2100_DJF <- cellStats(rt.PC_rcp26_2071_2100_DJF, median, na.rm = TRUE)
  
  rt.PC_rcp45_2011_2040_DJF <- mask(PC_rcp45_2011_2040_DJF, blank.raster)
  rt.PC_rcp45_2011_2040_DJF <- cellStats(rt.PC_rcp45_2011_2040_DJF, median, na.rm = TRUE)
  
  rt.PC_rcp45_2041_2070_DJF <- mask(PC_rcp45_2041_2070_DJF, blank.raster)
  rt.PC_rcp45_2041_2070_DJF <- cellStats(rt.PC_rcp45_2041_2070_DJF, median, na.rm = TRUE)
  
  rt.PC_rcp45_2071_2100_DJF <- mask(PC_rcp45_2071_2100_DJF, blank.raster)
  rt.PC_rcp45_2071_2100_DJF <- cellStats(rt.PC_rcp45_2071_2100_DJF, median, na.rm = TRUE)
  
  rt.PC_rcp85_2011_2040_DJF <- mask(PC_rcp85_2011_2040_DJF, blank.raster)
  rt.PC_rcp85_2011_2040_DJF <- cellStats(rt.PC_rcp85_2011_2040_DJF, median, na.rm = TRUE)
  
  rt.PC_rcp85_2041_2070_DJF <- mask(PC_rcp85_2041_2070_DJF, blank.raster)
  rt.PC_rcp85_2041_2070_DJF <- cellStats(rt.PC_rcp85_2041_2070_DJF, median, na.rm = TRUE)
  
  rt.PC_rcp85_2071_2100_DJF <- mask(PC_rcp85_2071_2100_DJF, blank.raster)
  rt.PC_rcp85_2071_2100_DJF <- cellStats(rt.PC_rcp85_2071_2100_DJF, median, na.rm = TRUE)
  
  #----------------------------------------------------------------------------------------------------------------
  # Season MAM
  #----------------------------------------------------------------------------------------------------------------
  rt.PC_rcp26_2011_2040_MAM <- mask(PC_rcp26_2011_2040_MAM, blank.raster)
  rt.PC_rcp26_2011_2040_MAM <- cellStats(rt.PC_rcp26_2011_2040_MAM, median, na.rm = TRUE)
  
  rt.PC_rcp26_2041_2070_MAM <- mask(PC_rcp26_2041_2070_MAM, blank.raster)
  rt.PC_rcp26_2041_2070_MAM <- cellStats(rt.PC_rcp26_2041_2070_MAM, median, na.rm = TRUE)
  
  rt.PC_rcp26_2071_2100_MAM <- mask(PC_rcp26_2071_2100_MAM, blank.raster)
  rt.PC_rcp26_2071_2100_MAM <- cellStats(rt.PC_rcp26_2071_2100_MAM, median, na.rm = TRUE)
  
  rt.PC_rcp45_2011_2040_MAM <- mask(PC_rcp45_2011_2040_MAM, blank.raster)
  rt.PC_rcp45_2011_2040_MAM <- cellStats(rt.PC_rcp45_2011_2040_MAM, median, na.rm = TRUE)
  
  rt.PC_rcp45_2041_2070_MAM <- mask(PC_rcp45_2041_2070_MAM, blank.raster)
  rt.PC_rcp45_2041_2070_MAM <- cellStats(rt.PC_rcp45_2041_2070_MAM, median, na.rm = TRUE)
  
  rt.PC_rcp45_2071_2100_MAM <- mask(PC_rcp45_2071_2100_MAM, blank.raster)
  rt.PC_rcp45_2071_2100_MAM <- cellStats(rt.PC_rcp45_2071_2100_MAM, median, na.rm = TRUE)
  
  rt.PC_rcp85_2011_2040_MAM <- mask(PC_rcp85_2011_2040_MAM, blank.raster)
  rt.PC_rcp85_2011_2040_MAM <- cellStats(rt.PC_rcp85_2011_2040_MAM, median, na.rm = TRUE)
  
  rt.PC_rcp85_2041_2070_MAM <- mask(PC_rcp85_2041_2070_MAM, blank.raster)
  rt.PC_rcp85_2041_2070_MAM <- cellStats(rt.PC_rcp85_2041_2070_MAM, median, na.rm = TRUE)
  
  rt.PC_rcp85_2071_2100_MAM <- mask(PC_rcp85_2071_2100_MAM, blank.raster)
  rt.PC_rcp85_2071_2100_MAM <- cellStats(rt.PC_rcp85_2071_2100_MAM, median, na.rm = TRUE)
  
  #----------------------------------------------------------------------------------------------------------------
  # Season JJA
  #----------------------------------------------------------------------------------------------------------------
  rt.PC_rcp26_2011_2040_JJA <- mask(PC_rcp26_2011_2040_JJA, blank.raster)
  rt.PC_rcp26_2011_2040_JJA <- cellStats(rt.PC_rcp26_2011_2040_JJA, median, na.rm = TRUE)
  
  rt.PC_rcp26_2041_2070_JJA <- mask(PC_rcp26_2041_2070_JJA, blank.raster)
  rt.PC_rcp26_2041_2070_JJA <- cellStats(rt.PC_rcp26_2041_2070_JJA, median, na.rm = TRUE)
  
  rt.PC_rcp26_2071_2100_JJA <- mask(PC_rcp26_2071_2100_JJA, blank.raster)
  rt.PC_rcp26_2071_2100_JJA <- cellStats(rt.PC_rcp26_2071_2100_JJA, median, na.rm = TRUE)
  
  rt.PC_rcp45_2011_2040_JJA <- mask(PC_rcp45_2011_2040_JJA, blank.raster)
  rt.PC_rcp45_2011_2040_JJA <- cellStats(rt.PC_rcp45_2011_2040_JJA, median, na.rm = TRUE)
  
  rt.PC_rcp45_2041_2070_JJA <- mask(PC_rcp45_2041_2070_JJA, blank.raster)
  rt.PC_rcp45_2041_2070_JJA <- cellStats(rt.PC_rcp45_2041_2070_JJA, median, na.rm = TRUE)
  
  rt.PC_rcp45_2071_2100_JJA <- mask(PC_rcp45_2071_2100_JJA, blank.raster)
  rt.PC_rcp45_2071_2100_JJA <- cellStats(rt.PC_rcp45_2071_2100_JJA, median, na.rm = TRUE)
  
  rt.PC_rcp85_2011_2040_JJA <- mask(PC_rcp85_2011_2040_JJA, blank.raster)
  rt.PC_rcp85_2011_2040_JJA <- cellStats(rt.PC_rcp85_2011_2040_JJA, median, na.rm = TRUE)
  
  rt.PC_rcp85_2041_2070_JJA <- mask(PC_rcp85_2041_2070_JJA, blank.raster)
  rt.PC_rcp85_2041_2070_JJA <- cellStats(rt.PC_rcp85_2041_2070_JJA, median, na.rm = TRUE)
  
  rt.PC_rcp85_2071_2100_JJA <- mask(PC_rcp85_2071_2100_JJA, blank.raster)
  rt.PC_rcp85_2071_2100_JJA <- cellStats(rt.PC_rcp85_2071_2100_JJA, median, na.rm = TRUE)
  
  #----------------------------------------------------------------------------------------------------------------
  # Season SON
  #----------------------------------------------------------------------------------------------------------------
  rt.PC_rcp26_2011_2040_SON <- mask(PC_rcp26_2011_2040_SON, blank.raster)
  rt.PC_rcp26_2011_2040_SON <- cellStats(rt.PC_rcp26_2011_2040_SON, median, na.rm = TRUE)
  
  rt.PC_rcp26_2041_2070_SON <- mask(PC_rcp26_2041_2070_SON, blank.raster)
  rt.PC_rcp26_2041_2070_SON <- cellStats(rt.PC_rcp26_2041_2070_SON, median, na.rm = TRUE)
  
  rt.PC_rcp26_2071_2100_SON <- mask(PC_rcp26_2071_2100_SON, blank.raster)
  rt.PC_rcp26_2071_2100_SON <- cellStats(rt.PC_rcp26_2071_2100_SON, median, na.rm = TRUE)
  
  rt.PC_rcp45_2011_2040_SON <- mask(PC_rcp45_2011_2040_SON, blank.raster)
  rt.PC_rcp45_2011_2040_SON <- cellStats(rt.PC_rcp45_2011_2040_SON, median, na.rm = TRUE)
  
  rt.PC_rcp45_2041_2070_SON <- mask(PC_rcp45_2041_2070_SON, blank.raster)
  rt.PC_rcp45_2041_2070_SON <- cellStats(rt.PC_rcp45_2041_2070_SON, median, na.rm = TRUE)
  
  rt.PC_rcp45_2071_2100_SON <- mask(PC_rcp45_2071_2100_SON, blank.raster)
  rt.PC_rcp45_2071_2100_SON <- cellStats(rt.PC_rcp45_2071_2100_SON, median, na.rm = TRUE)
  
  rt.PC_rcp85_2011_2040_SON <- mask(PC_rcp85_2011_2040_SON, blank.raster)
  rt.PC_rcp85_2011_2040_SON <- cellStats(rt.PC_rcp85_2011_2040_SON, median, na.rm = TRUE)
  
  rt.PC_rcp85_2041_2070_SON <- mask(PC_rcp85_2041_2070_SON, blank.raster)
  rt.PC_rcp85_2041_2070_SON <- cellStats(rt.PC_rcp85_2041_2070_SON, median, na.rm = TRUE)
  
  rt.PC_rcp85_2071_2100_SON <- mask(PC_rcp85_2071_2100_SON, blank.raster)
  rt.PC_rcp85_2071_2100_SON <- cellStats(rt.PC_rcp85_2071_2100_SON, median, na.rm = TRUE)
  
  # Seasonal vectors are created
  GENERIC_DJF <- c(rt.PC_rcp26_2011_2040_DJF,rt.PC_rcp26_2041_2070_DJF,rt.PC_rcp26_2071_2100_DJF,rt.PC_rcp45_2011_2040_DJF,rt.PC_rcp45_2041_2070_DJF,rt.PC_rcp45_2071_2100_DJF,rt.PC_rcp85_2011_2040_DJF,rt.PC_rcp85_2041_2070_DJF,rt.PC_rcp85_2071_2100_DJF)
  GENERIC_MAM <- c(rt.PC_rcp26_2011_2040_MAM,rt.PC_rcp26_2041_2070_MAM,rt.PC_rcp26_2071_2100_MAM,rt.PC_rcp45_2011_2040_MAM,rt.PC_rcp45_2041_2070_MAM,rt.PC_rcp45_2071_2100_MAM,rt.PC_rcp85_2011_2040_MAM,rt.PC_rcp85_2041_2070_MAM,rt.PC_rcp85_2071_2100_MAM)
  GENERIC_JJA <- c(rt.PC_rcp26_2011_2040_JJA,rt.PC_rcp26_2041_2070_JJA,rt.PC_rcp26_2071_2100_JJA,rt.PC_rcp45_2011_2040_JJA,rt.PC_rcp45_2041_2070_JJA,rt.PC_rcp45_2071_2100_JJA,rt.PC_rcp85_2011_2040_JJA,rt.PC_rcp85_2041_2070_JJA,rt.PC_rcp85_2071_2100_JJA)
  GENERIC_SON <- c(rt.PC_rcp26_2011_2040_SON,rt.PC_rcp26_2041_2070_SON,rt.PC_rcp26_2071_2100_SON,rt.PC_rcp45_2011_2040_SON,rt.PC_rcp45_2041_2070_SON,rt.PC_rcp45_2071_2100_SON,rt.PC_rcp85_2011_2040_SON,rt.PC_rcp85_2041_2070_SON,rt.PC_rcp85_2071_2100_SON)
  
  # Seasonal vectors are renamed
  names(GENERIC_DJF) <- c("PC_rcp26_2011_2040_DJF","PC_rcp26_2041_2070_DJF","PC_rcp26_2071_2100_DJF","PC_rcp45_2011_2040_DJF","PC_rcp45_2041_2070_DJF","PC_rcp45_2071_2100_DJF","PC_rcp85_2011_2040_DJF","PC_rcp85_2041_2070_DJF","PC_rcp85_2071_2100_DJF")
  names(GENERIC_MAM) <- c("PC_rcp26_2011_2040_MAM","PC_rcp26_2041_2070_MAM","PC_rcp26_2071_2100_MAM","PC_rcp45_2011_2040_MAM","PC_rcp45_2041_2070_MAM","PC_rcp45_2071_2100_MAM","PC_rcp85_2011_2040_MAM","PC_rcp85_2041_2070_MAM","PC_rcp85_2071_2100_MAM")
  names(GENERIC_JJA) <- c("PC_rcp26_2011_2040_JJA","PC_rcp26_2041_2070_JJA","PC_rcp26_2071_2100_JJA","PC_rcp45_2011_2040_JJA","PC_rcp45_2041_2070_JJA","PC_rcp45_2071_2100_JJA","PC_rcp85_2011_2040_JJA","PC_rcp85_2041_2070_JJA","PC_rcp85_2071_2100_JJA")
  names(GENERIC_SON) <- c("PC_rcp26_2011_2040_SON","PC_rcp26_2041_2070_SON","PC_rcp26_2071_2100_SON","PC_rcp45_2011_2040_SON","PC_rcp45_2041_2070_SON","PC_rcp45_2071_2100_SON","PC_rcp85_2011_2040_SON","PC_rcp85_2041_2070_SON","PC_rcp85_2071_2100_SON")
  
  # A compiled vector is created
  GENERIC_SEASONS <- c(GENERIC_DJF, GENERIC_MAM, GENERIC_JJA, GENERIC_SON)
  
  # A compiled seasonal data.frame is created
  df.compiled.median <- data.frame(GENERIC_SEASONS)
  
  # A character variable is created
  #df.compiled.median$metric <- c("min")
  
  # data.frame is requested
  #View(df.compiled.median)
  
  #----------------------------------------------------------------------------------------------------------------
  # SUBBLOCK: Spatially-Averaged SD Value
  #----------------------------------------------------------------------------------------------------------------
  
  #----------------------------------------------------------------------------------------------------------------
  # Season DJF
  #----------------------------------------------------------------------------------------------------------------
  rt.PC_rcp26_2011_2040_DJF <- mask(PC_rcp26_2011_2040_DJF, blank.raster)
  rt.PC_rcp26_2011_2040_DJF <- cellStats(rt.PC_rcp26_2011_2040_DJF, sd, na.rm = TRUE)
  
  rt.PC_rcp26_2041_2070_DJF <- mask(PC_rcp26_2041_2070_DJF, blank.raster)
  rt.PC_rcp26_2041_2070_DJF <- cellStats(rt.PC_rcp26_2041_2070_DJF, sd, na.rm = TRUE)
  
  rt.PC_rcp26_2071_2100_DJF <- mask(PC_rcp26_2071_2100_DJF, blank.raster)
  rt.PC_rcp26_2071_2100_DJF <- cellStats(rt.PC_rcp26_2071_2100_DJF, sd, na.rm = TRUE)
  
  rt.PC_rcp45_2011_2040_DJF <- mask(PC_rcp45_2011_2040_DJF, blank.raster)
  rt.PC_rcp45_2011_2040_DJF <- cellStats(rt.PC_rcp45_2011_2040_DJF, sd, na.rm = TRUE)
  
  rt.PC_rcp45_2041_2070_DJF <- mask(PC_rcp45_2041_2070_DJF, blank.raster)
  rt.PC_rcp45_2041_2070_DJF <- cellStats(rt.PC_rcp45_2041_2070_DJF, sd, na.rm = TRUE)
  
  rt.PC_rcp45_2071_2100_DJF <- mask(PC_rcp45_2071_2100_DJF, blank.raster)
  rt.PC_rcp45_2071_2100_DJF <- cellStats(rt.PC_rcp45_2071_2100_DJF, sd, na.rm = TRUE)
  
  rt.PC_rcp85_2011_2040_DJF <- mask(PC_rcp85_2011_2040_DJF, blank.raster)
  rt.PC_rcp85_2011_2040_DJF <- cellStats(rt.PC_rcp85_2011_2040_DJF, sd, na.rm = TRUE)
  
  rt.PC_rcp85_2041_2070_DJF <- mask(PC_rcp85_2041_2070_DJF, blank.raster)
  rt.PC_rcp85_2041_2070_DJF <- cellStats(rt.PC_rcp85_2041_2070_DJF, sd, na.rm = TRUE)
  
  rt.PC_rcp85_2071_2100_DJF <- mask(PC_rcp85_2071_2100_DJF, blank.raster)
  rt.PC_rcp85_2071_2100_DJF <- cellStats(rt.PC_rcp85_2071_2100_DJF, sd, na.rm = TRUE)
  
  #----------------------------------------------------------------------------------------------------------------
  # Season MAM
  #----------------------------------------------------------------------------------------------------------------
  rt.PC_rcp26_2011_2040_MAM <- mask(PC_rcp26_2011_2040_MAM, blank.raster)
  rt.PC_rcp26_2011_2040_MAM <- cellStats(rt.PC_rcp26_2011_2040_MAM, sd, na.rm = TRUE)
  
  rt.PC_rcp26_2041_2070_MAM <- mask(PC_rcp26_2041_2070_MAM, blank.raster)
  rt.PC_rcp26_2041_2070_MAM <- cellStats(rt.PC_rcp26_2041_2070_MAM, sd, na.rm = TRUE)
  
  rt.PC_rcp26_2071_2100_MAM <- mask(PC_rcp26_2071_2100_MAM, blank.raster)
  rt.PC_rcp26_2071_2100_MAM <- cellStats(rt.PC_rcp26_2071_2100_MAM, sd, na.rm = TRUE)
  
  rt.PC_rcp45_2011_2040_MAM <- mask(PC_rcp45_2011_2040_MAM, blank.raster)
  rt.PC_rcp45_2011_2040_MAM <- cellStats(rt.PC_rcp45_2011_2040_MAM, sd, na.rm = TRUE)
  
  rt.PC_rcp45_2041_2070_MAM <- mask(PC_rcp45_2041_2070_MAM, blank.raster)
  rt.PC_rcp45_2041_2070_MAM <- cellStats(rt.PC_rcp45_2041_2070_MAM, sd, na.rm = TRUE)
  
  rt.PC_rcp45_2071_2100_MAM <- mask(PC_rcp45_2071_2100_MAM, blank.raster)
  rt.PC_rcp45_2071_2100_MAM <- cellStats(rt.PC_rcp45_2071_2100_MAM, sd, na.rm = TRUE)
  
  rt.PC_rcp85_2011_2040_MAM <- mask(PC_rcp85_2011_2040_MAM, blank.raster)
  rt.PC_rcp85_2011_2040_MAM <- cellStats(rt.PC_rcp85_2011_2040_MAM, sd, na.rm = TRUE)
  
  rt.PC_rcp85_2041_2070_MAM <- mask(PC_rcp85_2041_2070_MAM, blank.raster)
  rt.PC_rcp85_2041_2070_MAM <- cellStats(rt.PC_rcp85_2041_2070_MAM, sd, na.rm = TRUE)
  
  rt.PC_rcp85_2071_2100_MAM <- mask(PC_rcp85_2071_2100_MAM, blank.raster)
  rt.PC_rcp85_2071_2100_MAM <- cellStats(rt.PC_rcp85_2071_2100_MAM, sd, na.rm = TRUE)
  
  #----------------------------------------------------------------------------------------------------------------
  # Season JJA
  #----------------------------------------------------------------------------------------------------------------
  rt.PC_rcp26_2011_2040_JJA <- mask(PC_rcp26_2011_2040_JJA, blank.raster)
  rt.PC_rcp26_2011_2040_JJA <- cellStats(rt.PC_rcp26_2011_2040_JJA, sd, na.rm = TRUE)
  
  rt.PC_rcp26_2041_2070_JJA <- mask(PC_rcp26_2041_2070_JJA, blank.raster)
  rt.PC_rcp26_2041_2070_JJA <- cellStats(rt.PC_rcp26_2041_2070_JJA, sd, na.rm = TRUE)
  
  rt.PC_rcp26_2071_2100_JJA <- mask(PC_rcp26_2071_2100_JJA, blank.raster)
  rt.PC_rcp26_2071_2100_JJA <- cellStats(rt.PC_rcp26_2071_2100_JJA, sd, na.rm = TRUE)
  
  rt.PC_rcp45_2011_2040_JJA <- mask(PC_rcp45_2011_2040_JJA, blank.raster)
  rt.PC_rcp45_2011_2040_JJA <- cellStats(rt.PC_rcp45_2011_2040_JJA, sd, na.rm = TRUE)
  
  rt.PC_rcp45_2041_2070_JJA <- mask(PC_rcp45_2041_2070_JJA, blank.raster)
  rt.PC_rcp45_2041_2070_JJA <- cellStats(rt.PC_rcp45_2041_2070_JJA, sd, na.rm = TRUE)
  
  rt.PC_rcp45_2071_2100_JJA <- mask(PC_rcp45_2071_2100_JJA, blank.raster)
  rt.PC_rcp45_2071_2100_JJA <- cellStats(rt.PC_rcp45_2071_2100_JJA, sd, na.rm = TRUE)
  
  rt.PC_rcp85_2011_2040_JJA <- mask(PC_rcp85_2011_2040_JJA, blank.raster)
  rt.PC_rcp85_2011_2040_JJA <- cellStats(rt.PC_rcp85_2011_2040_JJA, sd, na.rm = TRUE)
  
  rt.PC_rcp85_2041_2070_JJA <- mask(PC_rcp85_2041_2070_JJA, blank.raster)
  rt.PC_rcp85_2041_2070_JJA <- cellStats(rt.PC_rcp85_2041_2070_JJA, sd, na.rm = TRUE)
  
  rt.PC_rcp85_2071_2100_JJA <- mask(PC_rcp85_2071_2100_JJA, blank.raster)
  rt.PC_rcp85_2071_2100_JJA <- cellStats(rt.PC_rcp85_2071_2100_JJA, sd, na.rm = TRUE)
  
  #----------------------------------------------------------------------------------------------------------------
  # Season SON
  #----------------------------------------------------------------------------------------------------------------
  rt.PC_rcp26_2011_2040_SON <- mask(PC_rcp26_2011_2040_SON, blank.raster)
  rt.PC_rcp26_2011_2040_SON <- cellStats(rt.PC_rcp26_2011_2040_SON, sd, na.rm = TRUE)
  
  rt.PC_rcp26_2041_2070_SON <- mask(PC_rcp26_2041_2070_SON, blank.raster)
  rt.PC_rcp26_2041_2070_SON <- cellStats(rt.PC_rcp26_2041_2070_SON, sd, na.rm = TRUE)
  
  rt.PC_rcp26_2071_2100_SON <- mask(PC_rcp26_2071_2100_SON, blank.raster)
  rt.PC_rcp26_2071_2100_SON <- cellStats(rt.PC_rcp26_2071_2100_SON, sd, na.rm = TRUE)
  
  rt.PC_rcp45_2011_2040_SON <- mask(PC_rcp45_2011_2040_SON, blank.raster)
  rt.PC_rcp45_2011_2040_SON <- cellStats(rt.PC_rcp45_2011_2040_SON, sd, na.rm = TRUE)
  
  rt.PC_rcp45_2041_2070_SON <- mask(PC_rcp45_2041_2070_SON, blank.raster)
  rt.PC_rcp45_2041_2070_SON <- cellStats(rt.PC_rcp45_2041_2070_SON, sd, na.rm = TRUE)
  
  rt.PC_rcp45_2071_2100_SON <- mask(PC_rcp45_2071_2100_SON, blank.raster)
  rt.PC_rcp45_2071_2100_SON <- cellStats(rt.PC_rcp45_2071_2100_SON, sd, na.rm = TRUE)
  
  rt.PC_rcp85_2011_2040_SON <- mask(PC_rcp85_2011_2040_SON, blank.raster)
  rt.PC_rcp85_2011_2040_SON <- cellStats(rt.PC_rcp85_2011_2040_SON, sd, na.rm = TRUE)
  
  rt.PC_rcp85_2041_2070_SON <- mask(PC_rcp85_2041_2070_SON, blank.raster)
  rt.PC_rcp85_2041_2070_SON <- cellStats(rt.PC_rcp85_2041_2070_SON, sd, na.rm = TRUE)
  
  rt.PC_rcp85_2071_2100_SON <- mask(PC_rcp85_2071_2100_SON, blank.raster)
  rt.PC_rcp85_2071_2100_SON <- cellStats(rt.PC_rcp85_2071_2100_SON, sd, na.rm = TRUE)
  
  # Seasonal vectors are created
  GENERIC_DJF <- c(rt.PC_rcp26_2011_2040_DJF,rt.PC_rcp26_2041_2070_DJF,rt.PC_rcp26_2071_2100_DJF,rt.PC_rcp45_2011_2040_DJF,rt.PC_rcp45_2041_2070_DJF,rt.PC_rcp45_2071_2100_DJF,rt.PC_rcp85_2011_2040_DJF,rt.PC_rcp85_2041_2070_DJF,rt.PC_rcp85_2071_2100_DJF)
  GENERIC_MAM <- c(rt.PC_rcp26_2011_2040_MAM,rt.PC_rcp26_2041_2070_MAM,rt.PC_rcp26_2071_2100_MAM,rt.PC_rcp45_2011_2040_MAM,rt.PC_rcp45_2041_2070_MAM,rt.PC_rcp45_2071_2100_MAM,rt.PC_rcp85_2011_2040_MAM,rt.PC_rcp85_2041_2070_MAM,rt.PC_rcp85_2071_2100_MAM)
  GENERIC_JJA <- c(rt.PC_rcp26_2011_2040_JJA,rt.PC_rcp26_2041_2070_JJA,rt.PC_rcp26_2071_2100_JJA,rt.PC_rcp45_2011_2040_JJA,rt.PC_rcp45_2041_2070_JJA,rt.PC_rcp45_2071_2100_JJA,rt.PC_rcp85_2011_2040_JJA,rt.PC_rcp85_2041_2070_JJA,rt.PC_rcp85_2071_2100_JJA)
  GENERIC_SON <- c(rt.PC_rcp26_2011_2040_SON,rt.PC_rcp26_2041_2070_SON,rt.PC_rcp26_2071_2100_SON,rt.PC_rcp45_2011_2040_SON,rt.PC_rcp45_2041_2070_SON,rt.PC_rcp45_2071_2100_SON,rt.PC_rcp85_2011_2040_SON,rt.PC_rcp85_2041_2070_SON,rt.PC_rcp85_2071_2100_SON)
  
  # Seasonal vectors are renamed
  names(GENERIC_DJF) <- c("PC_rcp26_2011_2040_DJF","PC_rcp26_2041_2070_DJF","PC_rcp26_2071_2100_DJF","PC_rcp45_2011_2040_DJF","PC_rcp45_2041_2070_DJF","PC_rcp45_2071_2100_DJF","PC_rcp85_2011_2040_DJF","PC_rcp85_2041_2070_DJF","PC_rcp85_2071_2100_DJF")
  names(GENERIC_MAM) <- c("PC_rcp26_2011_2040_MAM","PC_rcp26_2041_2070_MAM","PC_rcp26_2071_2100_MAM","PC_rcp45_2011_2040_MAM","PC_rcp45_2041_2070_MAM","PC_rcp45_2071_2100_MAM","PC_rcp85_2011_2040_MAM","PC_rcp85_2041_2070_MAM","PC_rcp85_2071_2100_MAM")
  names(GENERIC_JJA) <- c("PC_rcp26_2011_2040_JJA","PC_rcp26_2041_2070_JJA","PC_rcp26_2071_2100_JJA","PC_rcp45_2011_2040_JJA","PC_rcp45_2041_2070_JJA","PC_rcp45_2071_2100_JJA","PC_rcp85_2011_2040_JJA","PC_rcp85_2041_2070_JJA","PC_rcp85_2071_2100_JJA")
  names(GENERIC_SON) <- c("PC_rcp26_2011_2040_SON","PC_rcp26_2041_2070_SON","PC_rcp26_2071_2100_SON","PC_rcp45_2011_2040_SON","PC_rcp45_2041_2070_SON","PC_rcp45_2071_2100_SON","PC_rcp85_2011_2040_SON","PC_rcp85_2041_2070_SON","PC_rcp85_2071_2100_SON")
  
  # A compiled vector is created
  GENERIC_SEASONS <- c(GENERIC_DJF, GENERIC_MAM, GENERIC_JJA, GENERIC_SON)
  
  # A compiled seasonal data.frame is created
  df.compiled.sd <- data.frame(GENERIC_SEASONS)
  
  # A character variable is created
  #df.compiled.sd$metric <- c("max")
  
  # data.frame is requested
  #View(df.compiled.sd)
  
  # Relevant data.frames are binded
  df.compiled.total <- cbind(df.compiled,df.compiled.sd,df.compiled.median)
  
  # Region variable is added to data.frame
  df.compiled.total$Region <- w
  
  # List container is filled
  list.CRCR[[w]] <- df.compiled.total
  
} # Main loop is closed

# A compiled data.frame is created
df.compiled.total.export <- ldply(list.CRCR, data.frame)

# data.frame variables are renamed
names(df.compiled.total.export) <- c("Mean", "SD", "Median", "Region")

# Compiled data.frame is requested
View(df.compiled.total.export)

# A summary is requested
summary(df.compiled.total.export)

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Post-Processing Lattice Graphics
# Generates: Figure 12 of the Manuscript
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

# CR climati Region ShapeFile is loaded
shp <- shapefile("zonas_climaticas_def_crtm05")

# CRTM05 projection is assigned to rasterbrick object
crs(shp) <- CRTM05

# A color palette is created based on http://colorbrewer2.org/
#-20, 202,0,32, 255,-20         #CA0020
#-15.6, 221,73,75, 255,-15.6    #DD494B
#-11.1, 240,147,119, 255,-11.1  #F09377
#-6.67, 245,193,169, 255,-6.67  #F5C1A9
#-2.22, 247,229,221,  255,-2.22 #F7E5DD
#2.22, 225,236,242,255,2.22     #E1ECF2
#6.67, 180,214,231, 255,6.67    #B4D6E7
#11.1, 130,188,217, 255,11.1    #82BCD9
#15.6, 67,150,197, 255,15.6     #4396C5
#20, 5,113,176, 255,20          #0571B0

# A color palette is created based on http://colorbrewer2.org/
colr.rain3 <- colorRampPalette(c("#CA0020", # -10
                                 "#DD494B", # -8
                                 "#F09377", # -5
                                 "#F5C1A9", # -2
                                 "#F7E5DD", # -1 boundary
                                 "#E1ECF2", # 5
                                 "#B4D6E7", # 10
                                 "#82BCD9", # 15
                                 "#4396C5", # 20
                                 "#0571B0")) # 30

# Color scale breaks are defined
brk.near <- c(-40, -20, -15, -10, -5, 0, 5, 10, 15, 20, 35)
brk.mid <- c(-40, -20, -15, -10, -5, 0, 5, 10, 15, 20, 35)
brk.far <- c(-40, -20, -15, -10, -5, 0, 5, 10, 15, 20, 35)

# Color scale breaks are defined
brk2 <- do.breaks(c(-5, 5), 20)

#----------------------------------------------------------------------------------------------------------------
# Near-Future 2011_2040
#----------------------------------------------------------------------------------------------------------------

# A raster stack object is created
near.future.raster <- stack(PC_rcp26_2011_2040_DJF,
                            PC_rcp45_2011_2040_DJF,
                            PC_rcp85_2011_2040_DJF,
                            PC_rcp26_2011_2040_MAM,
                            PC_rcp45_2011_2040_MAM,
                            PC_rcp85_2011_2040_MAM,
                            PC_rcp26_2011_2040_JJA,
                            PC_rcp45_2011_2040_JJA,
                            PC_rcp85_2011_2040_JJA,                        
                            PC_rcp26_2011_2040_SON,
                            PC_rcp45_2011_2040_SON,
                            PC_rcp85_2011_2040_SON)

# A simple plot is created on raster level [1]
#plot(near.future.raster[[1]])

# Rasterbrick object names are replaced
names(near.future.raster) <- c("RCP26_DJF",
                               "RCP45_DJF",
                               "RCP85_DJF",
                               "RCP26_MAM",
                               "RCP45_MAM",
                               "RCP85_MAM",
                               "RCP26_JJA",
                               "RCP45_JJA",
                               "RCP85_JJA",                        
                               "RCP26_SON",
                               "RCP45_SON",
                               "RCP85_SON")

# A "levels" monthly vector is created for visualization puposes
v.select.name.near <- c("RCP26_DJF",
                        "RCP45_DJF",
                        "RCP85_DJF",
                        "RCP26_MAM",
                        "RCP45_MAM",
                        "RCP85_MAM",
                        "RCP26_JJA",
                        "RCP45_JJA",
                        "RCP85_JJA",                        
                        "RCP26_SON",
                        "RCP45_SON",
                        "RCP85_SON")

# Rasterbrick layer names are replaced
names(near.future.raster) <- v.select.name.near

# Mean precipitation plot is created
levelplot(near.future.raster, layout=c(3, 4),
          col.regions = colr.rain3, at = brk.near,
          colorkey = list(col = colr.rain3, at = brk.near)) +
  latticeExtra::layer(sp.polygons(shp, lwd = 2))

#----------------------------------------------------------------------------------------------------------------
# Mid-Future 2041_2070
#----------------------------------------------------------------------------------------------------------------

# A raster stack object is created
mid.future.raster <- stack(PC_rcp26_2041_2070_DJF,
                           PC_rcp45_2041_2070_DJF,
                           PC_rcp85_2041_2070_DJF,
                           PC_rcp26_2041_2070_MAM,
                           PC_rcp45_2041_2070_MAM,
                           PC_rcp85_2041_2070_MAM,
                           PC_rcp26_2041_2070_JJA,
                           PC_rcp45_2041_2070_JJA,
                           PC_rcp85_2041_2070_JJA,                        
                           PC_rcp26_2041_2070_SON,
                           PC_rcp45_2041_2070_SON,
                           PC_rcp85_2041_2070_SON)

# A simple plot is created on raster level [1]
#plot(mid.future.raster[[1]])

# Rasterbrick object names are replaced
names(mid.future.raster) <- c("RCP26_DJF",
                              "RCP45_DJF",
                              "RCP85_DJF",
                              "RCP26_MAM",
                              "RCP45_MAM",
                              "RCP85_MAM",
                              "RCP26_JJA",
                              "RCP45_JJA",
                              "RCP85_JJA",                        
                              "RCP26_SON",
                              "RCP45_SON",
                              "RCP85_SON")

# A "levels" monthly vector is created for visualization puposes
v.select.name.mid <- c("RCP26_DJF",
                       "RCP45_DJF",
                       "RCP85_DJF",
                       "RCP26_MAM",
                       "RCP45_MAM",
                       "RCP85_MAM",
                       "RCP26_JJA",
                       "RCP45_JJA",
                       "RCP85_JJA",                        
                       "RCP26_SON",
                       "RCP45_SON",
                       "RCP85_SON")

# Rasterbrick layer names are replaced
names(mid.future.raster) <- v.select.name.mid

# Mean precipitation plot is created
levelplot(mid.future.raster, layout=c(3, 4),
          col.regions = colr.rain3, at = brk.mid,
          colorkey = list(col = colr.rain3, at = brk.mid)) +
  latticeExtra::layer(sp.polygons(shp, lwd = 2))

#----------------------------------------------------------------------------------------------------------------
# Far-Future 2071_2100
#----------------------------------------------------------------------------------------------------------------

# A raster stack object is created
far.future.raster <- stack(PC_rcp26_2071_2100_DJF,
                           PC_rcp45_2071_2100_DJF,
                           PC_rcp85_2071_2100_DJF,
                           PC_rcp26_2071_2100_MAM,
                           PC_rcp45_2071_2100_MAM,
                           PC_rcp85_2071_2100_MAM,
                           PC_rcp26_2071_2100_JJA,
                           PC_rcp45_2071_2100_JJA,
                           PC_rcp85_2071_2100_JJA,                        
                           PC_rcp26_2071_2100_SON,
                           PC_rcp45_2071_2100_SON,
                           PC_rcp85_2071_2100_SON)

# A simple plot is created on raster level [1]
#plot(far.future.raster[[1]])

# Rasterbrick object names are replaced
names(far.future.raster) <- c("RCP26_DJF",
                              "RCP45_DJF",
                              "RCP85_DJF",
                              "RCP26_MAM",
                              "RCP45_MAM",
                              "RCP85_MAM",
                              "RCP26_JJA",
                              "RCP45_JJA",
                              "RCP85_JJA",                        
                              "RCP26_SON",
                              "RCP45_SON",
                              "RCP85_SON")

# A "levels" monthly vector is created for visualization puposes
v.select.name.far <- c("RCP26_DJF",
                       "RCP45_DJF",
                       "RCP85_DJF",
                       "RCP26_MAM",
                       "RCP45_MAM",
                       "RCP85_MAM",
                       "RCP26_JJA",
                       "RCP45_JJA",
                       "RCP85_JJA",                        
                       "RCP26_SON",
                       "RCP45_SON",
                       "RCP85_SON")

# Rasterbrick layer names are replaced
names(far.future.raster) <- v.select.name.far

# Mean precipitation plot is created
levelplot(far.future.raster, layout=c(3, 4),
          col.regions = colr.rain3, at = brk.far,
          colorkey = list(col = colr.rain3, at = brk.far)) +
  latticeExtra::layer(sp.polygons(shp, lwd = 2))

# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Time-series plots generation
# Generates: Figure 11 Bias Corrected
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////

# Input data is loaded and a data.frame is created
# This file is linked to Chirps_GPCC_comparison.XLSX !!!!
df.crcr.02 <- read.table("BIAS_CORRECTION_MONTHLY_aggregated_min_max_Ensemble_EQM.txt", header = TRUE)

# A "levels" monthly vector is created for visualization puposes
v.select.name.02 <- c("DJF", "MAM", "JJA","SON")

# Months are ordered to avoid ggplot plotting conflicts
df.crcr.02$Season <- ordered(df.crcr.02$Season, v.select.name.02)

# A "Regions" monthly vector is created for visualization puposes
v.select.regioncr <- c("North_Pacific", "Central_Pacific", "South_Pacific", "Central_Valley", "North", "Caribbean")

# Regions are ordered to avoid ggplot plotting conflicts
df.crcr.02$Region <- ordered(df.crcr.02$Region, v.select.regioncr)

# A color palette is created based on http://colorbrewer2.org/
scale03 <- c("#4daf4a", "#377eb8", "#e41a1c")

# A BC quarter subset is executed
df.crcr.02 <- subset(df.crcr.02, Method == "BC")

# Figure 11 of the manuscript
ggplot() +
  #geom_point(aes(x = Period,y = mean,shape = PC_RCPs,colour = PC_RCPs),
  #           data=df.crcr.02,size = 0.1) +
  facet_grid(facets = Season ~ Region + PC_RCPs, scales = 'free_y') +
  theme_bw() +
  geom_hline(data=df.crcr.02,yintercept = 0.0,size = 0.95,linetype = 2) +
  geom_hline(data=df.crcr.02,yintercept = -10,size = 0.85,linetype = 2, alpha=0.85, colour="#fec44f") +
  geom_hline(data=df.crcr.02,yintercept = 10,size = 0.85,linetype = 2, alpha=0.85, colour="#fec44f") +
  geom_hline(data=df.crcr.02,yintercept = -20,size = 0.85,linetype = 2, alpha=0.85, colour="#d95f0e") +
  geom_hline(data=df.crcr.02,yintercept = 20,size = 0.85,linetype = 2, alpha=0.85, colour="#d95f0e") +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0)) +
  #theme(axis.text.x = element_text(angle = 90.0)) +
  geom_pointrange(aes(x = Period,y = mean,ymin = min,ymax = max,colour = PC_RCPs,shape = PC_RCPs),
                  data=df.crcr.02,size = 0.75,linetype = 1) +
  geom_line(aes(x = Period,y = mean,colour = PC_RCPs,group = PC_RCPs),data=df.crcr.02, size = 1.20) +
  scale_color_manual(values = scale03) +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), 
        text=element_text(size=15,  family="serif"), legend.position="bottom") +
  xlab("30-year period") +
  ylab("1961-1990_Baseline Percent-Change (PC) in Precipitation  [%]")

# Figure 11 of the manuscript
ggplot() +
  #geom_point(aes(x = Period,y = mean,shape = PC_RCPs,colour = PC_RCPs),
  #           data=df.crcr.02,size = 0.1) +
  facet_grid(facets = Season ~ Region, scales = 'free_y') +
  theme_bw() +
  geom_hline(data=df.crcr.02,yintercept = 0.0,size = 0.95,linetype = 2) +
  geom_hline(data=df.crcr.02,yintercept = -10,size = 0.85,linetype = 2, alpha=0.85, colour="#fec44f") +
  geom_hline(data=df.crcr.02,yintercept = 10,size = 0.85,linetype = 2, alpha=0.85, colour="#fec44f") +
  geom_hline(data=df.crcr.02,yintercept = -20,size = 0.85,linetype = 2, alpha=0.85, colour="#d95f0e") +
  geom_hline(data=df.crcr.02,yintercept = 20,size = 0.85,linetype = 2, alpha=0.85, colour="#d95f0e") +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 10.0,min.n = 10.0)) +
  #theme(axis.text.x = element_text(angle = 90.0)) +
  geom_pointrange(aes(x = Period,y = mean,ymin = min,ymax = max,colour = PC_RCPs,shape = PC_RCPs),
                  data=df.crcr.02,size = 0.75,linetype = 1) +
  geom_line(aes(x = Period,y = mean,colour = PC_RCPs,group = PC_RCPs),data=df.crcr.02, size = 1.20) +
  scale_color_manual(values = scale03) +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), 
        text=element_text(size=15,  family="serif"), legend.position="bottom") +
  xlab("30-year period") +
  ylab("1961-1990_Baseline Percent-Change (PC) in Precipitation  [%]")
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////





# -----------------------------------------------------------------------------------------------------------
# Alternatives to Figure 11
# -----------------------------------------------------------------------------------------------------------
df.crcr.03 <- subset(df.crcr.02, Season == "DJF")

ggplot() +
  geom_line(aes(x = Period,y = mean,colour = PC_RCPs,group = PC_RCPs),data=df.crcr.03,size = 0.75) +
  theme(axis.text.x = element_text(angle = 90.0)) +
  facet_wrap(facets = ~Region, ncol = 1, scales = 'free_y') +
  geom_pointrange(aes(x = Period,y = mean,ymin = min,ymax = max,colour = PC_RCPs,shape = PC_RCPs),data=df.crcr.03,size = 0.9,position = position_jitter(width = 0.05)) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 8,min.n = 8)) +
  scale_x_discrete(expand = c(0.075,0.075)) +
  geom_hline(data=df.crcr.03,yintercept = 0.0,size = 0.75,linetype = 2) +
  theme_bw()+
  scale_color_manual(values = scale03) +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), 
        text=element_text(size=12,  family="serif"), legend.position="bottom") +
  xlab("30-year period") +
  ylab("1961-1990_Baseline Percent-Change (PC) in Precipitation  [%]")

df.crcr.03 <- subset(df.crcr.02, Season == "MAM")

ggplot() +
  geom_line(aes(x = Period,y = mean,colour = PC_RCPs,group = PC_RCPs),data=df.crcr.03,size = 0.75) +
  theme(axis.text.x = element_text(angle = 90.0)) +
  facet_wrap(facets = ~Region, ncol = 1, scales = 'free_y') +
  geom_pointrange(aes(x = Period,y = mean,ymin = min,ymax = max,colour = PC_RCPs,shape = PC_RCPs),data=df.crcr.03,size = 0.9,position = position_jitter(width = 0.05)) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 8,min.n = 8)) +
  scale_x_discrete(expand = c(0.075,0.075)) +
  geom_hline(data=df.crcr.03,yintercept = 0.0,size = 0.75,linetype = 2) +
  theme_bw()+
  scale_color_manual(values = scale03) +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), 
        text=element_text(size=12,  family="serif"), legend.position="bottom") +
  xlab("30-year period") +
  ylab("1961-1990_Baseline Percent-Change (PC) in Precipitation  [%]")

df.crcr.03 <- subset(df.crcr.02, Season == "JJA")

ggplot() +
  geom_line(aes(x = Period,y = mean,colour = PC_RCPs,group = PC_RCPs),data=df.crcr.03,size = 0.75) +
  theme(axis.text.x = element_text(angle = 90.0)) +
  facet_wrap(facets = ~Region, ncol = 1, scales = 'free_y') +
  geom_pointrange(aes(x = Period,y = mean,ymin = min,ymax = max,colour = PC_RCPs,shape = PC_RCPs),data=df.crcr.03,size = 0.9,position = position_jitter(width = 0.05)) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 8,min.n = 8)) +
  scale_x_discrete(expand = c(0.075,0.075)) +
  geom_hline(data=df.crcr.03,yintercept = 0.0,size = 0.75,linetype = 2) +
  theme_bw()+
  scale_color_manual(values = scale03) +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), 
        text=element_text(size=12,  family="serif"), legend.position="bottom") +
  xlab("30-year period") +
  ylab("1961-1990_Baseline Percent-Change (PC) in Precipitation  [%]")

df.crcr.03 <- subset(df.crcr.02, Season == "SON")

ggplot() +
  geom_line(aes(x = Period,y = mean,colour = PC_RCPs,group = PC_RCPs),data=df.crcr.03,size = 0.75) +
  theme(axis.text.x = element_text(angle = 90.0)) +
  facet_wrap(facets = ~Region, ncol = 1, scales = 'free_y') +
  geom_pointrange(aes(x = Period,y = mean,ymin = min,ymax = max,colour = PC_RCPs,shape = PC_RCPs),data=df.crcr.03,size = 0.9,position = position_jitter(width = 0.05)) +
  scale_y_continuous(breaks = scales::pretty_breaks(n = 8,min.n = 8)) +
  scale_x_discrete(expand = c(0.075,0.075)) +
  geom_hline(data=df.crcr.03,yintercept = 0.0,size = 0.75,linetype = 2) +
  theme_bw()+
  scale_color_manual(values = scale03) +
  theme(axis.text.x = element_text(vjust = 0.5,angle = 90.0), 
        text=element_text(size=12,  family="serif"), legend.position="bottom") +
  xlab("30-year period") +
  ylab("1961-1990_Baseline Percent-Change (PC) in Precipitation  [%]")
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////




































# ////////////////////////////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Time-series plots generation
# Generates: Figure 11 RAW values (Uncurrected)
# ////////////////////////////////////////////////////////////////////////////////////////////////////////////


# A external binary object is loaded
#load("C:/DATOS/R_ITC/XLSX/FUTURE_ENSEMBLE/Binary_Raster_RAW.RData")

PC_RAW_rcp26_2011_2040 <- stack( "PC_RAW_rcp26_2011_2040.tif")
PC_RAW_rcp26_2041_2070 <- stack( "PC_RAW_rcp26_2041_2070.tif")
PC_RAW_rcp26_2071_2100 <- stack( "PC_RAW_rcp26_2071_2100.tif")

PC_RAW_rcp45_2011_2040 <- stack( "PC_RAW_rcp45_2011_2040.tif")
PC_RAW_rcp45_2041_2070 <- stack( "PC_RAW_rcp45_2041_2070.tif")
PC_RAW_rcp45_2071_2100 <- stack( "PC_RAW_rcp45_2071_2100.tif")

PC_RAW_rcp85_2011_2040 <- stack( "PC_RAW_rcp85_2011_2040.tif")
PC_RAW_rcp85_2041_2070 <- stack( "PC_RAW_rcp85_2041_2070.tif")
PC_RAW_rcp85_2071_2100 <- stack( "PC_RAW_rcp85_2071_2100.tif")

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# JAN
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_JAN <- calc(PC_RAW_rcp26_2011_2040[[vs.JAN]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_JAN <- calc(PC_RAW_rcp26_2041_2070[[vs.JAN]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_JAN <- calc(PC_RAW_rcp26_2071_2100[[vs.JAN]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_JAN <- calc(PC_RAW_rcp45_2011_2040[[vs.JAN]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_JAN <- calc(PC_RAW_rcp45_2041_2070[[vs.JAN]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_JAN <- calc(PC_RAW_rcp45_2071_2100[[vs.JAN]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_JAN <- calc(PC_RAW_rcp85_2011_2040[[vs.JAN]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_JAN <- calc(PC_RAW_rcp85_2041_2070[[vs.JAN]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_JAN <- calc(PC_RAW_rcp85_2071_2100[[vs.JAN]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# FEB
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_FEB <- calc(PC_RAW_rcp26_2011_2040[[vs.FEB]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_FEB <- calc(PC_RAW_rcp26_2041_2070[[vs.FEB]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_FEB <- calc(PC_RAW_rcp26_2071_2100[[vs.FEB]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_FEB <- calc(PC_RAW_rcp45_2011_2040[[vs.FEB]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_FEB <- calc(PC_RAW_rcp45_2041_2070[[vs.FEB]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_FEB <- calc(PC_RAW_rcp45_2071_2100[[vs.FEB]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_FEB <- calc(PC_RAW_rcp85_2011_2040[[vs.FEB]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_FEB <- calc(PC_RAW_rcp85_2041_2070[[vs.FEB]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_FEB <- calc(PC_RAW_rcp85_2071_2100[[vs.FEB]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# MAR
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_MAR <- calc(PC_RAW_rcp26_2011_2040[[vs.MAR]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_MAR <- calc(PC_RAW_rcp26_2041_2070[[vs.MAR]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_MAR <- calc(PC_RAW_rcp26_2071_2100[[vs.MAR]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_MAR <- calc(PC_RAW_rcp45_2011_2040[[vs.MAR]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_MAR <- calc(PC_RAW_rcp45_2041_2070[[vs.MAR]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_MAR <- calc(PC_RAW_rcp45_2071_2100[[vs.MAR]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_MAR <- calc(PC_RAW_rcp85_2011_2040[[vs.MAR]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_MAR <- calc(PC_RAW_rcp85_2041_2070[[vs.MAR]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_MAR <- calc(PC_RAW_rcp85_2071_2100[[vs.MAR]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# APR
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_APR <- calc(PC_RAW_rcp26_2011_2040[[vs.APR]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_APR <- calc(PC_RAW_rcp26_2041_2070[[vs.APR]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_APR <- calc(PC_RAW_rcp26_2071_2100[[vs.APR]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_APR <- calc(PC_RAW_rcp45_2011_2040[[vs.APR]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_APR <- calc(PC_RAW_rcp45_2041_2070[[vs.APR]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_APR <- calc(PC_RAW_rcp45_2071_2100[[vs.APR]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_APR <- calc(PC_RAW_rcp85_2011_2040[[vs.APR]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_APR <- calc(PC_RAW_rcp85_2041_2070[[vs.APR]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_APR <- calc(PC_RAW_rcp85_2071_2100[[vs.APR]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# MAY
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_MAY <- calc(PC_RAW_rcp26_2011_2040[[vs.MAY]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_MAY <- calc(PC_RAW_rcp26_2041_2070[[vs.MAY]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_MAY <- calc(PC_RAW_rcp26_2071_2100[[vs.MAY]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_MAY <- calc(PC_RAW_rcp45_2011_2040[[vs.MAY]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_MAY <- calc(PC_RAW_rcp45_2041_2070[[vs.MAY]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_MAY <- calc(PC_RAW_rcp45_2071_2100[[vs.MAY]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_MAY <- calc(PC_RAW_rcp85_2011_2040[[vs.MAY]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_MAY <- calc(PC_RAW_rcp85_2041_2070[[vs.MAY]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_MAY <- calc(PC_RAW_rcp85_2071_2100[[vs.MAY]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# JUN
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_JUN <- calc(PC_RAW_rcp26_2011_2040[[vs.JUN]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_JUN <- calc(PC_RAW_rcp26_2041_2070[[vs.JUN]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_JUN <- calc(PC_RAW_rcp26_2071_2100[[vs.JUN]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_JUN <- calc(PC_RAW_rcp45_2011_2040[[vs.JUN]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_JUN <- calc(PC_RAW_rcp45_2041_2070[[vs.JUN]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_JUN <- calc(PC_RAW_rcp45_2071_2100[[vs.JUN]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_JUN <- calc(PC_RAW_rcp85_2011_2040[[vs.JUN]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_JUN <- calc(PC_RAW_rcp85_2041_2070[[vs.JUN]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_JUN <- calc(PC_RAW_rcp85_2071_2100[[vs.JUN]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# JUL
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_JUL <- calc(PC_RAW_rcp26_2011_2040[[vs.JUL]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_JUL <- calc(PC_RAW_rcp26_2041_2070[[vs.JUL]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_JUL <- calc(PC_RAW_rcp26_2071_2100[[vs.JUL]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_JUL <- calc(PC_RAW_rcp45_2011_2040[[vs.JUL]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_JUL <- calc(PC_RAW_rcp45_2041_2070[[vs.JUL]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_JUL <- calc(PC_RAW_rcp45_2071_2100[[vs.JUL]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_JUL <- calc(PC_RAW_rcp85_2011_2040[[vs.JUL]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_JUL <- calc(PC_RAW_rcp85_2041_2070[[vs.JUL]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_JUL <- calc(PC_RAW_rcp85_2071_2100[[vs.JUL]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# AUG
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_AUG <- calc(PC_RAW_rcp26_2011_2040[[vs.AUG]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_AUG <- calc(PC_RAW_rcp26_2041_2070[[vs.AUG]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_AUG <- calc(PC_RAW_rcp26_2071_2100[[vs.AUG]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_AUG <- calc(PC_RAW_rcp45_2011_2040[[vs.AUG]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_AUG <- calc(PC_RAW_rcp45_2041_2070[[vs.AUG]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_AUG <- calc(PC_RAW_rcp45_2071_2100[[vs.AUG]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_AUG <- calc(PC_RAW_rcp85_2011_2040[[vs.AUG]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_AUG <- calc(PC_RAW_rcp85_2041_2070[[vs.AUG]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_AUG <- calc(PC_RAW_rcp85_2071_2100[[vs.AUG]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# SEP
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_SEP <- calc(PC_RAW_rcp26_2011_2040[[vs.SEP]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_SEP <- calc(PC_RAW_rcp26_2041_2070[[vs.SEP]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_SEP <- calc(PC_RAW_rcp26_2071_2100[[vs.SEP]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_SEP <- calc(PC_RAW_rcp45_2011_2040[[vs.SEP]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_SEP <- calc(PC_RAW_rcp45_2041_2070[[vs.SEP]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_SEP <- calc(PC_RAW_rcp45_2071_2100[[vs.SEP]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_SEP <- calc(PC_RAW_rcp85_2011_2040[[vs.SEP]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_SEP <- calc(PC_RAW_rcp85_2041_2070[[vs.SEP]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_SEP <- calc(PC_RAW_rcp85_2071_2100[[vs.SEP]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# OCT
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_OCT <- calc(PC_RAW_rcp26_2011_2040[[vs.OCT]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_OCT <- calc(PC_RAW_rcp26_2041_2070[[vs.OCT]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_OCT <- calc(PC_RAW_rcp26_2071_2100[[vs.OCT]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_OCT <- calc(PC_RAW_rcp45_2011_2040[[vs.OCT]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_OCT <- calc(PC_RAW_rcp45_2041_2070[[vs.OCT]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_OCT <- calc(PC_RAW_rcp45_2071_2100[[vs.OCT]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_OCT <- calc(PC_RAW_rcp85_2011_2040[[vs.OCT]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_OCT <- calc(PC_RAW_rcp85_2041_2070[[vs.OCT]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_OCT <- calc(PC_RAW_rcp85_2071_2100[[vs.OCT]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# NOV
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_NOV <- calc(PC_RAW_rcp26_2011_2040[[vs.NOV]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_NOV <- calc(PC_RAW_rcp26_2041_2070[[vs.NOV]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_NOV <- calc(PC_RAW_rcp26_2071_2100[[vs.NOV]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_NOV <- calc(PC_RAW_rcp45_2011_2040[[vs.NOV]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_NOV <- calc(PC_RAW_rcp45_2041_2070[[vs.NOV]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_NOV <- calc(PC_RAW_rcp45_2071_2100[[vs.NOV]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_NOV <- calc(PC_RAW_rcp85_2011_2040[[vs.NOV]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_NOV <- calc(PC_RAW_rcp85_2041_2070[[vs.NOV]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_NOV <- calc(PC_RAW_rcp85_2071_2100[[vs.NOV]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# DEC
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_DEC <- calc(PC_RAW_rcp26_2011_2040[[vs.DEC]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_DEC <- calc(PC_RAW_rcp26_2041_2070[[vs.DEC]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_DEC <- calc(PC_RAW_rcp26_2071_2100[[vs.DEC]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_DEC <- calc(PC_RAW_rcp45_2011_2040[[vs.DEC]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_DEC <- calc(PC_RAW_rcp45_2041_2070[[vs.DEC]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_DEC <- calc(PC_RAW_rcp45_2071_2100[[vs.DEC]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_DEC <- calc(PC_RAW_rcp85_2011_2040[[vs.DEC]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_DEC <- calc(PC_RAW_rcp85_2041_2070[[vs.DEC]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_DEC <- calc(PC_RAW_rcp85_2071_2100[[vs.DEC]], mean, rm.na = TRUE)

# Seasonal......

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# DJF
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_DJF <- calc(PC_RAW_rcp26_2011_2040[[c(vs.DEC, vs.JAN, vs.FEB)]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_DJF <- calc(PC_RAW_rcp26_2041_2070[[c(vs.DEC, vs.JAN, vs.FEB)]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_DJF <- calc(PC_RAW_rcp26_2071_2100[[c(vs.DEC, vs.JAN, vs.FEB)]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_DJF <- calc(PC_RAW_rcp45_2011_2040[[c(vs.DEC, vs.JAN, vs.FEB)]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_DJF <- calc(PC_RAW_rcp45_2041_2070[[c(vs.DEC, vs.JAN, vs.FEB)]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_DJF <- calc(PC_RAW_rcp45_2071_2100[[c(vs.DEC, vs.JAN, vs.FEB)]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_DJF <- calc(PC_RAW_rcp85_2011_2040[[c(vs.DEC, vs.JAN, vs.FEB)]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_DJF <- calc(PC_RAW_rcp85_2041_2070[[c(vs.DEC, vs.JAN, vs.FEB)]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_DJF <- calc(PC_RAW_rcp85_2071_2100[[c(vs.DEC, vs.JAN, vs.FEB)]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# MAM
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_MAM <- calc(PC_RAW_rcp26_2011_2040[[c(vs.MAR, vs.APR, vs.MAY)]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_MAM <- calc(PC_RAW_rcp26_2041_2070[[c(vs.MAR, vs.APR, vs.MAY)]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_MAM <- calc(PC_RAW_rcp26_2071_2100[[c(vs.MAR, vs.APR, vs.MAY)]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_MAM <- calc(PC_RAW_rcp45_2011_2040[[c(vs.MAR, vs.APR, vs.MAY)]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_MAM <- calc(PC_RAW_rcp45_2041_2070[[c(vs.MAR, vs.APR, vs.MAY)]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_MAM <- calc(PC_RAW_rcp45_2071_2100[[c(vs.MAR, vs.APR, vs.MAY)]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_MAM <- calc(PC_RAW_rcp85_2011_2040[[c(vs.MAR, vs.APR, vs.MAY)]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_MAM <- calc(PC_RAW_rcp85_2041_2070[[c(vs.MAR, vs.APR, vs.MAY)]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_MAM <- calc(PC_RAW_rcp85_2071_2100[[c(vs.MAR, vs.APR, vs.MAY)]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# JJA
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_JJA <- calc(PC_RAW_rcp26_2011_2040[[c(vs.JUN, vs.JUL, vs.AUG)]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_JJA <- calc(PC_RAW_rcp26_2041_2070[[c(vs.JUN, vs.JUL, vs.AUG)]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_JJA <- calc(PC_RAW_rcp26_2071_2100[[c(vs.JUN, vs.JUL, vs.AUG)]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_JJA <- calc(PC_RAW_rcp45_2011_2040[[c(vs.JUN, vs.JUL, vs.AUG)]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_JJA <- calc(PC_RAW_rcp45_2041_2070[[c(vs.JUN, vs.JUL, vs.AUG)]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_JJA <- calc(PC_RAW_rcp45_2071_2100[[c(vs.JUN, vs.JUL, vs.AUG)]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_JJA <- calc(PC_RAW_rcp85_2011_2040[[c(vs.JUN, vs.JUL, vs.AUG)]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_JJA <- calc(PC_RAW_rcp85_2041_2070[[c(vs.JUN, vs.JUL, vs.AUG)]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_JJA <- calc(PC_RAW_rcp85_2071_2100[[c(vs.JUN, vs.JUL, vs.AUG)]], mean, rm.na = TRUE)

#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
# SON
#--------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
PC_RAW_rcp26_2011_2040_SON <- calc(PC_RAW_rcp26_2011_2040[[c(vs.SEP, vs.OCT, vs.NOV)]], mean, rm.na = TRUE)
PC_RAW_rcp26_2041_2070_SON <- calc(PC_RAW_rcp26_2041_2070[[c(vs.SEP, vs.OCT, vs.NOV)]], mean, rm.na = TRUE)
PC_RAW_rcp26_2071_2100_SON <- calc(PC_RAW_rcp26_2071_2100[[c(vs.SEP, vs.OCT, vs.NOV)]], mean, rm.na = TRUE)

PC_RAW_rcp45_2011_2040_SON <- calc(PC_RAW_rcp45_2011_2040[[c(vs.SEP, vs.OCT, vs.NOV)]], mean, rm.na = TRUE)
PC_RAW_rcp45_2041_2070_SON <- calc(PC_RAW_rcp45_2041_2070[[c(vs.SEP, vs.OCT, vs.NOV)]], mean, rm.na = TRUE)
PC_RAW_rcp45_2071_2100_SON <- calc(PC_RAW_rcp45_2071_2100[[c(vs.SEP, vs.OCT, vs.NOV)]], mean, rm.na = TRUE)

PC_RAW_rcp85_2011_2040_SON <- calc(PC_RAW_rcp85_2011_2040[[c(vs.SEP, vs.OCT, vs.NOV)]], mean, rm.na = TRUE)
PC_RAW_rcp85_2041_2070_SON <- calc(PC_RAW_rcp85_2041_2070[[c(vs.SEP, vs.OCT, vs.NOV)]], mean, rm.na = TRUE)
PC_RAW_rcp85_2071_2100_SON <- calc(PC_RAW_rcp85_2071_2100[[c(vs.SEP, vs.OCT, vs.NOV)]], mean, rm.na = TRUE)


# /////////////////////////////////////////////////////////////////////////////////////
# /////////////////////////////////////////////////////////////////////////////////////
# BLOCK: Post-Processing Lattice Graphics
# Generates: Figure 12. RAW (Uncurrected)
# /////////////////////////////////////////////////////////////////////////////////////
# /////////////////////////////////////////////////////////////////////////////////////

# CR climati Region ShapeFile is loaded
shp <- shapefile("zonas_climaticas_def_crtm05")

# CRTM05 projection is assigned to rasterbrick object
crs(shp) <- CRTM05

# A color palette is created based on http://colorbrewer2.org/
#-20, 202,0,32, 255,-20 #CA0020
#-15.6, 221,73,75, 255,-15.6 #DD494B
#-11.1, 240,147,119, 255,-11.1 #F09377
#-6.67, 245,193,169, 255,-6.67 #F5C1A9
#-2.22, 247,229,221, 255,-2.22 #F7E5DD
#2.22, 225,236,242,255,2.22 #E1ECF2
#6.67, 180,214,231, 255,6.67 #B4D6E7
#11.1, 130,188,217, 255,11.1 #82BCD9
#15.6, 67,150,197, 255,15.6 #4396C5
#20, 5,113,176, 255,20 #0571B0

# A color palette is created based on http://colorbrewer2.org/
colr.rain3 <- colorRampPalette(c("#CA0020", # -10
                                 "#DD494B", # -8
                                 "#F09377", # -5
                                 "#F5C1A9", # -2
                                 "#F7E5DD", # -1 boundary
                                 "#E1ECF2", # 5
                                 "#B4D6E7", # 10
                                 "#82BCD9", # 15
                                 "#4396C5", # 20
                                 "#0571B0")) # 30

# Color scale breaks are defined
brk.near <- c(-100, -60, -45, -30, -15, 0, 25, 50, 100, 150, 220)
brk.mid <- c(-100, -60, -45, -30, -15, 0, 25, 50, 100, 150, 220)
brk.far <- c(-100, -60, -45, -30, -15, 0, 25, 50, 100, 150, 220)

# Color scale breaks are defined
brk2 <- do.breaks(c(-5, 5), 20)

#----------------------------------------------------------------------------------------------------------------
# Near-Future 2011_2040
#----------------------------------------------------------------------------------------------------------------

# A raster stack object is created
near.future.raster <- stack(PC_RAW_rcp26_2011_2040_DJF,
                            PC_RAW_rcp45_2011_2040_DJF,
                            PC_RAW_rcp85_2011_2040_DJF,
                            PC_RAW_rcp26_2011_2040_MAM,
                            PC_RAW_rcp45_2011_2040_MAM,
                            PC_RAW_rcp85_2011_2040_MAM,
                            PC_RAW_rcp26_2011_2040_JJA,
                            PC_RAW_rcp45_2011_2040_JJA,
                            PC_RAW_rcp85_2011_2040_JJA,
                            PC_RAW_rcp26_2011_2040_SON,
                            PC_RAW_rcp45_2011_2040_SON,
                            PC_RAW_rcp85_2011_2040_SON)

# A simple plot is created on raster level [1]
#plot(near.future.raster[[1]])

# Rasterbrick object names are replaced
names(near.future.raster) <- c("RCP26_DJF",
                               "RCP45_DJF",
                               "RCP85_DJF",
                               "RCP26_MAM",
                               "RCP45_MAM",
                               "RCP85_MAM",
                               "RCP26_JJA",
                               "RCP45_JJA",
                               "RCP85_JJA",
                               "RCP26_SON",
                               "RCP45_SON",
                               "RCP85_SON")

# A "levels" monthly vector is created for visualization puposes
v.select.name.near <- c("RCP26_DJF",
                        "RCP45_DJF",
                        "RCP85_DJF",
                        "RCP26_MAM",
                        "RCP45_MAM",
                        "RCP85_MAM",
                        "RCP26_JJA",
                        "RCP45_JJA",
                        "RCP85_JJA",
                        "RCP26_SON",
                        "RCP45_SON",
                        "RCP85_SON")

# Rasterbrick layer names are replaced
names(near.future.raster) <- v.select.name.near

# Mean precipitation plot is created
levelplot(near.future.raster, layout=c(3, 4),
          col.regions = colr.rain3, at = brk.near,
          colorkey = list(col = colr.rain3, at = brk.near)) +
  latticeExtra::layer(sp.polygons(shp, lwd = 2))

#----------------------------------------------------------------------------------------------------------------
# Mid-Future 2041_2070
#----------------------------------------------------------------------------------------------------------------

# A raster stack object is created
mid.future.raster <- stack(PC_RAW_rcp26_2041_2070_DJF,
                           PC_RAW_rcp45_2041_2070_DJF,
                           PC_RAW_rcp85_2041_2070_DJF,
                           PC_RAW_rcp26_2041_2070_MAM,
                           PC_RAW_rcp45_2041_2070_MAM,
                           PC_RAW_rcp85_2041_2070_MAM,
                           PC_RAW_rcp26_2041_2070_JJA,
                           PC_RAW_rcp45_2041_2070_JJA,
                           PC_RAW_rcp85_2041_2070_JJA,
                           PC_RAW_rcp26_2041_2070_SON,
                           PC_RAW_rcp45_2041_2070_SON,
                           PC_RAW_rcp85_2041_2070_SON)

# A simple plot is created on raster level [1]
#plot(mid.future.raster[[1]])

# Rasterbrick object names are replaced
names(mid.future.raster) <- c("RCP26_DJF",
                              "RCP45_DJF",
                              "RCP85_DJF",
                              "RCP26_MAM",
                              "RCP45_MAM",
                              "RCP85_MAM",
                              "RCP26_JJA",
                              "RCP45_JJA",
                              "RCP85_JJA",
                              "RCP26_SON",
                              "RCP45_SON",
                              "RCP85_SON")

# A "levels" monthly vector is created for visualization puposes
v.select.name.mid <- c("RCP26_DJF",
                       "RCP45_DJF",
                       "RCP85_DJF",
                       "RCP26_MAM",
                       "RCP45_MAM",
                       "RCP85_MAM",
                       "RCP26_JJA",
                       "RCP45_JJA",
                       "RCP85_JJA",
                       "RCP26_SON",
                       "RCP45_SON",
                       "RCP85_SON")

# Rasterbrick layer names are replaced
names(mid.future.raster) <- v.select.name.mid

# Mean precipitation plot is created
levelplot(mid.future.raster, layout=c(3, 4),
          col.regions = colr.rain3, at = brk.mid,
          colorkey = list(col = colr.rain3, at = brk.mid)) +
  latticeExtra::layer(sp.polygons(shp, lwd = 2))

#----------------------------------------------------------------------------------------------------------------
# Far-Future 2071_2100
#----------------------------------------------------------------------------------------------------------------

# A raster stack object is created
far.future.raster <- stack(PC_RAW_rcp26_2071_2100_DJF,
                           PC_RAW_rcp45_2071_2100_DJF,
                           PC_RAW_rcp85_2071_2100_DJF,
                           PC_RAW_rcp26_2071_2100_MAM,
                           PC_RAW_rcp45_2071_2100_MAM,
                           PC_RAW_rcp85_2071_2100_MAM,
                           PC_RAW_rcp26_2071_2100_JJA,
                           PC_RAW_rcp45_2071_2100_JJA,
                           PC_RAW_rcp85_2071_2100_JJA,
                           PC_RAW_rcp26_2071_2100_SON,
                           PC_RAW_rcp45_2071_2100_SON,
                           PC_RAW_rcp85_2071_2100_SON)

# A simple plot is created on raster level [1]
#plot(far.future.raster[[1]])

# Rasterbrick object names are replaced
names(far.future.raster) <- c("RCP26_DJF",
                              "RCP45_DJF",
                              "RCP85_DJF",
                              "RCP26_MAM",
                              "RCP45_MAM",
                              "RCP85_MAM",
                              "RCP26_JJA",
                              "RCP45_JJA",
                              "RCP85_JJA",
                              "RCP26_SON",
                              "RCP45_SON",
                              "RCP85_SON")

# A "levels" monthly vector is created for visualization puposes
v.select.name.far <- c("RCP26_DJF",
                       "RCP45_DJF",
                       "RCP85_DJF",
                       "RCP26_MAM",
                       "RCP45_MAM",
                       "RCP85_MAM",
                       "RCP26_JJA",
                       "RCP45_JJA",
                       "RCP85_JJA",
                       "RCP26_SON",
                       "RCP45_SON",
                       "RCP85_SON")

# Rasterbrick layer names are replaced
names(far.future.raster) <- v.select.name.far

# Mean precipitation plot is created
levelplot(far.future.raster, layout=c(3, 4),
          col.regions = colr.rain3, at = brk.far,
          colorkey = list(col = colr.rain3, at = brk.far)) +
  latticeExtra::layer(sp.polygons(shp, lwd = 2))

# Binary R object is saved
save.image("C:/DATOS/R_ITC/XLSX/FUTURE_ENSEMBLE/Base_Spatial_Mean_EQM.RData")

#--------------------------------------------------------------------------------------------------------------------------------------
# END of code
#--------------------------------------------------------------------------------------------------------------------------------------
